Package: nph
Title: Planning and Analysing Survival Studies under Non-Proportional
        Hazards
Version: 2.1
Authors@R: c(
  person("Robin", "Ristl", email = "robin.ristl@meduniwien.ac.at", role = c("aut", "cre")),
  person("Nicolas", "Ballarini", email = "nicolas.ballarini@meduniwien.ac.at", role = c("ctb")))
Description: Piecewise constant hazard functions are used to flexibly model survival distributions with non-proportional hazards and 
	to simulate data from the specified distributions. A function to calculate weighted log-rank tests for the comparison of two
	hazard functions is included. Also, a function to calculate a test using the maximum of a set of test statistics from weighted
	log-rank tests (MaxCombo test) is provided. This test utilizes the asymptotic multivariate normal joint distribution of the
	separate test statistics. The correlation is estimated from the data. These methods are described in Ristl et al. (2021) 
	<doi:10.1002/pst.2062>.
	Finally, a function is provided for the estimation and inferential statistics of various parameters that quantify the difference
	between two survival curves. Eligible parameters are differences in survival probabilities, log survival probabilities,
	complementary log log (cloglog) transformed survival probabilities, quantiles of the survival functions, log transformed quantiles,
	restricted mean survival times, as well as an average hazard ratio, the Cox model score statistic (logrank statistic), and the
	Cox-model hazard ratio. Adjustments for multiple testing and simultaneous confidence intervals are calculated using a multivariate
	normal approximation to the set of selected parameters.
Date: 2022-05-16
Maintainer: Robin Ristl <robin.ristl@meduniwien.ac.at>
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.1.2
NeedsCompilation: no
Depends: R (>= 3.5.0)
Imports: stats, graphics, mvtnorm, ggplot2, muhaz, survival, multcomp
Suggests: knitr, shiny, shinycssloaders, formatR, styler, rmarkdown
VignetteBuilder: knitr
Packaged: 2022-05-16 19:57:14 UTC; Robin
Author: Robin Ristl [aut, cre],
  Nicolas Ballarini [ctb]
Repository: CRAN
Date/Publication: 2022-05-16 23:20:05 UTC
Built: R 4.6.0; ; 2026-04-26 07:05:29 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,mixpch)
S3method(plot,nphparams)
S3method(print,mixpch)
S3method(print,nphparams)
S3method(vcov,nphparams)
S3method(vcov,myobj)
S3method(coef,myobj)
S3method(print,wlogrank)
S3method(print,wlogrank_max)
export(logrank.maxtest)
export(logrank.test)
export(m2r)
export(nph_gui)
export(nphparams)
export(pchaz)
export(plot_diagram)
export(plot_shhr)
export(plot_subgroups)
export(pop_pchaz)
export(rSurv_conditional_fun)
export(rSurv_fun)
export(sample_conditional_fun)
export(sample_fun)
export(subpop_pchaz)
import(ggplot2)
import(graphics)
import(stats)
importFrom(mvtnorm,pmvnorm)
importFrom(stats,integrate)
importFrom(utils,head)
importFrom(survival,Surv)
importFrom(survival,survfit)
importFrom(survival,coxph)
importFrom(muhaz,muhaz)
importFrom(multcomp,glht)

nph - PH Client and Library
===

nph is a modern client and API library for accessing PH servers.


Installation
------------

To build nph, you should be able to simply run these commands:

   ./configure
   make
   make install


Encap Package Support
---------------------

To build this software as an Encap package, you can pass the
--enable-encap option to configure.  This will be automatically
enabled if the epkg or mkencap programs are detected on the system,
but can be overridden by the --disable-encap option.

When building an Encap package, the configure script will automatically
adjust the installation prefix to use an appropriate Encap package
directory.  It does this using a heuristic algorithm which examines the
values of the ${ENCAP_SOURCE} and ${ENCAP_TARGET} environment variables
and the argument to configure's --prefix option.

If mkencap was detected on the system, it will be automatically run during
"make install".  By default, epkg will also be run, but this can be
inhibited with the --disable-epkg-install configure option.

For information on the Encap package management system, see the WSG
Encap Archive:

   http://www.encap.org/


Readline Support
----------------

The configure script will attempt to find the readline library on
your system.  The readline package is available from:

   ftp://ftp.gnu.org/gnu/readline/

If you do not have readline-4.2 or newer, you will need to specify the
--disable-readline option to configure.


Installing nph Setuid
---------------------

If you wish to use PH's email authentication mechanism, you must install
nph to be setuid-root using the --enable-setuid-nph option to configure.

SECURITY WARNING:
   Installing nph as a setuid-root binary may constitute a serious
   security hole!  While I do not know of any security holes in nph,
   I make no guaruntees about your system security.  I strongly
   recommend against using --enable-setuid-nph unless it is
   absolutely unavoidable.


Default "mailmatch" Fields
--------------------------

For PH servers that don't have a "mailmatches" entry in their siteinfo
list, libphclient will use a built-in default for the list of fields to
check when resolving email addresses.  This default list can be set by
specifying the --with-mailmatches=LIST option to configure.  (Replace
"LIST" with a ':'-delimited list of field names.)

If this option is not specified, the default list is "alias:callsign".


More Information
----------------

After you install nph, you will need to tell it what server to connect to.
This can be set in the global ${sysconfdir}/ph_server file or by setting
the environment variable ${PH_SERVER}.  Alternately, you can invoke nph
with "-s servername" to specify the server name on the commandline.

For documentation on nph or on the libphclient API, see the enclosed
manpages.  For more information on the nph package, see:

   http://www.feep.net/nph/

Both source code and Encap binary distributions of the latest version
of nph are available there.


Supported Platforms
-------------------

I develop and test nph on the following platforms:

   AIX 4.3.3
   AIX 5.1
   HP-UX 11.00
   IRIX 6.5
   RedHat Linux 7.2
   Solaris 8
   Solaris 9

The nph code should build properly on the following platforms, but I
do not actively support them:

   AIX 3.2.5
   AIX 4.2.1
   FreeBSD
   HP-UX 10.20
   Linux/libc5
   RedHat Linux 8.0
   Mac OS X
   OpenBSD
   Solaris 2.5
   Solaris 2.6
   Solaris 7

If you successfully build nph on another platform, please email me a
patch and/or configuration information.


Compatibility Code
------------------

nph depends on some library calls which are not available or not
usable on some platforms.  To accomodate these systems, I've included
a version of these calls in the lib/compat subdirectory.

I've slightly modified these functions for integration into this source
tree, but the functionality has not been modified from the original
source.  Please note that while this code should work for you, I didn't
write it, so please don't send me bug reports on it.


Author
------

Feedback and bug reports are welcome.

Mark D. Roth <roth@feep.net>

##########################
# Home tab server

# change to results tab when clicking on link
observeEvent(input$toDataset,
             updateTabsetPanel(session, "mainNav", "datasetTab")
)
observeEvent(input$toSimulate,
             updateTabsetPanel(session, "mainNav", "simulateTab")
)
observeEvent(input$toExamples,
             updateTabsetPanel(session, "mainNav", "examplesTab")
)

##########################
# Examples tab server

# Examples -------------------------------------------------------------------
## Drop-down selection box for which data set --------------------------------
data_nocomp = reactive({
  set.seed(64566)
  data_example = data.frame(
    Group = c(rep("Test", 30), rep("Reference", 30)),
    value = c(rnorm(n = 30, mean = 0, sd = 1),
              rnorm(n = 30, mean = 0, sd = 1)))
  data_example
})

data_comp = reactive({
  set.seed(64566)
  data_example = data.frame(
    Group = c(rep("Test", 30), rep("Reference", 30)),
    value = c(rnorm(n = 30, mean = 0, sd = 4),
              rnorm(n = 30, mean = 0, sd = 1)))
  data_example
})

## Do the test -----------------------------------------------
e_test_result <- eventReactive(input$e_submit, {
  if (input$e_example == "Example of comparable products") {
    dat = data_nocomp()
  } else {
    dat = data_comp()
  }
  reference = dat[which(dat$Group == "Reference"), "value"]
  test      = dat[which(dat$Group == "Test"), "value"]
  result = testis.tails(Test = test, Reference = reference,
                        q = .1, alpha = .05)
})

output$e_test_result <- renderPrint({
  print(e_test_result())
})

output$e_test_plot <- renderPlot({
  plot(e_test_result())
})

# Load packages ----------------------------------------------------------------
library(markdown)
library(shiny)
# library(shinyBS)
library(shinycssloaders)
library(xlsx)
# Load data --------------------------------------------------------------------

# Define UI for application ----------------------------------------------------
ui <- shinyUI(
  bootstrapPage(title = '', theme = "merck.css",
                navbarPage(
                  title = tags$b("nph"),
                  windowTitle = "nph",
                  id = "mainNav",
                  inverse = TRUE,
                  fluid = FALSE,
                  collapsible = FALSE,
                  # theme = "bootstrap.min.css",
                  source(file.path("ui", "ui-tab-home.R"),     local = TRUE)$value,
                  source(file.path("ui", "ui-tab-simulate.R"), local = TRUE)$value,
                  source(file.path("ui", "ui-tab-power.R"), local = TRUE)$value,
                  source(file.path("ui", "ui-tab-condpower.R"), local = TRUE)$value,
                  source(file.path("ui", "ui-tab-about.R"),    local = TRUE)$value
                  )
  )
)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----eval=FALSE, fig.show='hold'----------------------------------------------
#  install.packages("nph")
#  
#  # For dev version
#  # install.packages("devtools")
#  devtools::install_github("repo/nph")

## ----echo=FALSE, fig.height=3.5, fig.show='hold', fig.width=10, warning=FALSE, out.width='100%'----
library(nph)
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- 1              # There are no subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(18, nrow = 1)),
  lambdaMat2    = m2r(matrix(11, nrow = 1)),
  lambdaProgMat = m2r(matrix( 9, nrow = 1)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1              # There are no subgroups
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(11, nrow = 1)),
  lambdaMat2    = m2r(matrix( 9, nrow = 1)),
  lambdaProgMat = m2r(matrix( 5, nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)
plot_shhr(A = K5, B = B5, main = "Different disease progression by treatment")

## ----echo=FALSE, fig.height=3.5, fig.show='hold', fig.width=10, warning=FALSE, out.width='100%'----
times <- c(0, 100, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- 1               # There are no subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(11, 18), nrow = 1)),
  lambdaMat2    = m2r(matrix(c( 9, 11), nrow = 1)),
  lambdaProgMat = m2r(matrix(c( 5,  9), nrow = 1)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1 
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(11, 11), nrow = 1)),
  lambdaMat2    = m2r(matrix(c( 9,  9), nrow = 1)),
  lambdaProgMat = m2r(matrix(c( 5,  5), nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

plot_shhr(K5, B5, main = "Different effect by time intervals")

## ----echo=FALSE, fig.height=3.5, fig.show='hold', fig.width=10, warning=FALSE, out.width='100%'----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <-  c(.2, .8)
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(30,
                               18),nrow = 2)),
  lambdaMat2    = m2r(matrix(c(20,
                               11),nrow = 2)),
  lambdaProgMat = m2r(matrix(c(15, 
                               9), nrow = 2)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1 
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(11,nrow = 1)),
  lambdaMat2    = m2r(matrix(9, nrow = 1)),
  lambdaProgMat = m2r(matrix(5, nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

plot_shhr(K5, B5, main = "Presence of subgroups with differential treatment effect")

## ----echo=FALSE, fig.height=4, fig.show='hold', fig.width=7, message=FALSE, warning=FALSE, out.width='75%', paged.print=TRUE----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- 1              # There are no subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(18, nrow = 1)),
  lambdaMat2    = m2r(matrix(11, nrow = 1)),
  lambdaProgMat = m2r(matrix( 9, nrow = 1)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1              # There are no subgroups
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(11, nrow = 1)),
  lambdaMat2    = m2r(matrix( 9, nrow = 1)),
  lambdaProgMat = m2r(matrix( 5, nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)
plot_diagram(K5, B5, which = "Control")

## ----echo=FALSE, fig.align='center', fig.show='hold', fig.width=12.5, warning=FALSE, out.width='50%'----
knitr::include_graphics("lambdamat.png")

## ---- echo=TRUE, fig.show='hold', fig.width=12.5, warning=FALSE, out.width='100%'----
times <- c(0, 100, 5 * 365)   # Time interval boundaries, in days
t_resp <- c(0.2, 0.8) #Proportion of subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(11, 30,
                               11, 18), byrow = TRUE, nrow = 2)),
  lambdaMat2    = m2r(matrix(c( 9, 20,
                                9, 11), byrow = TRUE, nrow = 2)),
  lambdaProgMat = m2r(matrix(c( 5, 15,
                                5,  9), byrow = TRUE, nrow = 2)),
  p = t_resp, discrete_approximation = TRUE
)

## ---- echo=TRUE, fig.show='hold', fig.width=6, warning=FALSE, out.width='33%'----
plot(B5, main = "Survival function")
plot(B5, fun = "haz", main = "Hazard function")
plot(B5, fun = "cumhaz", main = "Cumulative Hazard function")

## -----------------------------------------------------------------------------
times <- c(0, 100, 5 * 365)   # Time interval boundaries, in days
# Treatment group
B5 <- pop_pchaz(T = times,
                   lambdaMat1    = m2r(matrix(c(11, 30,
                                                11, 18), byrow = TRUE, nrow = 2)),
                   lambdaMat2    = m2r(matrix(c( 9, 20,
                                                 9, 11), byrow = TRUE, nrow = 2)),
                   lambdaProgMat = m2r(matrix(c( 5, 15,
                                                 5,  9), byrow = TRUE, nrow = 2)),
                   p = c(0.2, 0.8),#Proportion of subgroups
                discrete_approximation = TRUE 
)
# Control group
K5  <- pop_pchaz(T = times,
                    lambdaMat1    = m2r(matrix(c(11, 11), nrow = 1)),
                    lambdaMat2    = m2r(matrix(c( 9,  9), nrow = 1)),
                    lambdaProgMat = m2r(matrix(c( 5,  5), nrow = 1)),
                    p = 1, discrete_approximation = TRUE
)

## -----------------------------------------------------------------------------
# Study set up and Simulation of a data set until interim analysis at 150 events
set.seed(15657)
dat <- sample_fun(K5, B5,
                  r0 = 0.5,                     # Allocation ratio
                  eventEnd = 450,               # maximal number of events
                  lambdaRecr = 300 / 365,       # recruitment rate per day (Poisson assumption)
                  lambdaCens = 0.013 / 365,     # censoring rate per day  (Exponential assumption)
                  maxRecrCalendarTime = 3 * 365,# Maximal duration of recruitment
                  maxCalendar = 4 * 365.25)     # Maximal study duration
head(dat)
tail(dat)

## -----------------------------------------------------------------------------
logrank.test(time  = dat$y,
             event = dat$event,
             group = dat$group,
             # alternative = "greater",
             rho   = 1,
             gamma = 0) 
# survival::survdiff(formula = survival::Surv(time  = dat$y, event = dat$event) ~ dat$group)

## -----------------------------------------------------------------------------
lrmt = logrank.maxtest(
      time  = dat$y,
      event = dat$event,
      group = dat$group,
      rho   = c(0, 0, 1, 1),
      gamma = c(0, 1, 0, 1)
)
lrmt

## -----------------------------------------------------------------------------
lrmt$logrank.test[[1]]
lrmt$logrank.test[[2]]
lrmt$logrank.test[[3]]
lrmt$logrank.test[[4]]

##########################
# Dataset tab server


# downloadHandler() takes two arguments, both functions.
# The content function is passed a filename as an argument, and
#   it should write out data to that filename.
output$downloadData <- downloadHandler(

  # This function returns a string which tells the client
  # browser what name to use when saving the file.
  filename = function() {
    "tailTest_dataexample.csv"
  },

  # This function should write data to a file given to it by
  # the argument 'file'.
  content = function(file) {
    set.seed(98123)
    data_example = data.frame(
      Group = c(rep("Test", 30), rep("Reference", 30)),
      value = round(c(rnorm(n = 30, mean = 0, sd = .7),
                rnorm(n = 30, mean = 0, sd = 1)), 2))
    # Write to a file specified by the 'file' argument
    write.csv(data_example, file,
              row.names = FALSE)
  }
)


tailTest.env$outputData<-NULL

Dataset <- shiny::reactive({
  if(is.null(input$outputfile)){
    return(data.frame())
  }
  tailTest.env$outputData <- data.frame(do.call("read.csv",
                                               list(input$outputfile$datapath)))
  return(tailTest.env$outputData)
})

output$head_table <- renderTable(head(Dataset()),
                                 caption = "Preview first 6 rows of the dataset",
                                 caption.placement = getOption("xtable.caption.placement", "top"),
                                 caption.width = getOption("xtable.caption.width", NULL))

output$choose_Vars<-shiny::renderUI({
  if(is.null(input$outputfile))
    return()
  if(identical(Dataset(),'')||identical(Dataset(),data.frame()))
    return(NULL)
  tailTest.env$outputData <- Dataset()
  tailTest.env$NUM     <- dim(tailTest.env$outputData)[2] #Num of all variable  ##
  tailTest.env$Class   <- sapply(apply(tailTest.env$outputData,2,unique),length)  ##
  tailTest.env$Colnames <- colnames(tailTest.env$outputData) ##
  out <- column(6,
                  selectInput("group", label="Grouping variable", c("",tailTest.env$Colnames)),
                  selectInput("value", label="Values",            c("",tailTest.env$Colnames)))
  out
})

group_opts <- shiny::eventReactive(input$group, {
  if(is.null(input$outputfile))
    return()
  if(identical(Dataset(),'')||identical(Dataset(),data.frame()))
    return(NULL)
  if(is.null(input$group)){
    return(NULL)
  }
  dat = Dataset()
  unique(dat[input$group])
})

output$choose_groups <- shiny::renderUI({
  if(is.null(input$outputfile))
    return()
  if(identical(Dataset(),'')||identical(Dataset(),data.frame()))
    return(NULL)
  dat = Dataset()
  tailTest.env$outputData <- dat
  tailTest.env$NUM     <- dim(tailTest.env$outputData)[2] #Num of all variable  ##
  tailTest.env$Class   <- sapply(apply(tailTest.env$outputData,2,unique),length)  ##
  tailTest.env$Colnames <- colnames(tailTest.env$outputData) ##
  if(input$group == ""){
    return(NULL)
  }
  out <- column(6,
                selectInput("reference", label="Reference", c("",group_opts())),
                selectInput("test",      label="Test",      c("",group_opts())))
  out
})




## Do the test -----------------------------------------------

c_value <- reactive({
  if (is.na(input$c)) {
    return(NULL)
  } else{
    return(input$c)
  }
})

test_result <- eventReactive(input$submit, {
  dat = Dataset()
  reference = dat[which(dat$Group == input$reference), "value"]
  test = dat[which(dat$Group == input$test), "value"]
  result = testis.tails(Test = test, Reference = reference,
                        q = input$q, alpha = input$alpha, c = c_value())
  result
})

output$test_result <- renderPrint({
  print(test_result())
})

output$test_plot <- renderPlot({
  plot(test_result())
})

##########################
# Simulate tab UI

tabPanel(
  title = "Simulate",
  id    = "simulateTab",
  value = "simulateTab",
  name  = "simulateTab",
  class = "fade in",
  icon  = icon("table"),
  
  h1("Population parameters", style = "color:#f8c954"),
  #shiny::wellPanel(
   # downloadButton('s_downloadCode', "Download R code", class = "start-button"),
 #   h4("Download a reproducible R script for the code to #generate the results of this tab")),
  shiny::wellPanel(
    fluidRow(tags$h3("Time intervals"),
             column(1, "Number of time intervals"),
             column(4, sliderInput("s_T_timepoints", label = "", 
                                   value = 1, min = 1, max = 2, step = 1)),
             column(6, tags$b("Limits:"), tableOutput("s_T_time")),
             column(1, "")
    )),
  wellPanel(
    fluidRow(
      column(6, tags$h3("Treatment Group")),
      column(6, tags$h3("Control Group"))
    ),
    fluidRow(
      column(6, tags$h4("Specify parameters for Treatment Group")),
      column(6, tags$h4("Specify parameters for Control Group"))
    ),
    fluidRow(
      column(6,
             fluidRow(
               fluidRow(column(4,
                               sliderInput("s_T_subgr", label = "Number of subgroups", 
                                           value = 1, min = 1, max = 2, step = 1))),
               tags$b("Subgroup proportions:"), tableOutput("s_T_subgr_p"),
               tags$b("Subgroup labels:"), tableOutput("s_T_subgr_labels")),
             fluidRow(column(6,
                             fluidRow(
                               column(12,
                                      tags$b("lambdaMat1:"), 
                                      tableOutput("s_T_lambdaMat1")
                               )
                             ),
                             fluidRow(
                               column(12,
                                      tags$b("lambdaMat2:"), 
                                      tableOutput("s_T_lambdaMat2")
                               )
                             ),
                             fluidRow(
                               column(12,
                                      tags$b("lambdaProgMat:"),
                                      tableOutput("s_T_lambdaProgMat")
                               )
                             ))),
             fluidRow(column(12, checkboxInput("show_trt_diag", label = "Show diagram for treatment arm", value = FALSE))),
             conditionalPanel(condition = "input.show_trt_diag > 0",
                              fluidRow(column(12, plotOutput("trt_plot"))))),
      column(6,
             fluidRow(
               fluidRow(column(4,
                               sliderInput("s_C_subgr", label = "Number of subgroups", 
                                           step = 1, value = 1, min = 1, max = 2))),
               tags$b("Subgroup proportions:"), tableOutput("s_C_subgr_p"),
               tags$b("Subgroup labels:"), tableOutput("s_C_subgr_labels")),
             fluidRow(column(6,
                             fluidRow(
                               column(12,
                                      tags$b("lambdaMat1:"), 
                                      tableOutput("s_C_lambdaMat1")
                               )),
                             fluidRow(
                               column(12,
                                      tags$b("lambdaMat2:"), 
                                      tableOutput("s_C_lambdaMat2")
                               )),
                             fluidRow(
                               column(12,
                                      tags$b("lambdaProgMat:"),
                                      tableOutput("s_C_lambdaProgMat")
                               )
                               ))),
              fluidRow(column(12, checkboxInput("show_ctrl_diag", label = "Show diagram for control arm", value = FALSE))),
              conditionalPanel(condition = "input.show_ctrl_diag > 0",
                              fluidRow(column(12, plotOutput("ctrl_plot")))))
    )),
    wellPanel(
    fluidRow(
      fluidRow(column(5, ""),
               column(2, actionButton(inputId = "s_calculate", label = "Plot!", icon = icon("line-chart"))),
               column(5, "")),
      conditionalPanel(
        condition = "input.s_calculate > 0",
        withSpinner(fluidRow(column(width = 12,
                                    h3("Results"),
                                    plotOutput("panplot"),
                                    div(style = "min-height: 20px;"),
                                    fluidRow(column(width = 12, tableOutput("median_table"), align="center")))
        ))))
  )
)

#' Transform median time into rate
#' 
#' This helper function calculates the hazard rate per day of an exponential 
#' distribution from the median given in months.
#' 
#' @param x The median time in months to be transformed into rate
#' 
#' @export
m2r <- function(x){
  12 / 365.25 * log(2) / x
}

#' Draw a state space figure
#' 
#' A figure that shows the states and the possible transitions between them.
#' 
#' @param A An object of class \code{mixpch}, resembling the survival function in treatment group 0
#' @param B An object of class \code{mixpch}, resembling the survival function in treatment group 1
#' @param A_subgr_labels A character vector with the same length as A$p. It indicates names for the subgroups in A
#' @param B_subgr_labels A character vector with the same length as B$p. It indicates names for the subgroups in B
#' @param which Which treatment arm should be shown? One of "Both", "Experimental", "Control".
#' @param treatment_labels A character vector of length 2 indicating the treatment labels.
#' @param colors Either a vector of length two with colors for A and B, or "default".
#' @param show.rate A logical indicating whether the rate should be shown in the diagram
#' 
#' @import ggplot2
#' @export
plot_diagram = function(A,
                        B,
                        A_subgr_labels = "",
                        B_subgr_labels = "",
                        which = c("Both", "Experimental", "Control"),
                        treatment_labels = c("Experimental", "Control"),
                        colors = "default", show.rate = TRUE){
  which = match.arg(which)
  
  times = A$Tint
  t_pr = A$p
  t_label = A_subgr_labels
  t_lambdaMat1 = A$lambdaMat1
  t_lambdaMat2 = A$lambdaMat2
  t_lambdaProgMat = A$lambdaProgMat
  c_pr = B$p
  c_label = B_subgr_labels
  c_lambdaMat1 = B$lambdaMat1
  c_lambdaMat2 = B$lambdaMat2
  c_lambdaProgMat = B$lambdaProgMat
  
  if (length(times) > 3) stop("Only two intervals are supported")
  
  t_group = rep(t_label, each = 3)
  c_group = rep(c_label, each = 3)
  t_group_id = rep(1:length(t_pr), each = 3)
  c_group_id = rep(1:length(c_pr), each = 3)
  
  dat = data.frame(treatment = c(rep("Experimental", 3*length(t_pr)),
                                 rep("Control", 3*length(c_pr))),
                   treatment_labels = c(rep(treatment_labels[1], 3*length(t_pr)),
                                        rep(treatment_labels[2], 3*length(c_pr))),
                   group = c(t_group, c_group),
                   group_id = c(t_group_id, c_group_id))
  dat$x = c(-1, 0, 1) * 4
  dat$y = c( 1, 0, 1) * 2
  dat$x_begin = c(-1,        0 + 1/4,  1 - 1/4) * 4
  dat$y_begin = c( 1 - .5/2, 0,        1) * 2
  dat$x_end   = c( 0 -  1/4, 1      , -1 + 1/4) * 4
  dat$y_end   = c( 0,        1 -.5/2,  1) * 2
  dat$x_mid   = dat$x_begin + dat$x_end
  dat$y_mid   = dat$y_begin + dat$y_end
  
  dat$ends = c("last","last","first")
  dat$label = c("Alive\nNo progression","Progression","Death")
  dat$lambdas = c("lambda[P](t)",
                  "lambda[PD](t)",
                  "lambda[D](t)")
  dat$nudge_x = c(-.4,  0, 0)
  dat$nudge_y = c(-.4,-.4,.4)
  
  t_lambdas = data.frame()
  for (i in 1:length(t_pr)){
    t_lambdas = rbind(t_lambdas,
                      t_lambdaProgMat[i,],
                      t_lambdaMat2[i,],
                      t_lambdaMat1[i,])
  }
  c_lambdas = data.frame()
  for (i in 1:length(c_pr)){
    c_lambdas = rbind(c_lambdas,
                      c_lambdaProgMat[i,],
                      c_lambdaMat2[i,],
                      c_lambdaMat1[i,])
  }
  colnames(t_lambdas) = colnames(c_lambdas) = paste0("T", 1:(length(times)-1))
  dat = cbind(dat, rbind(t_lambdas, c_lambdas))
  if(show.rate){
    if (length(times) == 2){
      dat$lambda_label = paste0(dat$lambdas, ": ", round(dat$T1, 4))
    } else if (length(times) == 3){
      dat$lambdast1 = c(sprintf("lambda[P]^paste('[%s-%s]')*(t)",  0, times[2]),
                        sprintf("lambda[PD]^paste('[%s-%s]')*(t)", 0, times[2]),
                        sprintf("lambda[D]^paste('[%s-%s]')*(t)",  0, times[2]))
      dat$lambdast2 = c(sprintf("lambda[P]^paste('[%s-%s]')*(t)",  times[2], times[3]),
                        sprintf("lambda[PD]^paste('[%s-%s]')*(t)", times[2], times[3]),
                        sprintf("lambda[D]^paste('[%s-%s]')*(t)",  times[2], times[3]))
      dat$lambda_label = paste0("atop(",dat$lambdast1, ": ", round(dat$T1, 4), ",",
                                dat$lambdast2, ": ", round(dat$T2, 4), ")")
    } 
  } else{
    dat$lambda_label = dat$lambdas
  }
  
  if (which == "Experimental"){
    dat = dat[which(dat$treatment == "Experimental"), ]
  } else if (which == "Control"){
    dat = dat[which(dat$treatment == "Control"), ]      
  }
  
  if(colors == "default"){
    col_trt = "pink" 
    col_ctrl = "lightblue" 
  }
  ggplot(dat) + 
    facet_grid(group ~ treatment_labels) + 
    geom_rect(aes_string(xmin = "x - 1", 
                  xmax = "x + 1",
                  ymin = "y - 0.5", 
                  ymax = "y + 0.5",
                  fill = "treatment"))  +
    geom_text(aes_string(x = "x", 
                         y = "y", label = "label"))  +
    geom_text(aes_string(x = "(x_begin + x_end)/2 + nudge_x", 
                         y = "(y_begin + y_end)/2 + nudge_y",
                         label = "lambda_label"), 
              parse = TRUE)  +
    geom_segment(aes_string(x    = "x_begin", 
                            xend = "x_end" ,
                            y    = "y_begin" , 
                            yend = "y_end"),
                 arrow = arrow(type = "closed", 
                               length = unit(10, "points"),
                               ends = dat$ends)) +
    coord_equal(ylim = c(-0.5, 3.5), xlim = c(-5, 5)) + 
    theme_void() + 
    theme(legend.position = "none", 
          text = element_text(size = 20)) +
    scale_fill_manual(values = c("Experimental" = col_trt, 
                                 "Control" = col_ctrl)) -> p1
  
  p1
}


#' Plot of survival, hazard and hazard ratio of two groups as a function of time
#' 
#' A convenience function that uses the generic plot function in the nph package to
#' plot the three functions in a layout of 3 columns and 1 row.
#' 
#' @param A An object of class \code{mixpch}, resembling the survival function in treatment group 0
#' @param B An object of class \code{mixpch}, resembling the survival function in treatment group 1
#' @param main An overall title for the plot
#' @param xmax A maximum value for the x-axis. The plot is drawn using xlim = c(0, xmax)
#' @param ymax_haz A maximum value for the y-axis for the hazards plot. The plot is drawn using ylim = c(0, ymax_haz)
#' @param ymax_hr  A maximum value for the y-axis for the hazards ratio plot. The plot is drawn using ylim = c(0, ymax_hr)
#' 
#' @import graphics
#' @export
plot_shhr <- function(A, B, main = "", xmax = NULL, ymax_haz = NULL, ymax_hr = NULL) {
  if (is.null(xmax)) xmax = length(A$t)
  CX <- 2
  CX2 <- 1
  dd <- 4
  # Set graphical parameters. Save previous setup in old_par
  old_par = par(
    lwd = dd,
    font = 2,
    font.axis = 2,
    font.lab = 2,
    cex = CX,
    cex.lab = CX,
    cex.axis = CX,
    cex.main = CX * 1.5,
    mfrow = c(1, 3),
    oma = c(0, 0, 2, 0),
    mar = c(5, 5, 3, 2)
  )
  on.exit(par(old_par))
  
  # Plot the survival functions
  plot(A,
       ylim = c(0, 1),
       xlim = c(0,xmax),
       ylab = "Survival",
       xlab = "Days")
  plot(B, add = TRUE, lty = 2, col = 3)
  
  # Plot the hazard functions
  if (is.null(ymax_haz)) ymax_haz = max(B$haz, A$haz, na.rm = TRUE) + 0.001
  plot(A,
       "haz",
       xlim = c(0, xmax),
       ylim = c(0, ymax_haz),
       ylab = "Hazard",
       xlab = "Days")
  plot(B,
       "haz",
       add = TRUE,
       lty = 2,
       col = 3)
  
  # Plot the hazard ratio
  if (is.null(ymax_hr)) ymax_hr = max(B$haz / A$haz, na.rm = TRUE) + 1
  plot(
    A$t,
    B$haz / A$haz,
    xlim = c(0, xmax),
    ylim = c(0, ymax_hr),
    ylab = "Hazard ratio",
    xlab = "Days",
    type = "l"
  )
  
  # Add title
  mtext(
    outer = TRUE,
    side = 3,
    text = main, cex = 2,
    line = -2
  )

}


# Generic print function for the weighted log-rank tests
#' @export
print.wlogrank = function (x, digits = max(options()$digits - 4, 3), ...) {
  saveopt <- options(digits = digits)
  on.exit(options(saveopt))
  
  otmp <- x$obs
  etmp <- x$exp
  df <- (sum(1 * (etmp > 0))) - 1
  temp <- cbind(x$n, otmp, etmp,
                ((otmp - etmp)^2)/etmp, 
                ((otmp - etmp)^2)/x$var)
  dimnames(temp) <- list(names(x$n), c("N", "Observed", 
                                       "Expected", "(O-E)^2/E", "(O-E)^2/V"))
  # Print everything
  cat("Call:\n")
  dput(x$call)
  cat("\n")
  print(data.frame(temp, check.names = FALSE))
  cat("\n Chisq=", format(round(x$test$Chisq, 1)), " on", df, 
      "degrees of freedom, p=", format.pval(pchisq(x$test$Chisq, 
                                                   df, lower.tail = FALSE)))
  cat("\n rho   = ", x$test$rho,
      "gamma = ", x$test$gamma, "\n")
  invisible(x)
}


# Generic print function for the maximum of weighted log-rank tests
#' @export
print.wlogrank_max = function (x, digits = max(options()$digits - 4, 3), ...) {
  saveopt <- options(digits = digits)
  on.exit(options(saveopt))
  
  # Print everything -
  cat("Call:\n")
  dput(x$call)
  cat("\n Two sided p-value =", format(x$pmult), 
      paste0("(Bonferroni corrected: ", format(x$p.Bonf), ")\n"))
  cat("\n Individual weighted log-rank tests:\n") 
  print(x$tests)
  invisible(x)
}



#' Launch a GUI (shiny app) for the nph package
#'
#' @details The packages \code{shinycssloaders}, \code{formatR} and \code{styler} are required for correct display of the GUI.
#' The package \code{rmarkdown} with access to pandoc is required to save reports.
#' @export
nph_gui <- function(){
  appDir <- system.file("shiny", package = "nph")
  shiny::runApp(appDir, display.mode = "normal");
}

#' Draw a population composition plot
#' 
#' A figure that shows the composition of the population under study though time
#' 
#' @param A An object of class \code{mixpch}, resembling the survival function in treatment group 0
#' @param B An object of class \code{mixpch}, resembling the survival function in treatment group 1
#' @param colors Either a vector of length four with colors for A and B and subgroup 1 and 2, or "default".
#' @param max_time the maximum value for the x-axis.
#' @param position Either "stack" or "fill". By default (stack), the total population decreases through time. If position="fill", the size of the population is rescaled to show conditional percentages.
#' @param title The text for the title.
#' 
#' @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}, Nicolas Ballarini
#' @seealso \code{\link{pop_pchaz}}
#' @references
#' Robin Ristl, Nicolas Ballarini, Heiko Götte, Armin Schüler, Martin Posch, Franz König. Delayed treatment effects, treatment switching and 
#' heterogeneous patient populations: How to design and analyze RCTs in oncology. Pharmaceutical statistics. 2021; 20(1):129-145.
#' @examples
#' A <- pop_pchaz(Tint = c(0, 90, 365),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.2, 0.6, 0.2), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.4, 0.4), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' B <- pop_pchaz(Tint = c(0, 90, 365),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.1, 0.6, 0.1), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.04, 0.04), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' plot_subgroups(A, B, title = "position='stack'")
#' plot_subgroups(A, B, position='fill', title = "position='fill'")
#'
#'
#' @import ggplot2
#' @export
plot_subgroups = function(A,
                          B,
                          colors = "default",
                          max_time = max(A$Tint), 
                          position = c("stack", "fill"),
                          title = ""){

  position = match.arg(position)
  if(colors == "default"){
    cols_new = c("#FAAB18", "#fccc74", "#1380A1", "#71b2c6")
  } else{
    if(length(colors)!=4) stop("colors should be of length 4")
    cols_new = colors
  }
  
  A_n_subgr = length(A$p)
  B_n_subgr = length(B$p)
  
  t = A$t
  if(any(A_n_subgr!=2, B_n_subgr!=2)) stop("This function is only implemented for two subgroups in each treatment")
  
  nsubpop = 4
  lambdaProgMat = rbind(A$lambdaProgMat, B$lambdaProgMat)
  lambdaMat1 = rbind(A$lambdaMat1, B$lambdaMat1)
  lambdaMat2 = rbind(A$lambdaMat2, B$lambdaMat2)
  timezero = c(A$timezero, B$timezero)
  p = c(A$p, B$p)/2
  Tint = A$Tint
  S = S_scaled = list()
  # A$
  for(i in 1:nsubpop) {
    lambdaProg = lambdaProgMat[i,]
    lambda1 = lambdaMat1[i,]
    lambda2 = lambdaMat2[i,]
    timezero[i]
    if(all(lambdaProg == Inf)){
      funs = hazVFfun(Tint, lambda2)
    } else if(all(lambdaProg == 0)){
      funs = hazVFfun(Tint, lambda1)
    } else {
      funs = subpop_hazVFfun(Tint, lambda1, lambda2, lambdaProg, timezero[i])
    }
    S[[i]] = 1-funs$F
    S_scaled[[i]] = (1-funs$F)*p[i]
  }
  
  Smat = do.call(rbind,S)
  Smat_scaled = do.call(rbind,S_scaled)
  Smat[, 1:10]
  Smat_scaled[, 1:10]
  
  dat = do.call(rbind,lapply(1:ncol(Smat_scaled), function(x){
    values = Smat_scaled[, x]
    if(position=="fill") values = values / sum(values)
    data.frame(time = x,
               treatment = rep(c(1,2), each = 2),
               subgroup =  rep(c(1,2), 2),
               value =  values)
  }))
  comb = paste0(dat$treatment, dat$subgroup)
  dat$group = factor(comb, levels = rev(unique(comb)))
  names(cols_new) = levels(dat$group)
  
  # Calculate percentages for labels
  y1p = Smat_scaled[, 1]
  y1b1 = c(0,cumsum(y1p))
  y1b = c(0,cumsum(y1p))
  y1b = y1b[-length(y1b)] + y1p/2
  y1p = y1p*100
  
  y2p = Smat_scaled[, ncol(Smat_scaled)]
  if(position=="fill") y2p = y2p/sum(y2p)
  y2b1 = c(0,cumsum(y2p))
  y2b  = c(0,cumsum(y2p))
  y2b = y2b[-length(y2b)] + y2p/2
  y2p = y2p*100
  
  ggplot(dat) +
    geom_area(aes_string(x = "time", fill = "group", y = "value")) + 
    scale_fill_manual(values = cols_new, 
                      labels = c("CTRL - Subgr 2", "CTRL - Subgr 1", 
                                 "TRT - Subgr 2", "TRT - Subgr 1")) +
    guides(fill = guide_legend(title = " ", reverse=TRUE)) +
    coord_cartesian(expand = FALSE, clip="off") + 
    labs(x = "Time", y = "Percent", title = title) +
    theme(panel.background = element_rect(fill = "gray50"),
          panel.grid = element_blank(),
          legend.position = "bottom",
          axis.text.y=element_text(margin = margin(r = 10)),
          axis.text.y.right = element_text(margin = margin(l = 10)),
          axis.line.x.bottom = element_line(size = 1)) +
    scale_x_continuous(limits = c(0,max_time), breaks = seq(0, max_time, 500)) + 
    scale_y_continuous(breaks = y1b,
                       labels =  sprintf("%.1f%%", y1p), 
                       sec.axis = sec_axis(~., 
                                           breaks = y2b, 
                                           labels = sprintf("%.1f%%", y2p))) -> pppp
  pppp + theme(axis.ticks.length = unit(0.5, "cm"),
               axis.ticks.x = element_blank(),             
               axis.ticks.y = element_line(colour = "gray40"),
               axis.text.y = element_text(size = 11),
               axis.text.x = element_text(size = 12, color = "black", margin = margin(t = -10)))
    
}

##########################
# Power tab server

# Create ---------------------------------------------------------------------
# Initial definitions --------------------------------------------------------
p_lambdaRekr <- reactive({
  switch(input$p_lambdaRekr_unit,
         "day"   = input$p_lambdaRekr,
         "month" = input$p_lambdaRekr / (365.25/12),
         "year"  = input$p_lambdaRekr / 365.25)
})
p_lambdaZens <- reactive({
  switch(input$p_lambdaZens_unit,
         "day"   = input$p_lambdaZens,
         "month" = input$p_lambdaZens / (365.25/12),
         "year"  = input$p_lambdaZens / 365.25)
})
p_maxRekrKalenderZeit <- reactive({
  switch(input$p_maxRekrKalenderZeit_unit,
         "day"   = input$p_maxRekrKalenderZeit,
         "month" = input$p_maxRekrKalenderZeit * (365.25/12),
         "year"  = input$p_maxRekrKalenderZeit * 365.25)
})
p_maxKalender <- reactive({
  switch(input$p_maxKalender_unit,
         "day"   = input$p_maxKalender,
         "month" = input$p_maxKalender * (365.25/12),
         "year"  = input$p_maxKalender * 365.25)
})


# Calculate power --------------------------------------------------------
p_pow <- eventReactive(input$p_calculate, {
  set.seed(input$p_seed)
    colMeans(do.call(rbind, lapply(1:input$p_R, function(i){
      if (i %% 10 == 0) {
        cat(i, "\n")
        flush.console()
      }
      
      # Draw random data
      dat <- sample_fun(
        A = K5(),
        B = B5(),
        r0 = input$p_r0,
        eventEnd = input$p_eventEnde,
        lambdaRecr = p_lambdaRekr(),
        lambdaCens = p_lambdaZens(),
        maxRecrCalendarTime = p_maxRekrKalenderZeit(),
        maxCalendar = p_maxKalender()
      )
      LRT <- logrank.maxtest(
        time = dat$y,
        event = dat$event,
        group = dat$group,
        rho = c(0, 0, 1, 1),
        gamma = c(0, 1, 0, 1)
      )
      c(LRT$tests$p <= input$p_alpha, LRT$pmult <= input$p_alpha)
  })))
})


# Output results --------------------------------------------------------
output$p_results <- renderTable({
  data.frame(
    Test = c("Maximum Logrank", rep("Weighted Logrank", 4)),
    rho = c("0011", 0, 0, 1, 1),
    gamma = c("0101", 0, 1, 0, 1),
    power = p_pow())
}, 
caption = "<h4 style='text-align:center;color:black;'><b>Empirical power</b></h4>", 
caption.placement = getOption("xtable.caption.placement", "top"), 
caption.width = getOption("xtable.caption.width", NULL),
align = 'lccr', 
width = "50%")




###########---------------------------------------------------------------------

output$downloadReport <- downloadHandler(
  filename = function() {
    paste('my-report', sep = '.', switch(
      input$format, PDF = 'pdf', HTML = 'html', Word = 'docx'
    ))
  },
  
  content = function(file) {
    src <- normalizePath('nph-report.Rmd')
    
    # temporarily switch to the temp dir, in case you do not have write
    # permission to the current working directory
    owd <- setwd(tempdir())
    on.exit(setwd(owd))
    file.copy(src, 'nph-report.Rmd', overwrite = TRUE)
    # Set up parameters to pass to Rmd document
    params <- list(input = input,
                   p_pow = p_pow())
    
    library(rmarkdown)
    out <- render('nph-report.Rmd', switch(
      input$format,
      PDF = pdf_document(), HTML = html_document(), Word = word_document()
    ),  params = params,  envir = new.env(parent = globalenv()))
    file.rename(out, file)
  }
)


# Download code ----------------------------------------------------------------
output$p_downloadCode <- downloadHandler(
  filename = paste('nph-power-rcode.r'),
  content = function(file) {
    
    input1 = list()
    input1$times = 1
    input1$lambda = 2
    a = sprintf(
      "################################################################################
      ## nph package - Reproducible R code from shiny app.
      
      library('nph')
      
      times = %s                          # Define timebreaks for intervals
      
      # Create piecewise constant hazard objects for Treatment and Control arms
      # The lambda matrices are given in terms of the median survival times,
      # Therefore we use the nph::m2r function to translate from median to rate
      TRT <- pop_pchaz(
      Tint = times,
      lambdaMat1 = %s,   
      lambdaMat2 = %s,   
      lambdaProgMat = %s, 
      p = %s,
      timezero = FALSE, 
      discrete_approximation = TRUE
      )
      
      CRL <- pop_pchaz(
      Tint = times,
      lambdaMat1 = %s, 
      lambdaMat2 = %s,
      lambdaProgMat = %s, 
      p = %s,
      timezero = TRUE, 
      discrete_approximation = TRUE
      )
      
      print(TRT)
      print(CRL)
      
      # Create Survival, Hazard and hazard ratio plots
      plot_shhr(A = TRT, B = CRL)
      
      set.seed(%s)

      ### HINT: To speed up the computation, you can also use mclapply for parallel computing instead of lapply
      results = lapply(1:%s, function(i){
      if (i %%%% 10 == 0) {  # Counter to display in the console every 10 replications
      cat(i, '\\n')
      flush.console()
      }
      
      # Create one dataset
      dat <- sample_fun(
      A = TRT,
      B = CRL,
      r0 = %s,
      eventEnd = %s,
      lambdaRecr = %s,                       # Rate per day for recruiting patients, assuming recruitung follows a Poisson process
      lambdaCens = %s,                       # Rate per day for random censoring, assuming censoring times are exponential
      maxRecrCalendarTime = %s,              # Maximal duration of recruitment in days
      maxCalendar = %s                       # Maximal total study duration in days, after which the study stops
      )

      # Perform weighted log.rank tests
      LRT <- logrank.maxtest(
      time  = dat$y,
      event = dat$event,
      group = dat$group,
      rho   = c(0, 0, 1, 1),
      gamma = c(0, 1, 0, 1)
      )
      # Output the decision for each test
      c(LRT$tests$p <= %s, LRT$pmult <= %s)
      })
      
      power_table = power_table = data.frame(
      Test = c('Maximum Logrank', rep('Weighted Logrank', 4)),
      rho = c('0011', 0, 0, 1, 1),
      gamma = c('0101', 0, 1, 0, 1),
      power = colMeans(do.call(rbind, results)))
      
      print(power_table)
      ", 
    sprintf("c(%s)", paste0(s_T_times(), collapse = ", ")),   # The time intervals
    sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaMat1()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
    sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaMat2()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
    sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaProgMat()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
    sprintf("c(%s)", paste0(s_T_p(), collapse = ", ")),       # The proportions in the subgroups for treatment arm
    sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaMat1()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
    sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaMat2()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
    sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaProgMat()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
    sprintf("c(%s)", paste0(s_C_p(), collapse = ", ")),
    input$p_seed,
    input$p_R,
    input$p_r0,
    input$p_eventEnde,
    p_lambdaRekr(),
    p_lambdaZens(),
    p_maxRekrKalenderZeit(),
    p_maxKalender(),
    input$p_alpha, input$p_alpha
    )
    
    owd <- setwd(tempdir())
    on.exit(setwd(owd))
    capture.output(cat(a), file = paste0("test.R"))
    # tidy_file(file = "test.R", width.cutoff = 80)   # this is to format nicely the R code https://yihui.name/formatr/
    styler::style_file(path = "test.R", scope = "indention")  # this is to format nicely the R code
    
    out <- paste0("test.R")
    file.rename(out, file)
  }
)

##########################
# Power tab server


# downloadHandler() takes two arguments, both functions.
# The content function is passed a filename as an argument, and
#   it should write out data to that filename.
output$downloadData <- downloadHandler(
  
  # This function returns a string which tells the client
  # browser what name to use when saving the file.
  filename = function() {
    "nph_dataexample.csv"
  },
  
  # This function should write data to a file given to it by
  # the argument 'file'.
  content = function(file) {
    set.seed(input$cp_seed)
    data_example = sample_fun(
      A = K5(),
      B = B5(),
      r0 = input$cp_r0,
      eventEnd = 100,
      lambdaRecr = cp_lambdaRekr(),
      lambdaCens = cp_lambdaZens(),
      maxRecrCalendarTime = cp_maxRekrKalenderZeit(),
      maxCalendar = cp_maxKalender()
    )
    print(data_example)
    # data_example = data_example[, c(1,3,5)]
    # names(data_example) = c("group", "time", "event")
    # Write to a file specified by the 'file' argument
    write.csv(data_example, file,
              row.names = FALSE)
  }
)


nph.env$outputData<-NULL

Dataset <- shiny::reactive({
  if(is.null(input$outputfile)){
    return(data.frame())
  }
  dat <- data.frame(do.call("read.csv",
                            list(input$outputfile$datapath)))
  nph.env$outputData <- dat
  return(nph.env$outputData)
  
})

output$head_table <- renderTable(head(Dataset()),
                                 caption = "Preview first 6 rows of the dataset",
                                 caption.placement = getOption("xtable.caption.placement", "top"),
                                 caption.width = getOption("xtable.caption.width", NULL))

output$choose_Vars<-shiny::renderUI({
  if(is.null(input$outputfile))
    return("Please upload a dataset first")
  if(identical(Dataset(),'')||identical(Dataset(),data.frame()))
    return("Please upload a dataset first")
  nph.env$outputData <- Dataset()
  nph.env$NUM     <- dim(nph.env$outputData)[2] #Num of all variable  ##
  nph.env$Class   <- sapply(apply(nph.env$outputData,2,unique),length)  ##
  nph.env$Colnames <- colnames(nph.env$outputData) ##
  out <- column(6,
                selectInput("group", label="Grouping variable", c("",nph.env$Colnames), selected = ifelse("group" %in% nph.env$Colnames, "group", "")),
                selectInput("event", label="Event variable",    c("",nph.env$Colnames), selected = ifelse("event" %in% nph.env$Colnames, "event", "")),
                selectInput("y",     label="Time variable",     c("",nph.env$Colnames), selected = ifelse("y" %in% nph.env$Colnames, "y", "")),
                selectInput("inclusion", label="Calendar inclusion time variable", c("",nph.env$Colnames), selected = ifelse("inclusion" %in% nph.env$Colnames, "inclusion", "")),
                selectInput("yCalendar", label="Calendar event time variable",  c("",nph.env$Colnames), selected = ifelse("yCalendar" %in% nph.env$Colnames, "yCalendar", "")),
		    selectInput("adminCens", label="Administrative interim censoring indicator",  c("",nph.env$Colnames), selected = ifelse("adminCens" %in% nph.env$Colnames, "adminCens", ""))
		)
  out
})

group_opts <- shiny::eventReactive(input$group, {
  if(is.null(input$outputfile))
    return()
  if(identical(Dataset(),'')||identical(Dataset(),data.frame()))
    return(NULL)
  if(is.null(input$group)){
    return(NULL)
  }
  dat = Dataset()
  unique(dat[input$group])
})

output$choose_groups <- shiny::renderUI({
  if(is.null(input$outputfile))
    return()
  if(identical(Dataset(),'')||identical(Dataset(),data.frame()))
    return(NULL)
  dat = Dataset()
  nph.env$outputData <- dat
  nph.env$NUM     <- dim(nph.env$outputData)[2] #Num of all variable  ##
  nph.env$Class   <- sapply(apply(nph.env$outputData,2,unique),length)  ##
  nph.env$Colnames <- colnames(nph.env$outputData) ##
  # if(length(input$group) == 0){
  #   return(NULL)
  # }
  out <- column(6,
                selectInput("data_treatment", label="Treatment value in group variable", choices = c("",group_opts()), selected = ifelse(1 %in% group_opts(), "1", "")))
                # selectInput("data_control",   label="Control",   choices = c("",group_opts()), selected = ifelse(0 %in% group_opts(), "0", "")))
  out
})


# Create ---------------------------------------------------------------------
# Initial definitions --------------------------------------------------------
cp_lambdaRekr <- reactive({
  switch(input$cp_lambdaRekr_unit,
         "day"   = input$cp_lambdaRekr,
         "month" = input$cp_lambdaRekr / (365.25/12),
         "year"  = input$cp_lambdaRekr / 365.25)
})
cp_lambdaZens <- reactive({
  switch(input$cp_lambdaZens_unit,
         "day"   = input$cp_lambdaZens,
         "month" = input$cp_lambdaZens / (365.25/12),
         "year"  = input$cp_lambdaZens / 365.25)
})
cp_maxRekrKalenderZeit <- reactive({
  switch(input$cp_maxRekrKalenderZeit_unit,
         "day"   = input$cp_maxRekrKalenderZeit,
         "month" = input$cp_maxRekrKalenderZeit * (365.25/12),
         "year"  = input$cp_maxRekrKalenderZeit * 365.25)
})
cp_maxKalender <- reactive({
  switch(input$cp_maxKalender_unit,
         "day"   = input$cp_maxKalender,
         "month" = input$cp_maxKalender * (365.25/12),
         "year"  = input$cp_maxKalender * 365.25)
})


# Simulation of a data set until interim analysis at 150 events
# dat <- eventReactive(input$cp_calculate, {
#   set.seed(input$s_seed)
#   sample_fun(
#     A = K5(),
#     B = B5(),
#     r0 = input$cp_r0,
#     eventEnd = input$cp_eventinterim,
#     lambdaRecr = cp_lambdaRekr(),
#     lambdaCens = cp_lambdaZens(),
#     maxRecrCalendarTime = cp_maxRekrKalenderZeit(),
#     maxCalendar = cp_maxKalender()
#   )
# })

# Calculate power --------------------------------------------------------
cp_pow <- eventReactive(input$cp_calculate, {
  set.seed(input$cp_seed)
  
  ## We do not simulate one dataset but use the one provided by the user
  # dat = sample_fun(
  #   A = K5(),
  #   B = B5(),
  #   r0 = input$cp_r0,
  #   eventEnd = input$cp_eventinterim,
  #   lambdaRecr = cp_lambdaRekr(),
  #   lambdaCens = cp_lambdaZens(),
  #   maxRecrCalendarTime = cp_maxRekrKalenderZeit(),
  #   maxCalendar = cp_maxKalender()
  # )
  dat = Dataset()
  group = 1*(dat[input$group] == input$data_treatment)
  event = dat[input$event] 
  y = dat[input$y] 
  inclusion = dat[input$inclusion] 
  yCalendar = dat[input$yCalendar] 
  #neu: adminCens
  adminCens = dat[input$adminCens]
  dat = data.frame(group = group, inclusion = inclusion, y = y, yCalendar = yCalendar, event = event, adminCens=adminCens)
  dat$cumEvents <- cumsum(dat$event)
  
  colMeans(do.call(rbind, lapply(1:input$cp_R, function(i){
    if (i %% 10 == 0) {
      cat(i, "\n")
      flush.console()
    }
    
    # Draw random data
    dat_cond <- sample_conditional_fun(
      dat = dat,
      A = K5(),
      B = B5(),
      r0 = input$cp_r0,
      eventEnd = input$cp_eventEnde,
      lambdaRecr = cp_lambdaRekr(),
      lambdaCens = cp_lambdaZens(),
      maxRecrCalendarTime = cp_maxRekrKalenderZeit(),
      maxCalendar = cp_maxKalender()
    )
    LRT <- logrank.maxtest(
      time = dat_cond$y,
      event = dat_cond$event,
      group = dat_cond$group,
      rho = c(0, 0, 1, 1),
      gamma = c(0, 1, 0, 1)
    )
    c(LRT$tests$p <= input$cp_alpha, LRT$pmult <= input$cp_alpha)
  })))
})


# Output results --------------------------------------------------------
output$cp_results <- renderTable({
  data.frame(
    Test = c("Maximum Logrank", rep("Weighted Logrank", 4)),
    rho = c("0011", 0, 0, 1, 1),
    gamma = c("0101", 0, 1, 0, 1),
    power = cp_pow())
},
caption = "<h4 style='text-align:center;color:black;'><b>Empirical conditional power</b></h4>", 
caption.placement = getOption("xtable.caption.placement", "top"), 
caption.width = getOption("xtable.caption.width", NULL),
align = 'lccr', 
width = "50%")

output$cp_interimplot <- renderPlot({
  
  
  dat = Dataset()
  group = 1*(dat[input$group] == input$data_treatment)
  event = dat[input$event] 
  y = dat[input$y] 
  inclusion = dat[input$inclusion] 
  yCalendar = dat[input$yCalendar] 
  dat = data.frame(group = group, inclusion = inclusion, y = y, yCalendar = yCalendar, event = event)
  dat$cumEvents <- cumsum(dat$event)
  
  intplot(dat, K5(), B5(), main = "Interim data")
})


###########---------------------------------------------------------------------

output$cp_downloadReport <- downloadHandler(
  filename = function() {
    paste('my-report-condpower', sep = '.', switch(
      input$cp_format, PDF = 'pdf', HTML = 'html', Word = 'docx'
    ))
  },
  
  content = function(file) {
    src <- normalizePath('nph-report-condpower.Rmd')
    
    # temporarily switch to the temp dir, in case you do not have write
    # permission to the current working directory
    owd <- setwd(tempdir())
    on.exit(setwd(owd))
    file.copy(src, 'nph-report-condpower.Rmd', overwrite = TRUE)
    # Set up parameters to pass to Rmd document
    params <- list(input = input,
                   cp_pow = cp_pow(),
                   intplot = intplot,
                   dat = Dataset(),
                   K5 = K5(),
                   B5 = B5())
    
    library(rmarkdown)
    out <- render('nph-report-condpower.Rmd', switch(
      input$format,
      PDF = pdf_document(), HTML = html_document(), Word = word_document()
    ),  params = params,  envir = new.env(parent = globalenv()))
    file.rename(out, file)
  }
)




# Download code ----------------------------------------------------------------
output$cp_downloadCode <- downloadHandler(
  filename = paste('nph-condpower-rcode.r'),
  content = function(file) {
    
    input1 = list()
    input1$times = 1
    input1$lambda = 2
    a = sprintf(
      "################################################################################
      ## nph package - Reproducible R code from shiny app.
      
      library('nph')
      
      # We first define the dataset to be used. 
      # For reproducibility, we have copied it here, but you could also read it from the file
      group = %s
      event = %s
      y = %s
      inclusion = %s
      yCalendar = %s
      
      dat = data.frame(group = group, inclusion = inclusion, y = y, yCalendar = yCalendar, event = event)
      dat$cumEvents <- cumsum(dat$event)

      # Define model parameters
      times = %s                              # Define timebreaks for intervals
      
      # Create piecewise constant hazard objects for Treatment and Control arms
      # The lambda matrices are given in terms of the median survival times,
      # Therefore we use the nph::m2r function to translate from median to rate
      TRT <- pop_pchaz(
      Tint = times,
      lambdaMat1 = %s,   
      lambdaMat2 = %s,   
      lambdaProgMat = %s, 
      p = %s,
      timezero = FALSE, 
      discrete_approximation = TRUE
      )
      
      CRL <- pop_pchaz(
      Tint = times,
      lambdaMat1 = %s, 
      lambdaMat2 = %s,
      lambdaProgMat = %s, 
      p = %s,
      timezero = TRUE, 
      discrete_approximation = TRUE
      )
      
      print(TRT)
      print(CRL)
      
      # Create Survival, Hazard and hazard ratio plots
      plot_shhr(A = TRT, B = CRL)
      
      # Now plot model curves together with Kaplan-Meier curves
      intplot(dat, TRT, CRL, main = 'Interim data')
      
      set.seed(%s)
      
      ### HINT: To speed up the computation, you can also use mclapply for parallel computing instead of lapply
      results = lapply(1:%s, function(i){
      if (i %%%% 10 == 0) {  # Counter to display in the console every 10 replications
      cat(i, '\\n')
      flush.console()
      }
      
      # Draw random data
      dat_cond <- sample_conditional_fun(
      dat = dat,
      A = TRT,
      B = CRL,
      r0 = %s,
      eventEnd = %s,
      lambdaRecr = %s,                       
      lambdaCens = %s,                       
      maxRecrCalendarTime = %s,              
      maxCalendar = %s                       
      )

      # Perform weighted log.rank tests
      LRT <- logrank.maxtest(
      time  = dat$y,
      event = dat$event,
      group = dat$group,
      rho   = c(0, 0, 1, 1),
      gamma = c(0, 1, 0, 1)
      )
      # Output the decision for each test
      c(LRT$tests$p <= %s, LRT$pmult <= %s)
      })
      
      power_table = power_table = data.frame(
      Test = c('Maximum Logrank', rep('Weighted Logrank', 4)),
      rho = c('0011', 0, 0, 1, 1),
      gamma = c('0101', 0, 1, 0, 1),
      power = colMeans(do.call(rbind, results)))
      
      print(power_table)
      ", 
      sprintf("c(%s)", paste0(1*(nph.env$outputData[input$group] == input$data_treatment), collapse = ", ")),   
      sprintf("c(%s)", paste0(nph.env$outputData[input$event], collapse = ", ")),  
      sprintf("c(%s)", paste0(nph.env$outputData[input$y], collapse = ", ")),   
      sprintf("c(%s)", paste0(nph.env$outputData[input$inclusion], collapse = ", ")),   
      sprintf("c(%s)", paste0(nph.env$outputData[input$yCalendar], collapse = ", ")),  
      
      sprintf("c(%s)", paste0(s_T_times(), collapse = ", ")),   # The time intervals
      sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaMat1()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
      sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaMat2()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
      sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaProgMat()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
      sprintf("c(%s)", paste0(s_T_p(), collapse = ", ")),       # The proportions in the subgroups for treatment arm
      sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaMat1()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
      sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaMat2()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
      sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaProgMat()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
      sprintf("c(%s)", paste0(s_C_p(), collapse = ", ")),
      input$cp_seed,
      input$cp_R,
      input$cp_r0,
      input$cp_eventEnde,
      cp_lambdaRekr(),
      cp_lambdaZens(),
      cp_maxRekrKalenderZeit(),
      cp_maxKalender(),
      input$p_alpha, input$p_alpha
    )
    
    owd <- setwd(tempdir())
    on.exit(setwd(owd))
    
    cat(a)
    capture.output(cat(a), file = paste0("test.R"))
    tidy_file(file = "test.R", width.cutoff = 80)   # this is to format nicely the R code https://yihui.name/formatr/
    # styler::style_file(path = "test.R", scope = "indention")  # this is to format nicely the R code
    
    out <- paste0("test.R")
    file.rename(out, file)
  }
    )


# Download dataset at interim analysis -----------------------------------------

output$cp_downloadData <- downloadHandler(
  filename = function() {
    paste('interim-data', sep = '.', switch(
      input$cp_dataformat, csv = 'csv', excel = 'xls', txt = 'txt'))
  },
  
  content = function(file) {
    # Write to a file specified by the 'file' argument
    savefunc = switch(
      input$cp_dataformat, csv = write.csv, excel = write.xlsx, txt = write.table)
    savefunc(Dataset(), file,
              row.names = FALSE)
  }
)

##########################
# Simulate tab UI

tabPanel(
  title = "Conditional Power",
  id    = "condpowerTab",
  value = "condpowerTab",
  name  = "condpowerTab",
  class = "fade in",
  icon  = icon("line-chart"),
  h1("Calculate conditional power", style = "color:#f8c954"),
  #shiny::wellPanel(
   # downloadButton('cp_downloadCode', "Download R code", class = "start-button"),
  #  h4("Download a reproducible R script for the code to generate the results of this tab")),
  wellPanel(
    h2("1. Upload your data"),
    fluidRow(column(5,fileInput("outputfile",label="Select data file in csv")),
             # h5("Preview first 6 rows of the dataset"),
             fluidRow(column(6,tableOutput("head_table")))),
    fluidRow(column(12, "Note: The data should have an observation per row and four columns, one indicating the group (treatment or control), one indicating the time of the event, the event indicator, and the calendar time of the event.\n")),
    fluidRow(column(3, downloadButton('downloadData', 'Download an example dataset')))),
  wellPanel(
    h2("2. Choose the corresponding variables"),
    fluidRow(column(12,
                    uiOutput("choose_Vars"),
                    uiOutput("choose_groups"))
    )),
  wellPanel(
    h2("3. Select design parameters"),
    fluidRow(
      column(6, sliderInput("cp_R", "Number of simulations", min = 10, max = 1e4, value = 10, step = 50))),
    fluidRow(
      column(6, sliderInput("cp_r0", "Allocation ratio", min = 0, max = 1, value = 0.5))
      # ,column(6, numericInput("cp_eventinterim", "Maximal number of events in First stage data", min = 10, max = 1e6, value = 150))
      ,column(6, numericInput("cp_eventEnde", "Maximal number of events", min = 10, max = 1e6, value = 450))
     ), fluidRow(
      column(4, selectInput("cp_lambdaRekr_unit", "Recruitment rate unit",
                             choices = c("day", "month", "year"), selected = "year"))
      ,column(4, numericInput("cp_lambdaRekr", "Recruitment rate (Poisson assumption)", min = 0, max = 1e6, value = 300))
     ), fluidRow(
      column(4, selectInput("cp_lambdaZens_unit", "Censoring rate unit", 
                              choices = c("day", "month", "year"), selected = "year"))
      ,column(4, numericInput("cp_lambdaZens", "Censoring rate (Exponential assumption)", min = 0, max = 1e6, value = 0.013))
     ), fluidRow(
      column(4, selectInput("cp_maxRekrKalenderZeit_unit", "Maximal duration of recruitment unit", 
                              choices = c("day", "month", "year"), selected = "year"))
      ,column(4, numericInput("cp_maxRekrKalenderZeit", "Maximal duration of recruitment", min = 1, max = 1e6, value = 3))
     ), fluidRow(
      column(4, selectInput("cp_maxKalender_unit", "Maximal study duration unit", 
                              choices = c("day", "month", "year"), selected = "year"))
      ,column(4, numericInput("cp_maxKalender", "Maximal study duration", min = 1, max = 1e6, value = 4))
     ), fluidRow(
      column(6, numericInput("cp_alpha",
                              "Significance level for hypothesis test of equal hazard functions in both groups", 
                              min = 0, max = 1, value = 0.05))
      ,column(6, numericInput("cp_seed",
                              "Seed for simulations", 
                              min = 0, max = 1e5, value = floor(runif(1, min = 0, max = 1e5))))
    )
    )
  ,wellPanel(
    actionButton("cp_calculate", "Calculate power", icon = icon("calculator")),
    conditionalPanel(
      condition = "input.cp_calculate > 0",
      withSpinner(fluidRow(column(width = 12,
                      plotOutput(width = "50%", "cp_interimplot"), 
                      div(style = "min-height: 20px;"),
                      tableOutput("cp_results"), 
                      align="center"))),
      fluidRow(column(width = 6,
             h3("Create report"),
          radioButtons('cp_format', 'Document format', c('PDF', 'HTML', 'Word'),
                       inline = TRUE),
          downloadButton('cp_downloadReport')),
      column(width = 6, h3("Download interim data"),
          radioButtons('cp_dataformat', 'Data format', c('csv', 'excel', "txt"),
                       inline = TRUE),
          downloadButton('cp_downloadData'))),
      div(style = "min-height: 20px;"))
  )
)

##########################
# About tab UI

tabPanel(
  title = "About",
  id    = "aboutTab",
  value = "aboutTab",
  name  = "aboutTab",
  class = "home",
  icon  = icon("info-circle"),
  includeMarkdown(file.path("text", "about.md"))
  # includeMarkdown("inst/shiny/text/about.md")
)

##########################
# Examples tab UI

tabPanel(
  title = "Examples",
  id    = "examplesTab",
  value = "examplesTab",
  name  = "examplesTab",
  class = "fade in",
  icon  = icon("line-chart"),

  # h1("Hasse Diagramms"),
  h1("Examples of tail tests"),
  shiny::wellPanel(
    h2("1. Select an example"),
    fluidRow(column(8,
                    selectInput("e_example", label = "",
                                choices = c("Example of comparable products",
                                            "Example of no comparable products")))),
    shiny::fluidRow(column(12, actionButton("e_submit", "Calculate!")))
    ),
  shiny::wellPanel(
    conditionalPanel(
      condition = "input.e_submit > 0",
      h2("Results"),
      fluidRow(column(12,verbatimTextOutput("e_test_result"))),
      fluidRow(column(12,plotOutput("e_test_plot")))
    )
  )
)

#R Paket Survival non-proportional hazards

# @title Calculate survival for piecewise constant hazard
#
# @description Calculates hazard, cumulative hazard, survival and distribution function
# 	based on hazards that are constant over pre-specified time-intervals
# 	
# @param Tint vector of boundaries of time intervals (presumably in days) with piecewise constant hazard. The boundaries should be increasing and the first one should
#	be \code{0}, the last one should be larger than the assumed trial duration.
# @param lambda vector of piecewise constant hazards for the intervals speciefied via \code{Tint}. 
# 
# @details Function values are calculated for all integer time points between 0 and the maximum of \code{Tint}.
# @return A list with class \code{mixpch} containing the following components:
# \describe{
#	\item{\code{haz}}{Values of the hazard function.}
#	\item{\code{cumhaz}}{Values of the cumulative hazard function.}
#	\item{\code{S}}{Values of the survival function.}
#	\item{\code{F}}{Values of the distribution function.}
#	\item{\code{t}}{Time points for which the values of the different functions are calculated.}
# }
# @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}
# @seealso \code{\link{subpop_hazVFfun}}, \code{\link{pop_hazVFfun}}, \code{\link{plot.mixpch}}
# @examples
# hazVFfun(Tint=c(0,40,100),lambda=c(.02,.05))
#Das heißt ab inklusive Tag 0 gilt lambda=0.02, ab inklusive Tag 40 gilt lambda 0.05
#Wenn das Event am Tag 40 mit dem Hazard 0.05 eintritt, wird es am Tag 41 registriert.
hazVFfun<-function(Tint,lambda) {
  if (length(Tint) != (length(lambda) + 1)) {
    stop("The length of Tint should be equal to the length of lambda + 1")
  }
  t<-seq(0,max(Tint)-1,1)
  haz<-rep(0,length(t))
  cumhaz<-rep(0,length(t))
  for(i in 1:(length(Tint)-1)) {
    haz<-haz+lambda[i]*(Tint[i] <= t & t < Tint[i+1])
    cumhaz<-cumhaz+lambda[i]*(Tint[i+1]-Tint[i])*(t>=Tint[i+1])+ lambda[i]*(t-Tint[i])*(Tint[i] <= t & t < Tint[i+1])
  }
  #haz
  #cumhaz<-c(0,cumsum(haz[-length(t)]))
  #cumhaz-c(0,cumsum(haz[-length(t)]))
  S<-exp(-cumhaz)
  F<- 1-S
  out<-list(haz=haz,cumhaz=cumhaz,S=S,F=F,t=t,
            Tint = Tint, lambda = lambda)
  class(out)<-"mixpch" #damit wir die plot.mixpch Ffunktion verwenden koennen
  out
}

#lambdaTod = lambda1
#lambdaTodnachSwitch = lambda2
#lambdaSwitch = lambdaProg
#nachswithZeit0 = timezero

# @title Calculate survival for piecewise constant hazards with change after random time
#
# @description Calculates hazard, cumulative hazard, survival and distribution function
# 	based on hazards that are constant over pre-specified time-intervals
# 	
# @param Tint vector of boundaries of time intervals (presumably in days) with piecewise constant hazard. The boundaries should be increasing and the first one should
#	be \code{0}, the last one should be larger than the assumed trial duration.
# @param lambda1 vector of piecewise constant hazards before the changeing event happens, for the intervals speciefied via \code{Tint}. 
# @param lambda2 vector of piecewise constant hazards after the changeing event has happened, for the intervals speciefied via \code{Tint}. 
# @param lambdaProg vector of piecewise constant hazards for the changeing event, for the intervals speciefied via \code{Tint}. 
# @param timezero logical, indicating whether after the changeing event the timecount, governing which interval in \code{Tint} and which according value in 
#   \code{lambda2} is used, should restart at zero.
# 
# @details Function values are calculated for all integer time points between 0 and the maximum of \code{Tint}.
# @return A list with class \code{mixpch} containing the following components:
# \describe{
#	\item{\code{haz}}{Values of the hazard function.}
#	\item{\code{cumhaz}}{Values of the cumulative hazard function.}
#	\item{\code{S}}{Values of the survival function.}
#	\item{\code{F}}{Values of the distribution function.}
#	\item{\code{t}}{Time points for which the values of the different functions are calculated.}
# }
# @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}
# @seealso \code{\link{hazVFfun}}, \code{\link{pop_hazVFfun}}, \code{\link{plot.mixpch}}
# @examples
# subpop_hazVFfun(Tint=c(0,40,100),lambda1=c(0.2,0.4),lambda2=c(0.1,0.01),
#	lambdaProg=c(0.5,0.4),timezero=FALSE)
# subpop_hazVFfun(Tint=c(0,40,100),lambda1=c(0.2,0.4),lambda2=c(0.1,0.01),
#	lambdaProg=c(0.5,0.4),timezero=TRUE)
subpop_hazVFfun<-function(Tint,lambda1,lambda2,lambdaProg,timezero) {
  t<-seq(0,max(Tint)-1,1)
  
  hVF_Tod<-hazVFfun(Tint,lambda1)
  hVF_TodnachSwitch<-hazVFfun(Tint,lambda2)
  hVF_Switch<-hazVFfun(Tint,lambdaProg)
  #Mischen:
  #gehe alle moeglichen Switchzeitpunkt t durch,
  #fuer jeden Zeitpunkt berechen F bedingt auf diesen Switchzeitpunkt.
  #Mische diese F mit der WS fuer die Switchzeitpunkt
  nt<-length(t)
  
  # Calculate assuming that theprogression is at the end of the day
  Smat<-matrix(NA,nrow=nt,ncol=nt+1) #+1 weil auch die Moeglichkeit gar nicht zu switchen bis max(t) vorkommen kann
  #Zeilen: t, Spalten bedingte Survfunktion (P(Y>t|switch=t_spalte)) fuer switchzeit t_spalte
  prob_t<-c(hVF_Switch$F[1],diff(hVF_Switch$F),hVF_Switch$S[nt]) #WS fuer switch im Intervall (t_i-1,t]
  #if(sum(prob_t)!=1) warning("Prob. ist nicht 1")
  #S=Smat%*%prob_t
  set<-rep(TRUE,nt)	
  #delta<-diff(t)
  for(i in 1:(nt+1)) {
    #Es gibt zwei Versionen: 1) Zeit nach Switch geht fuer hazard_nachswitch(t) einfach weiter:
    if(!timezero) haz<-c(hVF_Tod$haz[!set],hVF_TodnachSwitch$haz[set])
    #2) Zeit nach Switch beginnt fuer hazard_nachswitch wieder bei 0.
    #bzw. nach switch zu Zeit s ist der hazard hazard_nachswitch(t-s):
    if(timezero) {
      #haz<-c(hVF_Tod$haz[!set],hVF_TodnachSwitch$haz[1:(nt-sum(!set))])
      haz<-c(hVF_Tod$haz[!set],hVF_TodnachSwitch$haz[rev(set)])
      #set ist immer von der Struktur FALSE,...,FALSE,TRUE,..,TRUE.
      #rev(set) bewirkt, dass die ersten sum(set) Eintraege von 
      #VF_TodnachSwitch$haz benutzt werden,
      #beachte: c(x,vektor[c(FALSE,...,FALSE)])  ergibt x
    }
    #cumhaz<-c(0,cumsum(haz[-nt]*delta)) #delta ist jetzt immer 1
    cumhaz<-c(0,cumsum(haz[-nt]))
    Smat[,i]<-exp(-cumhaz)
    set[i]<-!set[i] #Switchzeitpunkt um eine Position verschieben
  }
  #plot(t,hVF_TodnachSwitch$haz,col=1,ylim=c(0,3))
  #lines(t,hVF_Tod$haz,col=2)
  
  Smix1<-as.numeric(Smat%*%prob_t)
  
  
  # Calculate assuming that the progression is at the beginning of the day
  Smat<-matrix(NA,nrow=nt,ncol=nt) #+1 weil auch die Moeglichkeit gar nicht zu switchen bis max(t) vorkommen kann
  #Zeilen: t, Spalten bedingte Survfunktion (P(Y>t|switch=t_spalte)) fuer switchzeit t_spalte
  prob_t<-c(diff(hVF_Switch$F),hVF_Switch$S[nt]) #WS fuer switch im Intervall (t_i-1,t]
  #if(sum(prob_t)!=1) warning("Prob. ist nicht 1")
  #S=Smat%*%prob_t
  set<-rep(TRUE,nt)	
  #delta<-diff(t)
  for(i in 1:(nt)) {
    #Es gibt zwei Versionen: 1) Zeit nach Switch geht fuer hazard_nachswitch(t) einfach weiter:
    if(!timezero) haz<-c(hVF_Tod$haz[!set],hVF_TodnachSwitch$haz[set])
    #2) Zeit nach Switch beginnt fuer hazard_nachswitch wieder bei 0.
    #bzw. nach switch zu Zeit s ist der hazard hazard_nachswitch(t-s):
    if(timezero) {
      #haz<-c(hVF_Tod$haz[!set],hVF_TodnachSwitch$haz[1:(nt-sum(!set))])
      haz<-c(hVF_Tod$haz[!set],hVF_TodnachSwitch$haz[rev(set)])
      #set ist immer von der Struktur FALSE,...,FALSE,TRUE,..,TRUE.
      #rev(set) bewirkt, dass die ersten sum(set) Eintraege von 
      #VF_TodnachSwitch$haz benutzt werden,
      #beachte: c(x,vektor[c(FALSE,...,FALSE)])  ergibt x
    }
    #cumhaz<-c(0,cumsum(haz[-nt]*delta)) #delta ist jetzt immer 1
    cumhaz<-c(0,cumsum(haz[-nt]))
    Smat[,i]<-exp(-cumhaz)
    set[i]<-!set[i] #Switchzeitpunkt um eine Position verschieben
  }
  #plot(t,hVF_TodnachSwitch$haz,col=1,ylim=c(0,3))
  #lines(t,hVF_Tod$haz,col=2)
  
  Smix2<-as.numeric(Smat%*%prob_t)
  
  Smix = (Smix1+Smix2)/2
  
  #lines(t,Smix,col=5)
  #lines(t,Smat[,nt+1],col=4)
  
  Fmix<-1-Smix
  cummixhaz<- -log(Smix)
  mixhaz<- c(diff(cummixhaz),NA)
  #fmix<-c(diff(Fmix),NA) 
  
  
  
  
  #cummixhaz-cumsum(c(0,mixhaz[1:(length(mixhaz)-1)]))
  #zwei moegliche numerische Approximationen, besser die, die keine Division erforden, sonst Probleme bei Nennerwerten nahe 0.
  #fmix<-c(diff(Fmix)/delta,NA) 
  #mixhaz<-fmix/Smix
  #mixhaz2<- c(diff(cummixhaz)/delta,NA)
  #fmix2<-mixhaz2*Smix
  
  #plot(mixhaz,mixhaz2)
  #plot(fmix,fmix2)
  #abline(0,1)
  
  out<-list(haz=mixhaz,cumhaz=cummixhaz,S=Smix,F=Fmix,
            t=t, Tint = Tint, 
            lambda1 = lambda1, lambda2 = lambda2, lambdaProg = lambdaProg, timezero = timezero)
  class(out)<-c("mixpch", "subpop")  #mix of piecewise constantant hazard
  out
}


# @title Calculate survival for piecewise constant hazards with change after random time and mixture of subpopulations
#
# @description Calculates hazard, cumulative hazard, survival and distribution function
# 	based on hazards that are constant over pre-specified time-intervals
# 
# @param Tint vector of boundaries of time intervals (presumably in days) with piecewise constant hazard. The boundaries should be increasing and the first one should
#	be \code{0}, the last one should be larger than the assumed trial duration.
# @param lambdaMat1 matrix, each row contains the vector of piecewise constant hazards for one subpopulation before the changeing event happens, for the intervals speciefied via \code{Tint}. 
# @param lambdaMat2 matrix, each row contains the vector piecewise constant hazards for one subpopulation after the changeing event has happened, for the intervals speciefied via \code{Tint}. 
# @param lambdaProgMat matrix, each row contains the vector of piecewise constant hazards for one subpopulation for the changeing event, for the intervals speciefied via \code{Tint}. 
# @param p vector of relative sizes (proportions) of the subpopulations.
# @param timezero logical, indicating whether after the changing event the timecount, governing which interval in \code{Tint} and which according value in 
#   \code{lambda2} is used, should restart at zero. This argument is either of length 1 (applying the same to all subgroups) or the same length as the number of subgroups.
#
# @details Function values are calculated for all integer time points between 0 and the maximum of \code{Tint}.
# @return A list with class \code{mixpch} containing the following components:
# \describe{
#	\item{\code{haz}}{Values of the hazard function.}
#	\item{\code{cumhaz}}{Values of the cumulative hazard function.}
#	\item{\code{S}}{Values of the survival function.}
#	\item{\code{F}}{Values of the distribution function.}
#	\item{\code{t}}{Time points for which the values of the different functions are calculated.}
# }
# @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}
# @seealso \code{\link{hazVFfun}}, \code{\link{subpop_hazVFfun}}, \code{\link{plot.mixpch}}
# @examples
# pop_hazVFfun(Tint=c(0,40,100),lambdaMat1=matrix(c(0.2,0.1,0.4,0.1),2,2),
#	lambdaMat2=matrix(c(0.5,0.2,0.6,0.2),2,2),
#	lambdaProg=matrix(c(0.5,0.5,0.4,0.4),2,2),p=c(0.8,0.2),timezero=FALSE)
pop_hazVFfun<-function(Tint,lambdaMat1,lambdaMat2,lambdaProgMat,p,timezero=TRUE) {
  if (!identical(dim(as.matrix(lambdaMat1)), dim(as.matrix(lambdaMat2)), dim(as.matrix(lambdaProgMat)))) {
    stop("Dimensions of all lambda matrices should be the same")
  }
  # Now we are sure lambdaMat1,lambdaMat2,lambdaProgMat have the same dimensions
  # We can compare only one to Tint and p
  if ((length(Tint)-1) != ncol(as.matrix(lambdaMat1))){
    stop("The length of Tint should be equal to the number of column of the lambda matrices + 1")
  }
  if (length(p) != nrow(as.matrix(lambdaMat1))){
    stop("The length of p should be equal to the number of column of the lambda matrices")
  }
  if (length(timezero) == 1){
    timezero = rep(timezero, length(p))
  }
  if (length(timezero) != length(p)){
    stop("timezero should be of length 1 or the same length as p")
  }
  
	t<-seq(0,max(Tint)-1,1)

	#wichtig: damit cumsum fuer kumulativen Hazard genau stimmt:
	nsubpop<-length(p)
	Fmix<-rep(0,length(t))
	#fmix<-rep(0,length(t))
	for(i in 1:nsubpop) {
		hazVF_pop_i<-subpop_hazVFfun(Tint,lambdaMat1[i,],lambdaMat2[i,],lambdaProgMat[i,],timezero=timezero[i])
		Fmix<-Fmix + p[i]*hazVF_pop_i$F
		#fmix<-fmix + p[i]*hazVF_pop_i$f
	}
	Smix<-1-Fmix
	cummixhaz<- -log(Smix)
	mixhaz<- c(diff(cummixhaz),NA)
	#hazmix<-fmix/Smix #Achtung, wenn Smix nahe an 0 ist, wird das numerisch ungenau
	out<-list(haz=mixhaz,cumhaz=cummixhaz,S=Smix,F=Fmix,
	          t=t, Tint = Tint, 
	          lambdaMat1 = lambdaMat1, lambdaMat2 = lambdaMat2, 
	          lambdaProgMat = lambdaProgMat, p = p, timezero = timezero)
	class(out)<-c("mixpch", "pop") #mix of piecewise constantant hazard
	out
}

#' @title Plot mixpch Objects
#'
#' @description Plots survival and other functions stored in \code{mixpch} objects versus time.
#' 
#' @param x an object of class \code{mixpch}.
#' @param fun character string in \code{c("S","F","haz","cumhaz")} indicating which function to plot. Select \code{"S"} for the survival function,
#'	\code{"F"} for the distribution functin, \code{"haz"} for the hazard function or \code{"cumhaz"} for the cumulative hazard function.
#' @param add logical, indicates if the drawing should be added to an existing plot.
#' @param xlab label of the x-axis
#' @param ylab label of the y-axis
#' @param ... further arguments passed to the plotting functions
#' 
#' @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}
#' 
#' @seealso \code{\link{pchaz}}, \code{\link{subpop_pchaz}}, \code{\link{pop_pchaz}}
#' 
#' @references
#' Robin Ristl, Nicolas Ballarini, Heiko Götte, Armin Schüler, Martin Posch, Franz König. Delayed treatment effects, treatment switching and 
#' heterogeneous patient populations: How to design and analyze RCTs in oncology. Pharmaceutical statistics. 2021; 20(1):129-145. 
#'
#' @examples
#' A <- pop_pchaz(Tint = c(0, 90, 1500),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.2, 0.6, 0.2), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.4, 0.4), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' plot(A)
#' plot(A, "haz", add = TRUE)
#' 
#' @import graphics
#' @export
plot.mixpch<-function(x,fun=c("S","F","haz","cumhaz"),add=FALSE,ylab=fun,xlab="Time",...) {
	fun<-match.arg(fun,c("S","F","haz","cumhaz"))
	if(!add) plot(x$t,x[[fun]],type="l",ylab=ylab,xlab=xlab,...)
	if(add) lines(x$t,x[[fun]],...)
}


#' @title Draw random survival times from mixpch object.
#'
#' @description Draws independent random survival times from \code{mixpch} objects.
#' 
#' @param n Number of random draws
#' @param x An object of class \code{mixpch}
#' 
#' @return A vector of random survival times.
#' 
#' @details The mixpch object stores the survival function up to some time T. For random times equal or larger T, the value T is returned.
#'
#' @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}
#' 
#' @references
#' Robin Ristl, Nicolas Ballarini, Heiko Götte, Armin Schüler, Martin Posch, Franz König. Delayed treatment effects, treatment switching and 
#' heterogeneous patient populations: How to design and analyze RCTs in oncology. Pharmaceutical statistics. 2021; 20(1):129-145. 
#'
#' @seealso \code{\link{rSurv_conditional_fun}}, \code{\link{sample_fun}}, \code{\link{sample_conditional_fun}}
#' 
#' @examples
#' A <- pop_pchaz(Tint = c(0, 90, 1500),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.2, 0.6, 0.2), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.4, 0.4), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' rSurv_fun(n = 10, x = A)
#' 
#' @export
rSurv_fun<-function(n,x) {
	#max(t) muss >= maximale Beobachtungsdauer sein. Denn fuer grosze u wird einfach max(t) erzeugt.
	#x pop_hazVFfun output
	u<-runif(n)
	Y<-rep(0,n)
	for(i in 1:n) {
		#ind<-sum(x$F<=u[i])
		#Wir geben nur diskrete Zeiten auf einen Tag genau aus.
		#Auf diese Art (siehe unten) wird auf den naechsten ganzen Tag aufgerundet.
		#(Die mixpch Objekte kennen dazwischen die Verteilungsfunktion ja auch gar nicht genau.)
		#So kann auch keine Zeit = 0 gezogen werden.
		ind<-sum(x$F<=u[i])
		Y[i]<-x$t[ind]+1
		#ind<-sum(x$F<u[i])+1 #das ist das selbe, aber kann zu Index auszerhalb des Bereichs fuehren
		#Y[i]<-x$t[ind]
	}
	Y
}

#' @title Draw conditional random survival times from mixpch object.
#'
#' @description Draws independent random survival times from \code{mixpch} objects conditional on 
#' 	observed time.
#' @param x An object of class \code{mixpch}
#' @param y A vector of observed right censored times
#' @return A vector of random survival times, conditional on the observed censored times.
#' @details 
#' 	Note that the mixpch object stores the survival function up to some time T. For random times equal or larger T, the value T is returned.
#' @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}
#' @seealso \code{\link{rSurv_fun}}, \code{\link{sample_fun}}, \code{\link{sample_conditional_fun}}
#'
#' @references
#' Robin Ristl, Nicolas Ballarini, Heiko Götte, Armin Schüler, Martin Posch, Franz König. Delayed treatment effects, treatment switching and 
#' heterogeneous patient populations: How to design and analyze RCTs in oncology. Pharmaceutical statistics. 2021; 20(1):129-145. 
#'
#' @examples
#' A <- pop_pchaz(Tint = c(0, 90, 1500),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.2, 0.6, 0.2), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.4, 0.4), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' rSurv_conditional_fun(x = A, y = c(10,15,9,2,1))
#' @export
rSurv_conditional_fun<-function(x,y) {
	#max(t) muss >= maximale Beobachtungsdauer sein. Denn fuer grosze u wird einfach max(t) erzeugt.
	#x pop_hazVFfun output
	#y Beobachtungszeit bisher
	n<-length(y)
	u<-runif(n)
	Y<-rep(0,n)
	for(i in 1:n) {
		ind_y<-sum(x$t<=y[i])
		gr_y<-x$t>=y[i]
		F_bedingt<-((x$F - x$F[ind_y] )/ x$S[ind_y])[gr_y]
		#ind<-sum(F_bedingt<=u[i])
		#Wir geben nur diskrete Zeiten auf einen Tag genau aus.
		#Auf diese Art (siehe unten) wird auf den naechsten ganzen Tag aufgerundet.
		#(Die mixpch Objekte kennen dazwischen die Verteilungsfunktion ja auch gar nicht genau.)
		#ind<-sum(F_bedingt<u[i])+1 
		#Y[i]<-(x$t[gr_y])[ind]
		ind<-sum(F_bedingt<=u[i])
		Y[i]<-(x$t[gr_y])[ind]+1
	}
	Y
}

#bedingt auf 0 ergibt das selbe wie unbedingt (muss so sein, auch im R Code ist dann alles gleich)
#bed<-rSurv_conditional_fun(A,rep(0,1000))
#ran<-rSurv_fun(1000,A)
#t.test(bed,ran)
#qqplot(bed,ran)
#table(bed)
#table(ran)
###

#' @title Draw survival times based on study settings
#'
#' @description Simulates data for a randomized controlled survival study.
#' @param A An object of class \code{mixpch}, resembling the survival function in treatment group 0
#' @param B An object of class \code{mixpch}, resembling the survival function in treatment group 1
#' @param r0 Allocation ratio to group 0 (must be a number between 0 and 1)
#' @param eventEnd Number of events, after which the study stops
#' @param lambdaRecr Rate per day for recruiting patients, assuming recruitung follows a Poisson process
#' @param lambdaCens Rate per day for random censoring, assuming censoring times are exponential
#' @param maxRecrCalendarTime Maximal duration of recruitment in days
#' @param maxCalendar Maximal total study duration in days, after which the study stops
#' 
#' @return A data frame with each line representing data for one patient and the following columns:
#' \describe{
#'	\item{\code{group}}{Treatment group}
#'	\item{\code{inclusion}}{Start of observation in terms of calendar time}
#'	\item{\code{y}}{Observed survival/censored time}
#'	\item{\code{yCalendar}}{End of observation in terms of calendar time.}
#'	\item{\code{event}}{logical, \code{TRUE} indicates the observation ended with an event, \code{FALSE} corresponds to censored times}
#'	\item{\code{adminCens}}{logical, \code{True} indicates that the observation is subject to administrative censoring, i.e. the subject was observed until the 
#'		end of the study without an event.}
#'	\item{\code{cumEvents}}{Cumulative number of events over calendar time of end of observation}
#' }
#' The data frame is ordered by \code{yCalendar}
#' @details For simulating the data, patients are allocated randomly to either group (unrestricted randomization).
#' @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}
#' @seealso \code{\link{rSurv_fun}}, \code{\link{rSurv_conditional_fun}}, \code{\link{sample_conditional_fun}}
#'
#' @references
#' Robin Ristl, Nicolas Ballarini, Heiko Götte, Armin Schüler, Martin Posch, Franz König. Delayed treatment effects, treatment switching and 
#' heterogeneous patient populations: How to design and analyze RCTs in oncology. Pharmaceutical statistics. 2021; 20(1):129-145. 
#'
#' @examples
#' A <- pop_pchaz(Tint = c(0, 90, 1500),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.2, 0.6, 0.2), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.4, 0.4), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' B <- pop_pchaz(Tint = c(0, 90, 1500),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.1, 0.6, 0.1), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.04, 0.04), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' plot(A)
#' plot(B, add = TRUE)
#' dat <- sample_fun(A, B, r0 = 0.5, eventEnd = 30, lambdaRecr = 0.5,
#'   lambdaCens = 0.25 / 365, maxRecrCalendarTime = 2 * 365,
#'   maxCalendar = 4 * 365)
#' @export
sample_fun<-function(A,B,r0=0.5,eventEnd,lambdaRecr,lambdaCens,maxRecrCalendarTime,maxCalendar) {
  #A, B pop_hazVFfun Objekte
  if(maxCalendar<maxRecrCalendarTime | max(A$t)<maxCalendar | max(B$t)<maxCalendar) stop("Zeiten falsch spezifiziert.")
  
  #alle Raten pro Tag
  #r0<-0.5 #Randomisierungswahrsch fuer Gruppe 0
  
  #Rekrutierungsrate (Poissonprozess)
  #lambdaRecr<-0.5 #pro Tag, alles in Tagen
  #lambdaCens<-0.25/365
  n<-0 #Anzahl Pat
  e<-0 #Anzahl events
  kal<- 0 #kalenderzeit
  
  #eventEnd (frueher e_interim) #Eventzahl, bei der (Interim)analyse ausgefuehrt wird.
  #maxRecrCalendarTime<-2*365
  #maxCalendar<-4*365
  dat<-NULL
  nmax<-rpois(1,lambda=lambdaRecr * maxRecrCalendarTime)
  inclusion<-runif(nmax,0, maxRecrCalendarTime)
  inclusion<-round(inclusion)
  group<-sample(0:1,nmax,TRUE,c(r0,1-r0))
  n0<-sum(group==0)
  n1<-sum(group==1)
  y<-rep(0,nmax)
  y[group==0] <- rSurv_fun(n0,A)
  y[group==1] <- rSurv_fun(n1,B)
  if(lambdaCens>0) zens<-rexp(nmax,lambdaCens) else zens<-rep(Inf,nmax)
  event<-y<zens
  y<-ifelse(event,y,zens)
  y<-round(y)
  yCalendar<-inclusion+y
  #events nach maxCalendar zensieren
  zensEnde<-yCalendar>maxCalendar
  event[zensEnde]<-FALSE
  y[zensEnde]<-maxCalendar-inclusion[zensEnde]
  yCalendar[zensEnde]<-maxCalendar
  
  #suche die Zeit, bis zu der eventEnd events aufgetreten sind (falls das passiert ist).	
  dat<-data.frame(group,inclusion,y,yCalendar,event,adminCens=zensEnde)
  dat<-dat[order(dat$yCalendar),]
  #head(dat)
  dat$cumEvents<-cumsum(dat$event)
  n_events<-sum(dat$event)
  if(n_events>=eventEnd) {
    kalenderEnde<-dat$yCalendar[sum(dat$cumEvents<eventEnd)+1]
    #min(which(dat$cumEvents>=eventEnd))
    kalenderEnde
    #neu zensieren:
    dat<-dat[dat$inclusion<=kalenderEnde,]
    zensEnde2<-dat$yCalendar>kalenderEnde
    dat$adminCens<-zensEnde2
    dat$event[zensEnde2]<-FALSE
    dat$y[zensEnde2]<-kalenderEnde-dat$inclusion[zensEnde2]
    dat$yCalendar[zensEnde2]<-kalenderEnde
    
  } else {
    warning("Geplante Eventzahl nicht erreicht.")
  }
  dat$cumEvents<-cumsum(dat$event)
  #while(e<eventEnd) {
  #	kal<-kal+rexp(1,lambdaRecr)
  #	kal<-round(kal) #Zeiten auf ganze Tage runden:
  #	group<-sample(0:1,1,TRUE,c(p_0,1-p_0))
  #	if(group==0) zeit<-rSurv_fun(1,A) #kontrolle
  #	if(group==1) zeit<-rSurv_fun(1,B) #treatment
  #	zens<-rexp(1,lambdaCens)
  #	y<-min(zeit,zens)	
  #	y<-round(y) #Zeiten auf ganze Tage runden:
  #	yCalendar<-kal+y  #event/zens Zeit in Kalenderzeit
  #	event<-zeit<zens
  #	dat<-rbind(dat,data.frame(group,inclusion=kal,y,yCalendar,event))
  #	e<-sum(dat$yCalendar[dat$event]<=kal)
  #}
  #e
  #sum(dat$event)
  #dim(dat)
  #kal
  #hist(dat$inclusion)
  #dat_obs<-dat
  #head(dat)
  #dat_obs$event[dat_obs$yCalendar>kal]<-FALSE
  #sum(dat_obs$event)
  #dat_obs$yCalendar[dat_obs$yCalendar>kal]<-kal
  #dat_obs$y<-dat_obs$yCalendar-dat_obs$inclusion
  #head(dat_obs)
  #dat_obs
  #class(dat)<-"simsurv" #nicht sinnvoll, weil dadurch dat kein Dataframe mehr ist
  dat
}


##
#' @title Draw conditional survival times based on study settings
#'
#' @description Simulates data for a randomized controlled survival study conditional on observed interim data.
#' 
#' @param dat A data frame with the same structure and column names as the output of \code{\link{sample_fun}}, containing the data to condition on
#' @param A An object of class \code{mixpch}, resembling the survival function in treatment group 0
#' @param B An object of class \code{mixpch}, resembling the survival function in treatment group 1
#' @param r0 Allocation ratio to group 1 (must be a number between 0 and 1)
#' @param eventEnd Number of events, after which the study stops
#' @param lambdaRecr Rate per day for recruiting patients, assuming recruitung follows a Poisson process
#' @param lambdaCens Rate per day for random censoring, assuming censoring times are exponential
#' @param maxRecrCalendarTime Maximal duration of recruitment in days
#' @param maxCalendar Maximal total study duration in days, after which the study stops
#' 
#' @return A data frame with each line representing data for one patient and the following columns:
#' \describe{
#'	\item{\code{group}}{Treatment group}
#'	\item{\code{inclusion}}{Start of observation in terms of calendar time}
#'	\item{\code{y}}{Observed survival/censored time}
#'	\item{\code{yCalendar}}{End of observation in terms of calendar time.}
#'	\item{\code{event}}{logical, \code{TRUE} indicates the observation ended with an event, \code{FALSE} corresponds to censored times}
#'	\item{\code{adminCens}}{logical, \code{True} indicates that the observation is subject to administrative censoring, i.e. the subject was observed until the 
#'		end of the study without an event.}
#'	\item{\code{cumEvents}}{Cumulative number of events over calendar time of end of observation}
#' }
#' The data frame is ordered by \code{yCalendar}
#' @details For simulating the data, patients are allocated randomly to either group (unrestricted randomization).
#' @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}
#' @seealso \code{\link{rSurv_fun}}, \code{\link{rSurv_conditional_fun}}, \code{\link{sample_fun}}
#'
#' @references
#' Robin Ristl, Nicolas Ballarini, Heiko Götte, Armin Schüler, Martin Posch, Franz König. Delayed treatment effects, treatment switching and 
#' heterogeneous patient populations: How to design and analyze RCTs in oncology. Pharmaceutical statistics. 2021; 20(1):129-145. 
#'
#'
#' @examples
#' A <- pop_pchaz(Tint = c(0, 90, 1500),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.2, 0.6, 0.2), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.4, 0.4), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' B <- pop_pchaz(Tint = c(0, 90, 1500),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.1, 0.6, 0.1), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.04, 0.04), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' datinterim <- sample_fun(A, B, r0 = 0.5, eventEnd = 30, lambdaRecr = 1,
#'   lambdaCens = 0.25 / 365,
#'	 maxRecrCalendarTime = 3 * 365,
#'	 maxCalendar = 4 * 365)
#' datcond <- sample_conditional_fun(datinterim, A, B, r0 = 0.5, eventEnd = 60,
#'   lambdaRecr = 1, lambdaCens = 0.25 / 365, maxRecrCalendarTime = 3 * 365, 
#'   maxCalendar = 4 * 365)
#' @export
sample_conditional_fun<-function(dat,A,B,r0=0.5,eventEnd,lambdaRecr,lambdaCens,maxRecrCalendarTime,maxCalendar) {
  #Kalenderzeit kal, zu der Stichprobe beendet wurde
  kal<-max(dat$yCalendar)
  if(kal>maxRecrCalendarTime) stop("Rekrutierungsende erreicht.")
  if(maxCalendar<maxRecrCalendarTime | max(A$t)<maxCalendar | max(B$t)<maxCalendar) stop("Zeiten falsch spezifiziert.")
  
  
  #1) Fuer alle Zensierten eventzeiten und zenszeiten ziehen 
  dat_sim<-dat
  #setA<-dat$event==FALSE & dat$group==0
  #setB<-dat$event==FALSE & dat$group==1
  setA<-dat$adminCens==TRUE & dat$group==0
  setB<-dat$adminCens==TRUE & dat$group==1
  
  #zuerst Zensierungszeiten berechnen, denn dafuer brauchen wir das bisherige y
  setAB<-setA | setB
  if(lambdaCens>0) {
    zens_sim<-dat$y[setAB] + rexp(sum(setAB),lambdaCens) #Achtung, das geht nur, wenn die Zensierungszeiten exponentialverteilt sind,
    #weil nur dann die bedingte Verteilung wieder die selbe Exponentialverteilung ist.
  } else {
    zens_sim<-rep(Inf,sum(setAB))
  }
  
  #setA<-dat_sim$event==FALSE & dat$group==0
  #setB<-dat_sim$event==FALSE & dat$group==1
  
  dat_sim$y[setA]<-rSurv_conditional_fun(A,y=dat$y[setA])
  dat_sim$y[setB]<-rSurv_conditional_fun(B,y=dat$y[setB])
  #zensInd<-dat_sim$y[setAB]<zens_sim
  dat_sim$event[setAB]<- dat_sim$y[setAB]<zens_sim
  dat_sim$y[setAB & dat_sim$event==FALSE ]<-zens_sim[!(dat_sim$y[setAB] < zens_sim)]
  
  dat_sim$y[setAB]<-round(dat_sim$y[setAB])
  dat_sim$yCalendar[setAB]<-dat_sim$inclusion[setAB]+dat_sim$y[setAB]
  
  #head(dat_obs,10)
  #head(dat_sim,10)
  
  #2) hypothetische neue Patienten rekrutieren bis zum Ende der Rekrutierungszeit
  #Anzahl ist Poissonverteilt mit Rate lambdaRecr * (maxRecrCalendarTime -kal)
  nneu<-rpois(1,lambda=lambdaRecr * (maxRecrCalendarTime -kal))
  #wann sie rekrutiert werden ist gleichverteilt auf (kal,maxRecrCalendarTime]
  inclusionneu<-runif(nneu,kal,maxRecrCalendarTime)
  inclusionneu<-round(inclusionneu)
  #Gruppe zufaellig zuordnen
  groupneu<-sample(0:1,nneu,TRUE,c(r0,1-r0))
  #Eventzeiten ziehen
  yneu<-rep(NA,nneu)
  yneu[groupneu==0]<-rSurv_fun(sum(groupneu==0),A)
  yneu[groupneu==1]<-rSurv_fun(sum(groupneu==1),B)
  #Zensierung ziehen
  if(lambdaCens>0) zensneu<-rexp(nneu,lambdaCens) else zensneu<-rep(Inf,nneu)
  eventneu<-yneu<zensneu
  yneu[eventneu==FALSE]<-zensneu[eventneu==FALSE]
  yneu<-round(yneu)
  yCalendarneu<-inclusionneu+yneu  #event/zens Zeit in Kalenderzeit
  dat_simneu<-data.frame(group=groupneu,inclusion=inclusionneu,y=yneu,yCalendar=yCalendarneu,event=eventneu,adminCens=FALSE)
  dat_simneu$cumEvents<-cumsum(dat_simneu$event)
  
  
  #3) Daten zusammenfuegen und alle Zeiten die nach Studienende liegen zensieren
  dat_sim_alle<-rbind(dat_sim,dat_simneu)
  #suche die Zeit, bis zu der eventEnd events aufgetreten sind (falls das passiert ist).	
  dat_sim_alle<-dat_sim_alle[order(dat_sim_alle$yCalendar),]
  dat_sim_alle$cumEvents<-cumsum(dat_sim_alle$event)
  n_events<-sum(dat_sim_alle$event)
  if(n_events>=eventEnd) {
    kalenderEnde<-dat_sim_alle$yCalendar[sum(dat_sim_alle$cumEvents<eventEnd)+1]
  } else {
    warning("Geplante Eventzahl nicht erreicht.")
    # in diesem Fall alle Events zensieren, die nach Studienende liegen
    kalenderEnde<-maxCalendar
  }
  dat_sim_alle<-dat_sim_alle[dat_sim_alle$inclusion<=kalenderEnde,]
  zensEnde2<-dat_sim_alle$yCalendar>kalenderEnde
  dat_sim_alle$event[zensEnde2]<-FALSE
  dat_sim_alle$adminCens<-zensEnde2
  dat_sim_alle$y[zensEnde2]<-kalenderEnde-dat_sim_alle$inclusion[zensEnde2]
  dat_sim_alle$yCalendar[zensEnde2]<-kalenderEnde
  dat_sim_alle$cumEvents<-cumsum(dat_sim_alle$event)
  dat_sim_alle
}

###TESTS
#' @title Weighted log-rank test
#'
#' @aliases logrank.test print.wlogrank
#' @description Calculates a weighted log-rank test for the comparison of two groups.
#' 
#' @param time Vector of observed event/censored times
#' @param event logical vector or numeric vector with entries 0 or 1, indicating if an event was observed (TRUE or 1) or the time is censored (FALSE or 0)
#' @param group Vector of group allocations
#' @param alternative Either of \code{"two.sided"},\code{"less"} or \code{"greater"}, specifies if two-sided or respective
#'	one-sided p-values are calculated. In any case the z test statistic of each included weighted log-rank test 
#'	is based on the (weighted) sum of expected minus observed
#'	events in the group corresponding to the first factor level of \code{group}. Hence a small value of the test statistic corresponds to a 
#'	lower (weighted average) hazard rate in the first group.
#' @param rho Parameter to calculate weights in the rho-gamma family
#' @param gamma Parameter to calculate weights in the rho-gamma family
#' @param event_time_weights Optional vector of user defined weights. This weight vector needs to have one entry per event time (not per event, as multiple events may
#'	occur at the same time) and must be sorted by increasing event time.
#' 
#' @return A list with elements:
#' \describe{
#'	\item{\code{D}}{A data frame event numbers, numbers at risk and expected number of events for each event time}
#'	\item{\code{test}}{A data frame containing the z and chi-squared statistic for the one-sided and two-sided test, respectively,
#'		of the null hypothesis of equal hazard functions in both groups and the p-value for the one-sided test.
#'	}
#'	\item{\code{var}}{The estimated variance of the sum of expected minus observed events in the first group.}
#' }
#' 
#' @details 
#' For a given sample, let \eqn{\mathcal{D}} be the set of unique event times.
#' For a time-point \eqn{t \in \mathcal{D}}, let \eqn{n_{t,ctr}} and \eqn{n_{t,trt}} be 
#' the number of patients at risk in the control and treatment group and let
#' \eqn{d_{t,ctr}} and \eqn{d_{t,trt}} be the respective number of events. 
#' The expected number of events in the control group is calculated under the
#' least favorable configuration in \eqn{H_0}, 
#' \eqn{\lambda_{ctr}(t) = \lambda_{trt}(t)}, as \eqn{e_{t,ctr}=(d_{t,ctr}+d_{t,trt}) 
#' \frac{n_{t0}}{n_{t0}+n_{t1}}}. The conditional variance of \eqn{d_{t,ctr}}
#' is calculated from a hypergeometric distribution as 
#' \eqn{var(d_{t,ctr})=\frac{n_{t0} n_{t1} (d_{t0}+d_{t1}) (n_{t0}+n_{t1} - d_{t0} - d_{t1})}{(n_{t0}+n_{t1})^2 (n_{t0}+n_{t1}-1)}}.
#' Further define a weighting function \eqn{w(t)}.
#' The weighted logrank test statistic for a comparison of two groups is 
#' \deqn{z=\sum_{t \in \mathcal{D}} w(t) (d_{t,ctr}-e_{t,ctr}) / \sqrt{\sum_{t \in \mathcal{D}} w(t)^2 var(d_{t,ctr})}}
#' 
#' Under the the least favorable configuration in \eqn{H_0}, 
#' the test statistic is asymptotically standard normally distributed and large 
#' values of \eqn{z} are in favor of the alternative.
#' 
#' The function consider particular weights in the Fleming-Harrington \eqn{\rho-\gamma} 
#' family \eqn{w(t)=\hat S(t-)^\rho (1-\hat S(t-))^\gamma}. 
#' Here, \eqn{\hat{S}(t)=\prod_{s \in \mathcal{D}: s \leq t} 1-\frac{d_{t,ctr}+d_{t,trt}}{n_{t,ctr}+n_{t,trt}}}
#' is the pooled sample Kaplan-Meier estimator.
#' (Note: Prior to package version 2.1, \eqn{S(t)} was used in the definition of \eqn{\rho-\gamma} weights,
#' this was changed to \eqn{S(t-)} with version 2.1.)
#' Weights \eqn{\rho=0, \gamma=0} correspond to the standard logrank test with 
#' constant weights \eqn{w(t)=1}. Choosing \eqn{\rho=0, \gamma=1} puts more weight on 
#' late events, \eqn{\rho=1, \gamma=0} puts more weight on early events and 
#' \eqn{\rho=1, \gamma=1} puts most weight on events at intermediate time points.
#' 
#' @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}
#' @seealso \code{\link{logrank.maxtest}}
#' 
#' @references 
#' Robin Ristl, Nicolas Ballarini, Heiko Götte, Armin Schüler, Martin Posch, Franz König. Delayed treatment effects, treatment switching and 
#' heterogeneous patient populations: How to design and analyze RCTs in oncology. Pharmaceutical statistics. 2021; 20(1):129-145. 
#'
#' Thomas R Fleming and David P Harrington. Counting processes and survival analysis. John Wiley & Sons, 2011
#'
#' @examples
#' A <- pop_pchaz(Tint = c(0, 90, 1500),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.2, 0.6, 0.2), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.4, 0.4), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' B <- pop_pchaz(Tint = c(0, 90, 1500),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.1, 0.6, 0.1), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.04, 0.04), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' dat <- sample_fun(A, B, r0 = 0.5, eventEnd = 30,
#'   lambdaRecr = 0.5, lambdaCens = 0.25 / 365,
#'	 maxRecrCalendarTime = 2 * 365,
#'	 maxCalendar = 4 * 365)
#' logrank.test(dat$y, dat$event, dat$group)
#' 
#' 
#' @import stats
#' @export
logrank.test<-function(time,event,group,alternative=c("two.sided", "less", "greater"),
                       rho=0,gamma=0,event_time_weights=NULL) {
  call <- match.call()
  alternative = match.arg(alternative)
  n<-length(time)
  ng<-table(group)
  #ord<-order(time)
  #time<-time[ord]
  #event<-event[ord]
  #group<-factor(group[ord])
  group<-factor(group)
  #if(is.null(weights)) weights<-weights[ord]
  
  Ag<-aggregate(event,by=list(time=time,group=group),FUN=sum,drop=FALSE)
  Ag$x <- ifelse(is.na(Ag$x), 0, Ag$x) #Nico's fix, because in new R version, drop=FALSE creates NA instead of 0 for empty combinations
  #aggregate(event~gruppe*y,data=dat_sim_alle,FUN=sum,drop=FALSE)
  #aggregate(data.frame(event),by=list(time=time,group=group),FUN=sum,drop=FALSE)
  #Die drop=FALSE Einstellung ergibt diese Warnmeldung:
  #Warnmeldung:
  #In `levels<-`(`*tmp*`, value = if (nl == nL) as.character(labels) else paste0(labels,  :
  #  duplicated levels in factors are deprecated
  #tritt nicht auf, wenn y ganzzahlig ist
  
  #str(Ag)
  #Ag
  #cbind(Ag[Ag$group==levels(group)[1],],	Ag[Ag$group==levels(group)[2],])
  tab<-data.frame(time=Ag$time[Ag$group==levels(group)[1]],event1=Ag$x[Ag$group==levels(group)[1]],event2=Ag$x[Ag$group==levels(group)[2]])
  tab$atrisk1<-NA
  tab$atrisk2<-NA
  for(i in 1:dim(tab)[1]) {
    tab$atrisk1[i]<-sum(time[group==levels(group)[1]]>=tab$time[i])
    tab$atrisk2[i]<-sum(time[group==levels(group)[2]]>=tab$time[i])
  }
  
  nz<-dim(tab)[1]
  #tab$atrisk1<-ng[1]-c(0, cumsum(tab$event1)[1:(nz-1)])
  #tab$atrisk2<-ng[2]-c(0, cumsum(tab$event2)[1:(nz-1)])
  tab$atrisk<-tab$atrisk1+tab$atrisk2
  tab$event<-tab$event1+tab$event2
  #tab<-data.frame(time,event,group)#,atrisk,atrisk1,atrisk2,group)
  #tab
  #sf<-survfit(Surv(time=time,event=event)~group)
  #str(sf)
  #as.data.frame(sf)
  #sftab<-data.frame(time=sf$time,n.risk=sf$n.risk,n.event=sf$n.event,gruppe=rep(c(0,1),times=sf$strata))
  #sf0<-sftab[sftab$gruppe==0,]
  #sf1<-sftab[sftab$gruppe==1,]
  #sfall<-merge(sf0,sf1,by="time",all=TRUE)
  
  D<-tab[tab$event>0,]
  D$expected1<-D$event*D$atrisk1/D$atrisk
  D$expected2<-D$event*D$atrisk2/D$atrisk
  #D$expected1_mode<-(D$event+1)*(D$atrisk1+1)/(D$atrisk+2)
  
  #D$observed1<-as.numeric(D$group==levels(group)[1])
  D$diff1<-D$event1-D$expected1
  #D$var<-D$atrisk1*D$event*(D$atrisk-D$atrisk1)*(D$atrisk-D$event)/( D$atrisk^2*(D$atrisk-1) )
  D$var<-D$atrisk1*D$event*D$atrisk2*(D$atrisk-D$event)/( D$atrisk^2*(D$atrisk-1) )
  
  D$S<-	cumprod((D$atrisk-D$event)/D$atrisk) #Kaplan-Meier estimator
  D$Sminus<-c(1,D$S[-length(D$S)])

  #wenn weights bereitgestellt werden:
  if (is.null(event_time_weights)) {
	#rho-gamma weights now use S(t-) instead of S(t)
	D$w <- D$Sminus^rho * (1 - D$Sminus)^gamma 
  } else {
	#user defined weights now need one entry per event time, sorted by event time
	D$w <- event_time_weights/sum(event_time_weights)
	rho<-NA
	gamma<-NA
  }
  #if(is.null(weights)) D$w<-D$S^rho*(1-D$S)^gamma else D$w<-weights[D$time+1] #+1, denn Zeiten starten im weights Vektor mit t=0
  #weights muss Gewichte fuer alle Zeiten t=0,1,... bis zur maximalen Eventzeit enthalten
  #D$w<-D$S^rho*(1-D$S)^gamma
  #D
  Dvoll<-D #aufheben, falls wir das ganze D im output haben wollen
  D<-D[D$atrisk1>0 & D$atrisk2>0,] #sobald es nur mehr eine Gruppe gibt, gibt es keinen Beitrag mehr zur Teststatistik bzw. Varianz wird NaN.
  #Gewichtete Logrank Statistik
  z<-sum(D$w*D$diff1)/sqrt(sum(D$w^2*D$var)) #z-Statistik
  Chisq<-(sum(D$w*D$diff1))^2/sum(D$w^2*D$var)	#Chi-Quadrat Statisitk
  df<-nlevels(group)-1
  if(alternative=="two.sided")	p<-1-pchisq(Chisq,df=df)
  if(alternative=="less")	p<-pnorm(z)
  if(alternative=="greater") p<-1-pnorm(z)
  #S<-Surv(time=time,event=event)
  #lt<-survdiff(S~group) #ok
  #str(lt)
  #colSums(D)
  out = list(D=D,test=data.frame(rho,gamma,z,Chisq,df,p,alternative),var=sum(D$w^2*D$var),
             obs = c(sum(D$event1), sum(D$event2)),
             exp = c(sum(D$expected1), sum(D$expected2)),
             n   =  as.numeric(ng),
             call = call)
  class(out) = "wlogrank"
  out
}


###


#' @title Maximum combination (MaxCombo) log-rank test
#'
#' @aliases logrank.maxtest print.wlogrank_max
#' 
#' @description Calculates a MaxCombo test for the comparison of two groups based on the maximum of test statistics of a set of weighted log-rank tests
#' 
#' @param time Vector of observed event/censored times
#' @param event logical vector or numeric vector with entries 0 or 1, indicating if an event was observed (TRUE or 1) or the time is censored (FALSE or 0)
#' @param group Vector of group allocations
#' @param alternative Either of \code{"two.sided"},\code{"less"} or \code{"greater"}, specifies if two-sided or respective
#'	one-sided p-values are calculated. In any case the z test statistic of each included weighted log-rank test 
#'	is based on the (weighted) sum of expected minus observed
#'	events in the group corresponding to the first factor level of \code{group}. Hence a small value of the test statistic corresponds to a 
#'	lower (weighted average) hazard rate in the first group.
#' @param rho Vector of parameter values rho for a set of weighting functions in the rho-gamma family
#' @param gamma Vector of parameter values gamma for a set of weighting functions in the rho-gamma family
#' @param event_time_weights Optional matrix, each column containing a different weighting vector for the event times.
#' 	These weight vectors need to have one entry per event time (not per event, as multiple events may
#'	occur at the same time) and must be sorted by increasing event time.
#' @param algorithm algorithm for the multivariate normal integration to be used in \code{\link[mvtnorm]{pmvnorm}}.
#' 
#' @return A list with elements:
#' \describe{
#'	\item{\code{pmult}}{The two sided p-value for the null hypothesis of equal hazard functions in both groups, based on the multivariate 
#'	normal approximation for the z-statistics of differently weighted log-rank tests.}
#'	\item{\code{p.Bonf}}{The two sided p-value for the null hypothesis of equal hazard functions in both groups, based on a
#'		Bonferroni multiplicity adjustment for differently weighted log-rank tests.}
#'	\item{\code{tests}}{Data frame with z-statistics and two-sided unadjusted p-values of the individual weighted log-rank tests}
#'	\item{\code{korr}}{Estimated correlation matrix for the z-statistics of the differently weighted log-rank tests.}
#' }
#' 
#' @details 
#' To perform a maximum-type combination test, a set of \eqn{m} different weight 
#' functions \eqn{w_1(t), \dots, w_m(t)} is specified and the correspondingly
#' weighted logrank statistics \eqn{z_1,\dots,z_m} are calculated. The maximum
#' test statistic is \eqn{z_{max}=\max_{i=1,\dots,m} z_i}. If at least one of
#' the selected weight functions results in high power, we may expect a large
#' value of \eqn{z_{max}}. 
#' Under the least favorable configuration in \eqn{H_0}, approximately
#' \eqn{(Z_1,\dots,Z_m)\sim N_m({0},{\Sigma})}. The p-value of the maximum 
#' test, \eqn{P_{H_0}(Z_{max}>z_{max})=1-P(Z_1 \leq z_{max},\dots,Z_m \leq z_{max})},
#' is calculated based on this multivariate normal approximation via numeric integration.
#' The integration is done using \code{\link[mvtnorm]{pmvnorm}}. The default settings in
#' \code{logrank.maxtest} correspond to greater precision than the original default of
#' \code{\link[mvtnorm]{pmvnorm}}. Precision can be set via the argument \code{algorithm}.
#' Lower precision settings may speed up caclulation.
#' 
#' The multivariate normal approach automatically corrects for multiple testing with
#' different weights and does so efficiently since the correlation between the different 
#' tests is incorporated in \eqn{{\Sigma}}. For actual calculations, \eqn{{\Sigma}} is
#'  replaced by an estimate.
#  Note that \eqn{cov (w_i(t)d_{t,ctr},w_j(t)d_{t,ctr})=w_i(t)w_j(t) var(d_{t,ctr})},
#  at least approximately assuming weights are converging in probability to 
#  a non-random function. Thus the \eqn{i,j}-the element of \eqn{{\Sigma}} is estimated as
#  \deqn{\hat{cov}(Z_i,Z_j)=\sum_{t \in \mathcal{D}}  w_i(t)w_j(t)var(d_{t,ctr})/ \sqrt{\sum_{t \in \mathcal{D}}  w_i^2(t) var(d_{t,ctr})   \sum_{t \in \mathcal{D}} w_j^2(t) var(d_{t,ctr})}}
#'  
#' 
#' @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}
#' @seealso \code{\link{logrank.test}}
#'
#' @references
#' Robin Ristl, Nicolas Ballarini, Heiko Götte, Armin Schüler, Martin Posch, Franz König. Delayed treatment effects, treatment switching and 
#' heterogeneous patient populations: How to design and analyze RCTs in oncology. Pharmaceutical statistics. 2021; 20(1):129-145. 
#'
#' Pranab Ghosh, Robin Ristl, Franz König, Martin Posch, Christopher Jennison, Heiko Götte, Armin Schüler, Cyrus Mehta. Robust group sequential
#' designs for trials with survival endpoints and delayed response. Biometrical Journal. First published online: 21 December 2021
#'
#' Tarone RE. On the distribution of the maximum of the logrank statistic and the modified wilcoxon statistic. Biometrics. 1981; 37:79-85.
#'
#' Lee S-H. On the versatility of the combination of the weighted log-rank statistics. Comput Stat Data Anal. 2007; 51(12):6557-6564. 
#'
#' Karrison TG et al. Versatile tests for comparing survival curves based on weighted log-rank statistics. Stata J. 2016; 16(3):678-690. 
#'
#' @examples
#' A <- pop_pchaz(Tint = c(0, 90, 1500),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.2, 0.6, 0.2), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.4, 0.4), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' B <- pop_pchaz(Tint = c(0, 90, 1500),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2) / 365,
#'	 lambdaMat2 = matrix(c(0.5, 0.1, 0.6, 0.1), 2, 2) / 365,
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.04, 0.04), 2, 2) / 365,
#'	 p = c(0.8, 0.2), 
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' dat <- sample_fun(A, B, r0 = 0.5, eventEnd = 30,
#'   lambdaRecr = 0.5, lambdaCens = 0.25 / 365,
#'	 maxRecrCalendarTime = 2 * 365,
#'	 maxCalendar = 4 * 365)
#' logrank.maxtest(dat$y, dat$event, dat$group)
#' 
#' @importFrom mvtnorm pmvnorm
#' @export
logrank.maxtest<-function(time,event,group,alternative=c("two.sided", "less","greater"),
                          rho=c(0,0,1),gamma=c(0,1,0),event_time_weights=NULL,algorithm = mvtnorm::GenzBretz(maxpts = 50000, abseps = 0.00001, releps = 0)) {  #event_time_weights ist Matrix, pro Spalte ein Gewichtsvektor
  #require(mvtnorm) #das muss fuer das Paket aus der Funktion in die namespace datei verschoben werden.
  call <- match.call()
  alternative = match.arg(alternative)
  if(!is.null(rho))	mrg<-length(rho) else mrg<-0
  if(!is.null(event_time_weights)) {
    if(!is.matrix(event_time_weights)) event_time_weights<-matrix(event_time_weights,ncol=1)
    mw<-dim(event_time_weights)[2]
  } else {
    mw<-0
  }
  m<-mrg+mw
  
  testListe<-vector(mode="list",length=m)
  z<-rep(0,m)
  p<-rep(0,m)
  for(i in 1:m) {
    if(i<=mrg) testListe[[i]]<-logrank.test(time=time,event=event,group=group,alternative=alternative,rho=rho[i],gamma=gamma[i])
    if(i>mrg) testListe[[i]]<-logrank.test(time=time,event=event,group=group,alternative=alternative,event_time_weights=event_time_weights[,i-mrg])
    z[i]<-testListe[[i]]$test$z
    p[i]<-testListe[[i]]$test$p
  }
  V<-diag(1,m)
  for(i in 1:(m-1)) {
    for(j in 2:m) {
      kov<-sum(testListe[[i]]$D$w*testListe[[j]]$D$w*testListe[[i]]$D$var)  #test1$D$var==test2$D$var
      kor<-kov/sqrt(testListe[[i]]$var)/sqrt(testListe[[j]]$var)
      V[i,j]<-V[j,i]<-kor
    }
  }
  #V
  #z
  if(alternative=="two.sided") {
    maxz<-max(abs(z))
    low=-rep(maxz,m)
    up=rep(maxz,m)
    pmult<-1-mvtnorm::pmvnorm(lower=low,upper=up,corr=V,algorithm=algorithm)
  }
  
  if(alternative=="less") {
    minz<-min(z)
    low=rep(minz,m)
    up=rep(Inf,m)
    pmult<-1-mvtnorm::pmvnorm(lower=low,upper=up,corr=V,algorithm=algorithm)
  }
  if(alternative=="greater") {
    maxz<-max(z)
    low=rep(-Inf,m)
    up=rep(maxz,m)
    pmult<-1-mvtnorm::pmvnorm(lower=low,upper=up,corr=V,algorithm=algorithm)
  }
  
  #pmult
  #p
  #2*(1-pnorm(maxz))
  p.Bonf=p.adjust(p,"bonferroni")
  #tests=data.frame(rho,gamma,z,p)
  out = list(pmult=pmult[[1]],p.Bonf=min(p.Bonf),tests=data.frame(Test=1:m,z,p),korr=V,alternative,
             logrank.test = testListe,
             call = call)
  class(out) = "wlogrank_max"
  out
}

##########################
# Simulate tab server
# Create ---------------------------------------------------------------------
# Initial definitions --------------------------------------------------------

## Treatment group parameters --------------------------------------------------

# Number of subgroups ----------------------------------------------------------
observe({
  isolate({
    output$s_T_subgr_p <-renderTable({
      if (input$s_T_subgr<1) return("Number of subgroups should be > 0")
      if (input$s_T_subgr>=10) return("Number of subgroups should be < 10")
      num.inputs.col1 <- paste0("<input id='s_T_p", 1:input$s_T_subgr, 
                                "' class='shiny-bound-input' ",
                                "type='number' max = 1 min = 0 step = 0.1 value=", 
                                1/input$s_T_subgr, " />")
      out = as.data.frame(t(num.inputs.col1),
                    row.names = "Subgroup proportions")
      names(out) = paste0("S", 1:input$s_T_subgr)
      out
    },
    sanitize.text.function = function(x) x, 
    # rownames = TRUE, 
    width = '100%')
    
    output$s_T_subgr_labels <-renderTable({
      if (input$s_T_subgr<1) return("Number of subgroups should be > 0")
      if (input$s_T_subgr>=10) return("Number of subgroups should be < 10")
      num.inputs.col1 <- paste0("<div class='form-group shiny-input-container' style='width: 20%;'><input id='s_T_label", 
                                1:input$s_T_subgr, 
                                "' class='shiny-bound-input' ",
                                "type='text' value='Subgr", 
                                1:input$s_T_subgr, "' /></div>")
      out = as.data.frame(t(num.inputs.col1),
                          row.names = "Subgroup labels")
      names(out) = paste0("S", 1:input$s_T_subgr)
      out
    },
    sanitize.text.function = function(x) x, 
    # rownames = TRUE, 
    width = '100%')
  })
})


s_T_p <- reactive({
  unlist(lapply(1:input$s_T_subgr, function(i){
    input[[paste0("s_T_p", i)]]
  }))
})
s_T_labels <- reactive({
  unlist(lapply(1:input$s_T_subgr, function(i){
    input[[paste0("s_T_label", i)]]
  }))
})

# output$s_T_p <- renderText(s_T_p())

# Number of time intervals -----------------------------------------------------
observe({
  isolate({
    output$s_T_time <-renderTable({
      if (input$s_T_timepoints<1) return("Number of timepoints should be > 0")
      if (input$s_T_timepoints>=10) return("Number of timepoints should be < 10")
      num.inputs.col1 <- 
        c("<input id='s_T_time0' class='shiny-bound-input' type='number' max = 999999 min = 0 step = 0 value=0 disabled=true>",
          paste0("<input id='s_T_time", 1:input$s_T_timepoints, 
                 "' class='shiny-bound-input' ",
                 "type='number' max = 999999 min = 0 step = 1 value=", 
                 (1:input$s_T_timepoints)*3000, ">"))
      out = as.data.frame(t(num.inputs.col1),
                          row.names = "Subgroup proportions")
      names(out) = paste0("T", 0:input$s_T_timepoints)
      out
    }, sanitize.text.function = function(x) x, 
    # rownames = TRUE, 
    width = '100%')
  })
})


s_T_times <- reactive({
  unlist(lapply(0:input$s_T_timepoints, function(i){
    input[[paste0("s_T_time", i)]]
  }))
})

# output$s_T_times <- renderText(s_T_times())






### lambdaMat1 -----------------------------------------------------------------
s_T_grid_Mat1 <- reactive({
  grid_comb. =  expand.grid(1:input$s_T_timepoints, 1:input$s_T_subgr)
  paste0("T_Mat1_s", grid_comb.$Var1, "t", grid_comb.$Var2)
})
observe({
  isolate({
    output$s_T_lambdaMat1 <-renderTable({
      if(is.null(s_T_grid_Mat1())) return(NULL)
      num.inputs.col1 <- 
        paste0("<input id='s_", s_T_grid_Mat1(), 
               "' class='shiny-bound-input' ",
               " type='number' max = 999 min = 0 step = 1 value=", 
                 18, ">")
      out = as.data.frame(matrix(num.inputs.col1, nrow = input$s_T_subgr))
      
      t1=paste0("s_T_time", (1:input$s_T_timepoints)-1)
      t2=paste0("s_T_time", (1:input$s_T_timepoints))
      names(out) = unlist(lapply(1:input$s_T_timepoints, function(i)
        paste0(input[[t1[i]]], "-", input[[t2[i]]])))
      # names(out) = paste0("T", 1:input$s_T_timepoints)
      rownames(out) = paste0("S", 1:input$s_T_subgr)
      out
    }, sanitize.text.function = function(x) x, 
    rownames = TRUE,
    width = 'auto')
  })
})


s_T_lambdaMat1 <- reactive({
  mm = unlist(lapply(1:length(s_T_grid_Mat1()), function(i){
    input[[paste0("s_", s_T_grid_Mat1()[i])]]
  }))
  if(is.null(mm)) return(NULL)
  if(length(mm)%%input$s_T_subgr != 0) return(NULL) # Catch error when mm is not ready to be in the matrix
  matrix(fu(mm), nrow = input$s_T_subgr)
})

output$s_T_lambdaMat1_print <- renderTable({
  if(is.null(s_T_grid_Mat1())) return(NULL)
  s_T_lambdaMat1()
  }, digits = 6)

### lambdaMat2 -----------------------------------------------------------------
s_T_grid_Mat2 <- reactive({
  grid_comb. =  expand.grid(1:input$s_T_timepoints, 1:input$s_T_subgr)
  paste0("T_Mat2_s", grid_comb.$Var1, "t", grid_comb.$Var2)
})
observe({
  isolate({
    output$s_T_lambdaMat2 <-renderTable({
      if(is.null(s_T_grid_Mat2())) return(NULL)
      num.inputs.col1 <- 
        paste0("<input id='s_", s_T_grid_Mat2(), 
               "' class='shiny-bound-input' ",
               " type='number' max = 999 min = 0 step = 1 value=", 
               18, ">")
      out = as.data.frame(matrix(num.inputs.col1, nrow = input$s_T_subgr))
      
      t1=paste0("s_T_time", (1:input$s_T_timepoints)-1)
      t2=paste0("s_T_time", (1:input$s_T_timepoints))
      names(out) = unlist(lapply(1:input$s_T_timepoints, function(i)
        paste0(input[[t1[i]]], "-", input[[t2[i]]])))
      rownames(out) = paste0("S", 1:input$s_T_subgr)
      out
    }, sanitize.text.function = function(x) x, 
    rownames = TRUE,
    width = 'auto')
  })
})


s_T_lambdaMat2 <- reactive({
  mm = unlist(lapply(1:length(s_T_grid_Mat2()), function(i){
    input[[paste0("s_", s_T_grid_Mat2()[i])]]
  }))
  if(is.null(mm)) return(NULL)
  if(length(mm)%%input$s_T_subgr != 0) return(NULL) # Catch error when mm is not ready to be in the matrix
  matrix(fu(mm), nrow = input$s_T_subgr)
})

output$s_T_lambdaMat2_print <- renderTable({
  if(is.null(s_T_grid_Mat2())) return(NULL)
  s_T_lambdaMat2()
  }, digits = 6)


### lambdaProgMat --------------------------------------------------------------
s_T_grid_Prog <- reactive({
  grid_comb. =  expand.grid(1:input$s_T_timepoints, 1:input$s_T_subgr)
  paste0("T_Prog_s", grid_comb.$Var1, "t", grid_comb.$Var2)
})

observe({
  isolate({
    output$s_T_lambdaProgMat <-renderTable({
      if(is.null(s_T_grid_Prog())) return(NULL)
      num.inputs.col1 <- 
        paste0("<input id='s_", s_T_grid_Prog(), 
               "' class='shiny-bound-input' ",
               " type='number' max = 999 min = 0 step = 1 value=", 
               18, ">")
      out = as.data.frame(matrix(num.inputs.col1, nrow = input$s_T_subgr))
      
      t1=paste0("s_T_time", (1:input$s_T_timepoints)-1)
      t2=paste0("s_T_time", (1:input$s_T_timepoints))
      names(out) = unlist(lapply(1:input$s_T_timepoints, function(i)
        paste0(input[[t1[i]]], "-", input[[t2[i]]])))
      rownames(out) = paste0("S", 1:input$s_T_subgr)
      out
    }, sanitize.text.function = function(x) x, 
    rownames = TRUE,
    width = 'auto')
  })
})


s_T_lambdaProgMat <- reactive({
  mm = unlist(lapply(1:length(s_T_grid_Prog()), function(i){
    input[[paste0("s_", s_T_grid_Prog()[i])]]
  }))
  if(is.null(mm)) return(NULL)
  if(length(mm)%%input$s_T_subgr != 0) return(NULL) # Catch error when mm is not ready to be in the matrix
  matrix(fu(mm), nrow = input$s_T_subgr)
})

output$s_T_lambdaProgMat_print <- renderTable({
  if(is.null(s_T_grid_Prog())) return(NULL)
  s_T_lambdaProgMat()
}, digits = 6)




## Control group parameters --------------------------------------------------

# Number of subgroups ----------------------------------------------------------
observe({
  isolate({
    output$s_C_subgr_p <-renderTable({
      if (input$s_C_subgr<1) return("Number of subgroups should be > 0")
      if (input$s_C_subgr>=10) return("Number of subgroups should be < 10")
      num.inputs.col1 <- paste0("<input id='s_C_p", 1:input$s_C_subgr, 
                                "' class='shiny-bound-input' ",
                                "type='number' max = 1 min = 0 step = 0.1 value=", 
                                1/input$s_C_subgr, " />")
      out = as.data.frame(t(num.inputs.col1),
                          row.names = "Subgroup proportions")
      names(out) = paste0("S", 1:input$s_C_subgr)
      out
    },
    sanitize.text.function = function(x) x, 
    # rownames = TRUE, 
    width = '100%')
    
    output$s_C_subgr_labels <-renderTable({
      if (input$s_C_subgr<1) return("Number of subgroups should be > 0")
      if (input$s_C_subgr>=10) return("Number of subgroups should be < 10")
      num.inputs.col1 <- paste0("<input id='s_C_label", 1:input$s_C_subgr, 
                                "' class='shiny-bound-input' ",
                                "type='text' value='Subgr", 
                                1:input$s_C_subgr, "' />")
      out = as.data.frame(t(num.inputs.col1),
                          row.names = "Subgroup labels")
      names(out) = paste0("S", 1:input$s_C_subgr)
      out
    },
    sanitize.text.function = function(x) x, 
    width = '100%')
  })
})


s_C_p <- reactive({
  unlist(lapply(1:input$s_C_subgr, function(i){
    input[[paste0("s_C_p", i)]]
  }))
})
s_C_labels <- reactive({
  unlist(lapply(1:input$s_T_subgr, function(i){
    input[[paste0("s_C_label", i)]]
  }))
})


### lambdaMat1 -----------------------------------------------------------------
s_C_grid_Mat1 <- reactive({
  grid_comb. =  expand.grid(1:input$s_T_timepoints, 1:input$s_C_subgr)
  paste0("C_Mat1_s", grid_comb.$Var1, "t", grid_comb.$Var2)
})
observe({
  isolate({
    output$s_C_lambdaMat1 <-renderTable({
      if(is.null(s_C_grid_Mat1())) return(NULL)
      num.inputs.col1 <- 
        paste0("<input id='s_", s_C_grid_Mat1(), 
               "' class='shiny-bound-input' ",
               " type='number' max = 999 min = 0 step = 1 value=", 
               18, ">")
      out = as.data.frame(matrix(num.inputs.col1, nrow = input$s_C_subgr))
      
      t1=paste0("s_T_time", (1:input$s_T_timepoints)-1)
      t2=paste0("s_T_time", (1:input$s_T_timepoints))
      names(out) = unlist(lapply(1:input$s_T_timepoints, function(i)
        paste0(input[[t1[i]]], "-", input[[t2[i]]])))
      # names(out) = paste0("T", 1:input$s_T_timepoints)
      rownames(out) = paste0("S", 1:input$s_C_subgr)
      out
    }, sanitize.text.function = function(x) x, 
    rownames = TRUE,
    width = 'auto')
  })
})


s_C_lambdaMat1 <- reactive({
  input$s_C_lM1_s1t1
  mm = unlist(lapply(1:length(s_C_grid_Mat1()), function(i){
    input[[paste0("s_", s_C_grid_Mat1()[i])]]
  }))
  if(is.null(mm)) return(NULL)
  if(length(mm)%%input$s_C_subgr != 0) return(NULL) # Catch error when mm is not ready to be in the matrix
  matrix(fu(mm), nrow = input$s_C_subgr)
})

output$s_C_lambdaMat1_print <- renderTable({
  if(is.null(s_C_grid_comb())) return(NULL)
  s_C_lambdaMat1()
})

### lambdaMat2 -----------------------------------------------------------------
s_C_grid_Mat2 <- reactive({
  grid_comb. =  expand.grid(1:input$s_T_timepoints, 1:input$s_C_subgr)
  paste0("C_Mat2_s", grid_comb.$Var1, "t", grid_comb.$Var2)
})
observe({
  isolate({
    output$s_C_lambdaMat2 <-renderTable({
      if(is.null(s_C_grid_Mat2())) return(NULL)
      num.inputs.col1 <- 
        paste0("<input id='s_", s_C_grid_Mat2(), 
               "' class='shiny-bound-input' ",
               " type='number' max = 999 min = 0 step = 1 value=", 
               18, ">")
      out = as.data.frame(matrix(num.inputs.col1, nrow = input$s_C_subgr))
      
      t1=paste0("s_T_time", (1:input$s_T_timepoints)-1)
      t2=paste0("s_T_time", (1:input$s_T_timepoints))
      names(out) = unlist(lapply(1:input$s_T_timepoints, function(i)
        paste0(input[[t1[i]]], "-", input[[t2[i]]])))
      # names(out) = paste0("T", 1:input$s_T_timepoints)
      rownames(out) = paste0("S", 1:input$s_C_subgr)
      out
    }, sanitize.text.function = function(x) x, 
    rownames = TRUE,
    width = 'auto')
  })
})


s_C_lambdaMat2 <- reactive({
  mm = unlist(lapply(1:length(s_C_grid_Mat2()), function(i){
    input[[paste0("s_", s_C_grid_Mat2()[i])]]
  }))
  if(is.null(mm)) return(NULL)
  if(length(mm)%%input$s_C_subgr != 0) return(NULL) # Catch error when mm is not ready to be in the matrix
  matrix(fu(mm), nrow = input$s_C_subgr)
})

output$s_C_lambdaMat2_print <- renderTable({
  if(is.null(s_C_grid_Mat2())) return(NULL)
  s_C_lambdaMat2()
})


### lambdaProgMat --------------------------------------------------------------
s_C_grid_Prog <- reactive({
  grid_comb. =  expand.grid(1:input$s_T_timepoints, 1:input$s_C_subgr)
  paste0("C_Prog_s", grid_comb.$Var1, "t", grid_comb.$Var2)
})

observe({
  isolate({
    output$s_C_lambdaProgMat <-renderTable({
      if(is.null(s_C_grid_Prog())) return(NULL)
      num.inputs.col1 <- 
        paste0("<input id='s_", s_C_grid_Prog(), 
               "' class='shiny-bound-input' ",
               " type='number' max = 999 min = 0 step = 1 value=", 
               18, ">")
      out = as.data.frame(matrix(num.inputs.col1, nrow = input$s_C_subgr))
      
      t1=paste0("s_T_time", (1:input$s_T_timepoints)-1)
      t2=paste0("s_T_time", (1:input$s_T_timepoints))
      names(out) = unlist(lapply(1:input$s_T_timepoints, function(i)
        paste0(input[[t1[i]]], "-", input[[t2[i]]])))
      # names(out) = paste0("T", 1:input$s_T_timepoints)
      rownames(out) = paste0("S", 1:input$s_C_subgr)
      out
    }, sanitize.text.function = function(x) x, 
    rownames = TRUE,
    width = 'auto')
  })
})


s_C_lambdaProgMat <- reactive({
  mm = unlist(lapply(1:length(s_C_grid_Prog()), function(i){
    input[[paste0("s_", s_C_grid_Prog()[i])]]
  }))
  if(is.null(mm)) return(NULL)
  if(length(mm)%%input$s_C_subgr != 0) return(NULL) # Catch error when mm is not ready to be in the matrix
  matrix(fu(mm), nrow = input$s_C_subgr)
})

output$s_C_lambdaProgMat_print <- renderTable({
  if(is.null(s_C_grid_Prog())) return(NULL)
  s_C_lambdaProgMat()
}, digits = 8)



### Analysis -------------------------------------------------------------------


#Calculation of the hazard and survival functions.
#The hazard rates per day before and after PD and the hazard for PD are defined in lambdaMat1, lambdaMat2 and lambdaProgMat
#The matrices contain one row per subgroup and one column per time interval

B5 <- reactive({
  pop_hazVFfun(
    Tint = s_T_times(),
    lambdaMat1 = s_T_lambdaMat1(),
    lambdaMat2 = s_T_lambdaMat2(),
    lambdaProgMat = s_T_lambdaProgMat(),
    p = s_T_p(),
    timezero = FALSE
  )
})

K5 <- reactive({
  pop_hazVFfun(
    Tint = s_T_times(),
    lambdaMat1 = s_C_lambdaMat1(),
    lambdaMat2 = s_C_lambdaMat2(),
    lambdaProgMat = s_C_lambdaProgMat(),
    p = s_C_p(),
  timezero = TRUE
) #Setting timezero=TRUE here means we have the same delayed onset for switchers after progression as we have initially for patients in the treatment group.
})



# Create Plot ------------------------------------------------------------------
react_panplot = eventReactive(input$s_calculate,{
  print(plot_shhr(A = K5(), B = B5()))
})

output$panplot = renderPlot(react_panplot())

output$ctrl_plot = renderPlot({
  plot_diagram(A = K5(),
               B = K5(), 
               s_C_labels(), 
               s_C_labels(), 
               which = "Control")
})
output$trt_plot = renderPlot({
  plot_diagram(A = B5(),
               B = B5(), 
               s_T_labels(), 
               s_T_labels(), 
               which = "Experimental")
})


# Calculate medians ------------------------------------------------------------
k_median = reactive({
  K = K5()
  pos_below = suppressWarnings(min(K$t[(K$S <=  0.5)]) + 1)
  pos_avobe = max(K$t[(K$S >=  0.5)]) + 1
  dd = K$S[pos_avobe] - K$S[pos_below] 
  ((pos_avobe-1)*abs(K$S[pos_below] - 0.5) + (pos_below-1)*abs(K$S[pos_avobe] - 0.5))/dd
})
b_median = reactive({
  K = B5()
  pos_below = suppressWarnings(min(K$t[(K$S <=  0.5)]) + 1)
  pos_avobe = max(K$t[(K$S >=  0.5)]) + 1
  dd = K$S[pos_avobe] - K$S[pos_below] 
  ((pos_avobe-1)*abs(K$S[pos_below] - 0.5) + (pos_below-1)*abs(K$S[pos_avobe] - 0.5))/dd
})

median_table = eventReactive(input$s_calculate,
 data.frame(group = c("Experimental", "Control"),
            median = c(b_median(), k_median()))
)
output$median_table = renderTable({
  median_table()
},
caption = "<h4 style='text-align:center;color:black;'><b>Median survival times by treatment group</b></h4>", 
caption.placement = getOption("xtable.caption.placement", "top"), 
caption.width = getOption("xtable.caption.width", NULL))





# Download code ----------------------------------------------------------------
output$s_downloadCode <- downloadHandler(
  filename = paste('nph-simulate-rcode.r'),
  content = function(file) {
    
    input1 = list()
    input1$times = 1
    input1$lambda = 2
    a = sprintf(
      "################################################################################
       ## nph package - Reproducible R code from shiny app.

library('nph')

times = %s                          # Define timebreaks for intervals

# Create piecewise constant hazard objects for Treatment and Control arms
# The lambda matrices are given in terms of the median survival times,
# Therefore we use the nph::m2r function to translate from median to rate
TRT <- pop_pchaz(
    Tint = times,
    lambdaMat1 = %s,   
    lambdaMat2 = %s,   
    lambdaProgMat = %s, 
    p = %s,
    timezero = FALSE, 
  discrete_approximation = TRUE
)

CRL <- pop_pchaz(
    Tint = times,
    lambdaMat1 = %s, 
    lambdaMat2 = %s,
    lambdaProgMat = %s, 
    p = %s,
  timezero = TRUE, 
  discrete_approximation = TRUE
)

print(TRT)
print(CRL)


# Creat state diagrams
TRT_subgr_labels = %s
CRL_subgr_labels = %s
plot_diagram(A = TRT,
             B = CRL, 
             A_subgr_labels = TRT_subgr_labels,  
             B_subgr_labels = CRL_subgr_labels, 
             which = 'Control')

plot_diagram(A = TRT,
             B = CRL, 
             A_subgr_labels = TRT_subgr_labels,  
             B_subgr_labels = CRL_subgr_labels, 
             which = 'Experimental')


# Create Survival, Hazard and hazard ratio plots
plot_shhr(A = TRT, B = CRL)


", 
    sprintf("c(%s)", paste0(s_T_times(), collapse = ", ")),   # The time intervals
    sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaMat1()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
    sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaMat2()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
    sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaProgMat()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
    sprintf("c(%s)", paste0(s_T_p(), collapse = ", ")),       # The proportions in the subgroups for treatment arm
    sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaMat1()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
    sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaMat2()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
    sprintf("m2r(matrix(c(%s), nrow = %s, ncol = %s))", paste0(m2r(s_T_lambdaProgMat()), collapse = ", "), input$s_T_subgr, input$s_T_timepoints),      
    sprintf("c(%s)", paste0(s_C_p(), collapse = ", ")),       # The proportions in the subgroups for control arm
    sprintf("c(%s)", paste0(paste0("'", s_T_labels(), "'"), collapse = ", ")),       # The proportions in the subgroups for control arm
    sprintf("c(%s)", paste0(paste0("'", s_C_labels(), "'"), collapse = ", "))       # The proportions in the subgroups for control arm
    )
    
    owd <- setwd(tempdir())
    on.exit(setwd(owd))
    
    capture.output(cat(a), file = paste0("test.R"))
    # tidy_file(file = "test.R", width.cutoff = 80)   # this is to format nicely the R code https://yihui.name/formatr/
    styler::style_file(path = "test.R", scope = "indention")  # this is to format nicely the R code
    
    out <- paste0("test.R")
    file.rename(out, file)
  }
)

nph.env<-new.env()

library(ggplot2)
library(dplyr)
library(formatR)

#library(nph)
server <- shinyServer(function(input, output, session) {
  options(stringsAsFactors = F)
  # # include logic for each tab
  source(file.path("server", "srv-tab-home.R"),    local = TRUE)$value
  source(file.path("server", "srv-tab-simulate.R"),   local = TRUE)$value
  source(file.path("server", "srv-tab-power.R"),local = TRUE)$value
  source(file.path("server", "srv-tab-condpower.R"),local = TRUE)$value
  source(file.path("server", "helper_functions.R"),local = TRUE)$value
  source(file.path("server", "paket_nph_1-9.R"),local = TRUE)$value
  source(file.path("server", "additional_functions.R"),local = TRUE)$value
  source(file.path("server", "continuous_functions.R"),local = TRUE)$value
  


})

## haz_fun is a function that generates a hazard function
# haz_fun <- function(Tint, lambda){
#   hf = approxfun(x = Tint, y = c(lambda),
#                  method = "constant", 
#                  yleft = 0, yright = lambda[length(lambda)], 
#                  f = 0, ties = "ordered")
#   class(hf) = "stepfun"
#   hf
# }
haz_fun <- function(Tint, lambda){
  function(v) {
    mapply(function(vv){
      Tp0 = c(-Inf, Tint)
      lambda0 = c(0, lambda)
      index = max(which(vv >= Tp0))
      lambda0[index]
    }, v)}
}
# hf = haz_fun(Tint = c(0, 40), lambda = c(.02,.05))
# hf
# # plot(hf, verticals = FALSE, pch = 1)
# hf(0)
# hf(0:100)

# Cumulative hazard function -
## cumhaz_fun is a function that generates a Cumulative hazard function
cumhaz_fun <- function(Tint, lambda){
  function(v) {
    mapply(function(vv){
      Tp1 = c(Tint[-1], Inf)
      Tp1 = ifelse(vv > Tp1, Tp1, vv)
      d = (Tp1-Tint)
      sum(d*lambda*(d>0))
    }, v)}
}
# chf = cumhaz_fun(Tint = c(0, 40), lambda = c(.02,.05))
# chf
# plot(chf, pch = 1)
# chf(0)
# chf(100)
# plot(chf, pch = 1, from = 0, to = 100)
# lines(1:100, chf(1:100), pch = 1, col = 2)

# Survival function -
## surv_fun is a function that generates a survival function
surv_fun <- function(Tint, lambda){
  chf = function(v) {
    mapply(function(vv){
      Tp1 = c(Tint[-1], Inf)
      Tp1 = ifelse(vv > Tp1, Tp1, vv)
      d = (Tp1-Tint)
      d*lambda*(d>0)
      exp(-sum(d*lambda*(d>0)))}, v)}
  # class(chf) = "stepfun"
  chf
}

# Cumulative distribution function -
cdf_fun <- function(Tint, lambda){
  chf = function(v) {
    mapply(function(vv){
      Tp1 = c(Tint[-1], Inf)
      Tp1 = ifelse(vv > Tp1, Tp1, vv)
      d = (Tp1-Tint)
      d*lambda*(d>0)
      1-exp(-sum(d*lambda*(d>0)))}, v)}
  # class(chf) = "stepfun"
  chf
}

# distribution function -
pdf_fun <- function(Tint, lambda){
  hf = haz_fun(Tint = Tint, lambda = lambda)
  chf = function(v) {
    mapply(function(vv){
      Tp1 = c(Tint[-1], Inf)
      Tp1 = ifelse(vv > Tp1, Tp1, vv)
      d = (Tp1-Tint)
      d*lambda*(d>0)
      ee = sum(d*lambda*(d>0))
      exp(-ee)*hf(vv)}, # h(t) = f(t) / (1-F(t)), then f(t) = (1-F(t)) * h(t)
      v)}
  # class(chf) = "stepfun"
  chf
}

## Internal function to calculate all functions at once
internal_pchaz <- function(Tint, lambda){
  funs_T   = Tint[-length(Tint)]          # For creating the functions, we dont need the last value of T as we assume the hazard is constant until \infty. 
  haz_f    = haz_fun(Tint = funs_T, lambda = lambda)     # Creates the hazard function
  cumhaz_f = cumhaz_fun(Tint = funs_T, lambda =lambda)  # Creates the cumhazard function
  surv_f   = surv_fun(Tint = funs_T, lambda = lambda)    # Creates the survival function
  cdf_f    = cdf_fun(Tint = funs_T, lambda = lambda)     # Creates the cdf 
  pdf_f    = pdf_fun(Tint = funs_T, lambda = lambda) # Creates the pdf 
  out <- list(haz_f    = haz_f,  cumhaz_f = cumhaz_f, 
              surv_f   = surv_f, cdf_f    = cdf_f, 
              pdf_f    = pdf_f)
  out
}

#' @title Calculate survival for piecewise constant hazard
#'
#' @description Calculates hazard, cumulative hazard, survival and distribution function
#' 	based on hazards that are constant over pre-specified time-intervals. 
#' 	
#' @param Tint vector of length \eqn{k+1}, for the boundaries of \eqn{k} time intervals (presumably in days) with piecewise constant hazard. The boundaries should be increasing and the first one should
#'	be \code{0}, the last one should be larger than the assumed trial duration.
#' @param lambda vector of length \eqn{k} with the piecewise constant hazards for the intervals specified via \code{Tint}. 
#' 
#' @details Given \eqn{k} time intervals \eqn{[t_{j-1},t_j), j=1,\dots,k} with 
#'  \eqn{0 =  t_0 < t_1 \dots < t_k}, the function assume constant hazards \eqn{\lambda_{j}} at each interval.
#'  The resulting hazard function is
#'  \eqn{\lambda(t) =\sum_{j=1}^k \lambda_{j} {1}_{t \in [t_{j-1},t_j)}},
#'  the cumulative hazard function is\\ 
#'  \eqn{\Lambda(t) = \int_0^t \lambda(s) ds =\sum_{j=1}^k \left( (t_j-t_{j-1})\lambda_{j} {1}_{t > t_j} + (t-t_{j-1}) \lambda_{j} {1}_{t \in [t_{j-1},t_j) } \right)} 
#'  and the survival function \eqn{S(t) = e^{-\Lambda(t)}}.
#'  The output includes the functions values calculated for all integer time points
#'  between 0 and the maximum of \code{Tint}.
#'  Additionally, a list with functions is also given to calculate the values at any arbitrary point \eqn{t}.
#' 
#' @return A list with class \code{mixpch} containing the following components:
#' \describe{
#'	\item{\code{haz}}{Values of the hazard function over discrete times t.}
#'	\item{\code{cumhaz}}{Values of the cumulative hazard function over discrete times t.}
#'	\item{\code{S}}{Values of the survival function over discrete times t.}
#'	\item{\code{F}}{Values of the distribution function over discrete times t.}
#'	\item{\code{t}}{Time points for which the values of the different functions are calculated.}
#'	\item{\code{Tint}}{Input vector of boundaries of time intervals.}
#'	\item{\code{lambda}}{Input vector of piecewise constant hazards.}
#'	\item{\code{funs}}{A list with functions to calculate the hazard, cumulative hazard, survival, pdf and cdf over arbitrary continuous times.}
#' }
#' 
#' @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}, Nicolas Ballarini
#' @seealso \code{\link{subpop_pchaz}}, \code{\link{pop_pchaz}}, \code{\link{plot.mixpch}}
#' @references
#' Robin Ristl, Nicolas Ballarini, Heiko Götte, Armin Schüler, Martin Posch, Franz König. Delayed treatment effects, treatment switching and 
#' heterogeneous patient populations: How to design and analyze RCTs in oncology. Pharmaceutical statistics. 2021; 20(1):129-145.
#' @examples
#' pchaz(Tint = c(0, 40, 100), lambda=c(.02, .05))
#Das heiszt ab inklusive Tag 0 gilt lambda=0.02, ab inklusive Tag 40 gilt lambda 0.05
#Wenn das Event am Tag 40 mit dem Hazard 0.05 eintritt, wird es am Tag 41 registriert.
#' @export
pchaz <- function(Tint, lambda){
  if (length(Tint) != (length(lambda) + 1)){
    stop("The length of Tint should be equal to the length of lambda + 1")
  }
  if (any(lambda < 0)) {
    stop("Only positive values are allowed for the hazard rates")
  }
  if (any(diff(Tint) <= 0)) {
    stop("Tint should be non-decreasing")
  }
  
  call <- match.call()
  t        = seq(0, max(Tint) - 1, 1)  # We define a vector to discretize time 
  funs = internal_pchaz(Tint, lambda)
  
  haz      = funs$haz_f(t)
  cumhaz   = funs$cumhaz_f(t)
  surv     = funs$surv_f(t)
  cdf      = funs$cdf_f(t)
  pdf      = funs$pdf_f(t)
  
  out<-list(haz = haz, cumhaz = cumhaz, S = surv, F = cdf, f = pdf, 
            t=t, Tint = Tint, lambda = lambda, 
            call = call,
            funs = funs)
  class(out)<-"mixpch" #damit wir die plot.mixpch Funktion verwenden koennen
  out
}

# hvfffun = pchaz(Tint = c(0, 40, 100), lambda = c(.02,.05))
# library(nph)
# hvfffun2 = hazVFfun(Tint = c(0, 40, 100), lambda = c(.02,.05))
# plot(hvfffun2, fun = "haz")
# plot(hvfffun,  fun = "haz", add = TRUE, col = "red")
# plot(hvfffun$haz, hvfffun2$haz)
# plot(hvfffun2, fun = "cumhaz")
# plot(hvfffun,  fun = "cumhaz", add = TRUE, col = "red")
# plot(hvfffun2, fun = "S")
# plot(hvfffun,  fun = "S", add = TRUE, col = "red")
# plot(hvfffun2, fun = "F")
# plot(hvfffun,  fun = "F", add = TRUE, col = "red")


## Internal function to calculate all functions at once
internal_subpop_pchaz<-function(Tint, lambda1, lambda2, lambdaProg, timezero = FALSE,
                                int_control) {
  hVF_Tod <- internal_pchaz(Tint, lambda1)
  hVF_TodnachSwitch <- internal_pchaz(Tint, lambda2)
  hVF_Switch <- internal_pchaz(Tint, lambdaProg)
  
  # We first create haz, cumhaz, surv, cdf and pdf functions
  if(!timezero){
    int1 = function(s, t) {
      t1 = hVF_Tod$cumhaz_f(min(s, t))
      t2 = (hVF_TodnachSwitch$cumhaz_f(t) - hVF_TodnachSwitch$cumhaz_f(s)) * (s < t)
      exp(-(t1 + t2)) * hVF_Switch$pdf_f(s)
    }
  } else{
    int1 = function(s, t) {
      t1 = hVF_Tod$cumhaz_f(min(s, t))
      t2 = (hVF_TodnachSwitch$cumhaz_f(t - s)) * (s < t)
      exp(-(t1 + t2)) * hVF_Switch$pdf_f(s)
    }
  }
  int1_vec = Vectorize(int1)
  
  # Mix of Survival functions
  Smix = function(v) {
    mapply(function(x){
      Ss = integrate(f = int1_vec, lower = 0, upper = Inf, t = x, 
                     rel.tol = int_control$rel.tol, abs.tol = int_control$abs.tol)$value
      Ss
    }, v)
  }
  
  # Mix of CDF functions
  Fmix = function(v) {
    mapply(function(x){
      Ss = integrate(f = int1_vec, lower = 0, upper = Inf, t = x, 
                     rel.tol = int_control$rel.tol, abs.tol = int_control$abs.tol)$value
      1 - (Ss)}, v)
  }
  
  # Mix of Cumulative hazards functions
  cummixhaz = function(v) {
    mapply(function(x){
      Ss = integrate(f = int1_vec, lower = 0, upper = Inf, t = x, 
                     rel.tol = int_control$rel.tol, abs.tol = int_control$abs.tol)$value
      -log(Ss)}, v)
  }
  
  if(!timezero){
    int2 = function(s, t) {
      t1 = hVF_Tod$cumhaz_f(min(s, t))
      t2 = (hVF_TodnachSwitch$cumhaz_f(t) - hVF_TodnachSwitch$cumhaz_f(s)) * (s < t)
      exp(-(t1 + t2)) * hVF_Switch$pdf_f(s) * 
        (hVF_Tod$haz_f(t) * (s>t) + hVF_TodnachSwitch$haz_f(t) * (s<t))
    }
  } else {
    int2 = function(s, t) {
      t1 = hVF_Tod$cumhaz_f(min(s, t))
      t2 = (hVF_TodnachSwitch$cumhaz_f(t - s)) * (s < t)
      exp(-(t1 + t2)) * hVF_Switch$pdf_f(s) * 
        (hVF_Tod$haz_f(t) * (s>t) + hVF_TodnachSwitch$haz_f(t-s) * (s<t))
    }
  }
  int2_vec = Vectorize(int2)
  
 # Mix of hazard functions
 hmix = function(v) {
    mapply(function(x){
      invS = (1/Smix(x))
      Ss = integrate(f = int2_vec, lower = 0, upper = Inf, t = x, 
                     rel.tol = int_control$rel.tol, abs.tol = int_control$abs.tol)$value
      invS * Ss
    }, v)
  }
  
  out<-list(haz_f    = hmix, cumhaz_f = cummixhaz, 
            surv_f   = Smix, cdf_f    = Fmix)
  out
}



#' @title Calculate survival for piecewise constant hazards with change after random time
#'
#' @description Calculates hazard, cumulative hazard, survival and distribution function
#' 	based on hazards that are constant over pre-specified time-intervals
#' 	
#' @param Tint vector of length \eqn{k+1}, for the boundaries of \eqn{k} time intervals (presumably in days) with piecewise constant hazard. The boundaries should be increasing and the first one should
#'	be \code{0}, the last one should be larger than the assumed trial duration.
#' @param lambda1 vector of length \eqn{k} for piecewise constant hazards before the changing event happens, for the intervals specified via \code{T}. 
#' @param lambda2 vector of length \eqn{k} for piecewise constant hazards after the changing event has happened, for the intervals specified via \code{T}. 
#' @param lambdaProg vector of length \eqn{k} for piecewise constant hazards for the changing event, for the intervals specified via \code{T}. 
#' @param timezero logical, indicating whether after the changeing event the timecount, governing which interval in \code{Tint} and which according value in 
#'   \code{lambda2} is used, should restart at zero.
#' @param int_control A list with the \code{rel.tol} and \code{abs.tol} paramaters to be passed to the  \code{\link{integrate}} function. 
#' @param discrete_approximation if TRUE, the function uses an approximation based on discretizing the time, instead of integrating. This speeds up the calculations
#'   
#' @details We assume that the time to disease progression \eqn{T_{PD}} is governed
#'  by a separate process with hazard function \eqn{\eta(t)}, 
#'  which does not depend on the hazard function for death \eqn{\lambda(t)}. 
#'  \eqn{\eta(t)}, too, may be modelled as piecewise constant or, for simplicity, 
#'  as constant over time. We define \eqn{\lambda_{prePD}(t)} and \eqn{\lambda_{postPD}(t)} 
#'  as the hazard functions for death before and after disease progression. 
#'  Conditional on \eqn{T_{PD}=s}, the hazard function for death is 
#'  \eqn{\lambda(t|T_{PD}=s)=\lambda_{prePD}(t){I}_{t\leq s}+\lambda_{postPD}(t){I}_{t>s}}.
#'  The conditional survival function is 
#'  \eqn{S(t|T_{PD}=s)=\exp(-\int_0^t \lambda(t|T_{PD}=s)ds)}. 
#'  The unconditional survival function results from integration over all 
#'  possible progression times as \eqn{S(t)=\int_0^t S(t|T_{PD}=s)dP(T_{PD}=s)}. 
#'  The output includes the function values calculated for all integer time points
#'  between 0 and the maximum of \code{Tint}.
#'  Additionally, a list with functions is also given to calculate the values at any arbitrary point \eqn{t}.
#' 
#' @return  A list with class \code{mixpch} containing the following components:
#' \describe{
#'	\item{\code{haz}}{Values of the hazard function.}
#'	\item{\code{cumhaz}}{Values of the cumulative hazard function.}
#'	\item{\code{S}}{Values of the survival function.}
#'	\item{\code{F}}{Values of the distribution function.}
#'	\item{\code{t}}{Time points for which the values of the different functions are calculated.}
#'	\item{\code{Tint}}{Input vector of boundaries of time intervals.}
#'	\item{\code{lambda1}}{Input vector of piecewise constant hazards before the changing event happen.}
#'	\item{\code{lambda2}}{Input vector of piecewise constant hazards after the changing event happen.}
#'	\item{\code{lambdaProg}}{Input vector of piecewise constant hazards for the changing event.}
#'	\item{\code{funs}}{A list with functions to calculate the hazard, cumulative hazard, survival, and cdf over arbitrary continuous times.}
#' }
#' @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}, Nicolas Ballarini
#' @seealso \code{\link{pchaz}}, \code{\link{pop_pchaz}}, \code{\link{plot.mixpch}}
#' @references
#' Robin Ristl, Nicolas Ballarini, Heiko Götte, Armin Schüler, Martin Posch, Franz König. Delayed treatment effects, treatment switching and 
#' heterogeneous patient populations: How to design and analyze RCTs in oncology. Pharmaceutical statistics. 2021; 20(1):129-145.
#' @examples
#' subpop_pchaz(Tint = c(0, 40, 100), lambda1 = c(0.2, 0.4), lambda2 = c(0.1, 0.01),
#'	lambdaProg = c(0.5, 0.4),timezero = FALSE, discrete_approximation = TRUE)
#' subpop_pchaz(Tint = c(0, 40, 100), lambda1 = c(0.2, 0.4), lambda2 = c(0.1, 0.01),
#'	lambdaProg = c(0.5, 0.4), timezero = TRUE, discrete_approximation = TRUE)
#' @export
subpop_pchaz<-function(Tint, lambda1, lambda2, lambdaProg, timezero = FALSE, 
                       int_control = list(rel.tol = .Machine$double.eps^.4, 
                                          abs.tol = 1e-9),
                       discrete_approximation = FALSE) {
  call <- match.call()
  
  if (any(diff(Tint) <= 0)) {
    stop("Tint should be non-decreasing")
  }
  if (discrete_approximation){
    out = subpop_hazVFfun(Tint, lambda1, lambda2, lambdaProg, timezero)
    return(out)
  }
  
  t <- seq(0, max(Tint) - 1, 1)
  if(!all(c("rel.tol", "abs.tol") %in% names(int_control))){
    int_control = list(rel.tol = .Machine$double.eps^.4, 
                       abs.tol = 1e-9)
  }
  # Handle special cases where lambda progr is 0 or Inf (progression is not considered)
  if(all(lambdaProg == Inf)){
    funs = internal_pchaz(Tint, lambda2)
  } else if(all(lambdaProg == 0)){
    funs = internal_pchaz(Tint, lambda1)
  } else {
    funs = internal_subpop_pchaz(Tint, lambda1, lambda2, lambdaProg, timezero, int_control)
  }
  
  haz = funs$haz_f(t)
  cumhaz = funs$cumhaz_f(t)
  S = funs$surv_f(t)
  F = funs$cdf_f(t)
  
  out <- list(haz = haz, cumhaz = cumhaz, S = S, F = F, 
              t=t, Tint = Tint, 
              lambda1 = lambda1, lambda2 = lambda2, lambdaProg = lambdaProg,  timezero = timezero,
              call = call,
              funs = funs)
  
  class(out) <- c("mixpch", "subpop") #mix of piecewise constantant hazard
  out
}
# a1 = subpop_hazVFfun(Tint=c(0,40,100),
#                      lambda1=c(0.2,0.4),
#                      lambda2=c(0.1,0.01),
#                      lambdaProg=c(0.5,0.4),timezero=FALSE)
# a2 = subpop_pchaz(Tint=c(0,40,100),
#                   lambda1=c(0.2,0.4),
#                   lambda2=c(0.1,0.01),
#                   lambdaProg=c(0.5,0.4),
#                   timezero=FALSE)
# a2$haz
# plot(a1)
# plot(a2, add = TRUE, col = 2)
# str(a2)


#' @title Calculate survival for piecewise constant hazards with change after random time and mixture of subpopulations
#'
#' @description Calculates hazard, cumulative hazard, survival and distribution function
#' 	based on hazards that are constant over pre-specified time-intervals
#' 	
#' @param Tint vector of length \eqn{k+1}, for the boundaries of \eqn{k} time intervals (presumably in days) with piecewise constant hazard. The boundaries should be increasing and the first one should
#'	be \code{0}, the last one should be larger than the assumed trial duration.
#' @param lambdaMat1 matrix of dimension \eqn{m}-by-\eqn{k}, each row contains the vector of piecewise constant hazards for one subpopulation before the changeing event happens, for the intervals speciefied via \code{Tint}. 
#' @param lambdaMat2 matrix of dimension \eqn{m}-by-\eqn{k}, each row contains the vector piecewise constant hazards for one subpopulation after the changeing event has happened, for the intervals speciefied via \code{Tint}. 
#' @param lambdaProgMat matrix of dimension \eqn{m}-by-\eqn{k}, each row contains the vector of piecewise constant hazards for one subpopulation for the changeing event, for the intervals speciefied via \code{Tint}. 
#' @param p vector of length \eqn{m} for relative sizes (proportions) of the subpopulations. They should sum up to 1.
#' @param timezero logical, indicating whether after the changing event the timecount, governing which interval in \code{Tint} and which according value in 
#'   \code{lambda2} is used, should restart at zero. This argument is either of length 1 (applying the same to all subgroups) or the same length as the number of subgroups.
#' @param int_control A list with additional paramaters to be passed to the  \code{\link{integrate}} function. 
#' @param discrete_approximation if TRUE, the function uses an approximation based on discretizing the time, instead of integrating. This speeds up the calculations
#'   
#' @details Given \eqn{m} subgroups with relative sizes \eqn{p_1, \dots, p_m} and 
#' subgroup-specific survival functions \eqn{S{l}(t)}, 
#' the marginal survival function is the mixture \eqn{S(t)=\sum_{l=1}^m p_l S_{l}(t)}.
#' Note that the respective hazard function is not a linear combination of the 
#' subgroup-specific hazard functions. 
#' It may be calculated by the general relation \eqn{\lambda(t)=-\frac{dS(t)}{dt}\frac{1}{S(t)}}.
#' In each subgroup, the hazard is modelled as a piecewise constant hazard, with
#' the possibility to also model disease progression. 
#' Therefore, each row of the hazard rates is used in \code{\link{subpop_pchaz}}. 
#' See \code{\link{pchaz}} and \code{\link{subpop_pchaz}}
#' for more details.
#' The output includes the function values calculated for all integer time points
#'  between 0 and the maximum of \code{Tint}.
#' 
#' Note: this function may be very slow in cases where many time points need to be calculated. If this happens, use
#' \code{discrete_approximation = TRUE}. 
#' 
#' @return A list with class \code{mixpch} containing the following components:
#' \describe{
#'	\item{\code{haz}}{Values of the hazard function.}
#'	\item{\code{cumhaz}}{Values of the cumulative hazard function.}
#'	\item{\code{S}}{Values of the survival function.}
#'	\item{\code{F}}{Values of the distribution function.}
#'	\item{\code{t}}{Time points for which the values of the different functions are calculated.}
#' }
#' @author Robin Ristl, \email{robin.ristl@@meduniwien.ac.at}, Nicolas Ballarini
#' @seealso \code{\link{pchaz}}, \code{\link{subpop_pchaz}}, \code{\link{plot.mixpch}}
#' @references
#' Robin Ristl, Nicolas Ballarini, Heiko Götte, Armin Schüler, Martin Posch, Franz König. Delayed treatment effects, treatment switching and 
#' heterogeneous patient populations: How to design and analyze RCTs in oncology. Pharmaceutical statistics. 2021; 20(1):129-145.
#' @examples
#' pop_pchaz(Tint = c(0, 40, 100),
#'   lambdaMat1 = matrix(c(0.2, 0.1, 0.4, 0.1), 2, 2),
#'	 lambdaMat2 = matrix(c(0.5, 0.2, 0.6, 0.2), 2, 2),
#'	 lambdaProg = matrix(c(0.5, 0.5, 0.4, 0.4), 2, 2),
#'	 p = c(0.8, 0.2),
#'	 timezero = FALSE, discrete_approximation = TRUE)
#' @importFrom stats integrate
#' @export
pop_pchaz <- function(Tint, lambdaMat1, lambdaMat2, lambdaProgMat, p, timezero=FALSE, 
                      int_control = list(rel.tol = .Machine$double.eps^.4, 
                                         abs.tol = 1e-9),
                      discrete_approximation = FALSE) {
  call <- match.call()
  if (!identical(dim(as.matrix(lambdaMat1)), dim(as.matrix(lambdaMat2)), dim(as.matrix(lambdaProgMat)))) {
    stop("Dimensions of all lambda matrices should be the same")
  }
  if (any(diff(Tint) <= 0)) {
    stop("Tint should be non-decreasing")
  }
  if (any(c(lambdaMat1, lambdaMat2, lambdaProgMat) < 0)) {
    stop("Only positive values are allowed for the hazard rates")
  }
  # Now we are sure lambdaMat1,lambdaMat2,lambdaProgMat have the same dimensions
  # We can compare only one to Tint and p
  if ((length(Tint)-1) != ncol(as.matrix(lambdaMat1))){
    stop("The length of Tint should be equal to the number of column of the lambda matrices + 1")
  }
  if (length(p) != nrow(as.matrix(lambdaMat1))){
    stop("The length of p should be equal to the number of column of the lambda matrices")
  }  
  if (abs(sum(p)-1) > .Machine$double.eps^0.5){
    stop("The prevalences in p should sum up to 1")
  }  
  if (length(timezero) == 1){
    timezero = rep(timezero, length(p))
  }
  if (length(timezero) != length(p)){
    stop("timezero should be of length 1 or the same length as p")
  }
  
  if(discrete_approximation){
    out = pop_hazVFfun(Tint, lambdaMat1, lambdaMat2, lambdaProgMat, p, timezero)
    
    return(out)
  }
  
  if(!all(c("rel.tol", "abs.tol") %in% names(int_control))){
    int_control = list(rel.tol = .Machine$double.eps^.4, 
                       abs.tol = 1e-9)
  }
  t <- seq(0, max(Tint) - 1, 1)
  
  #wichtig: damit cumsum fuer kumulativen Hazard genau stimmt:
  nsubpop <- length(p)
  Fmix <- rep(0,length(t))
  for(i in 1:nsubpop) {
    lambdaProg = lambdaProgMat[i,]
    lambda1 = lambdaMat1[i,]
    lambda2 = lambdaMat2[i,]
    if(all(lambdaProg == Inf)){
      funs = internal_pchaz(Tint, lambda2)
    } else if(all(lambdaProg == 0)){
      funs = internal_pchaz(Tint, lambda1)
    } else {
      funs = internal_subpop_pchaz(Tint, lambda1, lambda2, lambdaProg, timezero[i], int_control)
    }
    Fmix <- Fmix + p[i]*funs$cdf_f(t)
  }
  Smix <- 1 - Fmix
  cummixhaz <- -log(Smix)
  mixhaz <- c(diff(cummixhaz), NA)
  #hazmix<-fmix/Smix #Achtung, wenn Smix nahe an 0 ist, wird das numerisch ungenau
  out <- list(haz = mixhaz, cumhaz = cummixhaz, S = Smix, F = Fmix,
              t=t, Tint = Tint, 
              lambdaMat1 = lambdaMat1, lambdaMat2 = lambdaMat2, 
              lambdaProgMat = lambdaProgMat, p = p, timezero = timezero,
              call = call)
  class(out) <- c("mixpch", "pop") #mix of piecewise constantant hazard
  out
}
# a1 = pop_hazVFfun(Tint=c(0,40,100),lambdaMat1=matrix(c(0.2,0.1,0.4,0.1),2,2),
#              lambdaMat2=matrix(c(0.5,0.2,0.6,0.2),2,2),
#              lambdaProg=matrix(c(0.5,0.5,0.4,0.4),2,2),p=c(0.8,0.2),timezero=FALSE)
# a2 = pop_pchaz(Tint=c(0,40,100),lambdaMat1=matrix(c(0.2,0.1,0.4,0.1),2,2),
#              lambdaMat2=matrix(c(0.5,0.2,0.6,0.2),2,2),
#              lambdaProg=matrix(c(0.5,0.5,0.4,0.4),2,2),p=c(0.8,0.2),timezero=FALSE)
# 
# 
# plot(a1)
# plot(a1, add = TRUE, col = 2)
# plot(a1, fun = "cumhaz")
# plot(a1, fun = "cumhaz", add = TRUE, col = 2)

#' @export
print.mixpch = function(x, ...){
  m1 = "pchaz object: piecewise constant hazard"
  if(class(x)[2] %in% "subpop") m1 = sprintf("%s, with change after random time", m1)
  if(class(x)[2] %in% "pop")    m1 = sprintf("%s, with change after random time and mixture of subpopulations", m1)
  
  tt = c("Time intervals: ",
         paste0(rep("|", length(x$Tint)), collapse = "-----"), 
         "\n           ",
         paste0(sprintf("%6.f", x$Tint), collapse = ""), "\n\n")
  
  ss = ""
  if(class(x)[2] %in% "pop"){
    ss = paste0("Number of subgroups: ", length(x$p))
    if(length(x$p)>1) ss = sprintf("%s, with prevalences: %s", 
                                   ss,
                                   paste0(x$p, collapse = " - "))
  }
  
  overall_median = which.min(abs(x$F - 0.5))+1
  if (overall_median == max(x$Tint)) overall_median = NA
  mm = c("Median survival time: ", overall_median)
  
  cat(m1, "\n\n")
  cat(tt)
  cat(ss, "\n")
  cat(mm, "\n")
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(cache = FALSE,
  collapse = TRUE,
  comment = "#>"
)
options()

## ----fig.show='hold'----------------------------------------------------------
library(nph)

## ----001, fig.show='hold', fig.width=12.5, warning=FALSE, out.width='100%'----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- 1              # There are no subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(18,nrow = 1)),
  lambdaMat2    = m2r(matrix(11,nrow = 1)),
  lambdaProgMat = m2r(matrix(9, nrow = 1)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1              # There are no subgroups
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(11,nrow = 1)),
  lambdaMat2    = m2r(matrix(9, nrow = 1)),
  lambdaProgMat = m2r(matrix(5, nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

pp = plot_diagram(B5, K5)
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----002,  fig.show='hold', fig.width=12.5, fig.height=10.5, warning=FALSE, out.width='100%'----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <-  c(.2, .8)
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(30,
                               18),nrow = 2)),
  lambdaMat2    = m2r(matrix(c(20,
                               11),nrow = 2)),
  lambdaProgMat = m2r(matrix(c(15, 
                               9), nrow = 2)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1 
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(11,nrow = 1)),
  lambdaMat2    = m2r(matrix(9, nrow = 1)),
  lambdaProgMat = m2r(matrix(5, nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

pp = plot_diagram(B5, K5, A_subgr_labels = c("S1", "S2"))
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----003,  fig.show='hold', fig.width=11, warning=FALSE, out.width='100%'-----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <-  c(.5, .5)
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(30,
                               18),nrow = 2)),
  lambdaMat2    = m2r(matrix(c(20,
                               11),nrow = 2)),
  lambdaProgMat = m2r(matrix(c(15, 
                               9), nrow = 2)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1 
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(11,nrow = 1)),
  lambdaMat2    = m2r(matrix(9, nrow = 1)),
  lambdaProgMat = m2r(matrix(5, nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----004,  fig.show='hold', fig.width=12.5, fig.height=10.5, warning=FALSE, out.width='100%'----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <-  c(.5, .5)
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(30,
                               18), nrow = 2)),
  lambdaMat2    = m2r(matrix(c(30,
                               11), nrow = 2)),
  lambdaProgMat = m2r(matrix(c(15, 
                                9), nrow = 2)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1 
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(11, nrow = 1)),
  lambdaMat2    = m2r(matrix( 9, nrow = 1)),
  lambdaProgMat = m2r(matrix( 5, nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)


pp = plot_diagram(B5, K5, A_subgr_labels = c("S1", "S2"))
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----005, fig.show='hold', fig.width=12.5, warning=FALSE, out.width='100%'----
times <- c(0, 100, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- 1               # There are no subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(11, 18), nrow = 1)),
  lambdaMat2    = m2r(matrix(c( 9, 11), nrow = 1)),
  lambdaProgMat = m2r(matrix(c( 5,  9), nrow = 1)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1 
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(11, 11), nrow = 1)),
  lambdaMat2    = m2r(matrix(c( 9,  9), nrow = 1)),
  lambdaProgMat = m2r(matrix(c( 5,  5), nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)


pp = plot_diagram(B5, K5)
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----006, fig.show='hold', fig.width=12.5, fig.height=10.5, warning=FALSE, out.width='100%'----
times <- c(0, 100, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- c(0.5, 0.5) #Proportion of subgroups
B5 <- pop_pchaz(
  T = times,
    lambdaMat1    = m2r(matrix(c(11, 30,
                                 11, 18), byrow = TRUE, nrow = 2)),
    lambdaMat2    = m2r(matrix(c( 9, 20,
                                  9, 11), byrow = TRUE, nrow = 2)),
    lambdaProgMat = m2r(matrix(c( 5, 15,
                                  5,  9), byrow = TRUE, nrow = 2)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1 
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(11, 11), nrow = 1, ncol = 2)),
  lambdaMat2    = m2r(matrix(c( 9,  9), nrow = 1, ncol = 2)),
  lambdaProgMat = m2r(matrix(c( 5,  5), nrow = 1, ncol = 2)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

pp = plot_diagram(B5, K5, A_subgr_labels = c("S1", "S2"))
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----007, fig.show='hold', fig.width=12.5, fig.height=10.5, warning=FALSE, out.width='100%'----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- 1               # There are no subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(18, nrow = 1)),
  lambdaMat2    = m2r(matrix(11, nrow = 1)),
  lambdaProgMat = m2r(matrix( 9, nrow = 1)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- c(1/3, 2/3) #non-switcher, switcher reponse
K5  <- pop_pchaz(
  T = times, 
  lambdaMat1    = m2r(matrix(c(11, 
                               11), nrow = 2, ncol = 1)),
  lambdaMat2    = m2r(matrix(c( 9, 
                               11), nrow = 2, ncol = 1)),
  lambdaProgMat = m2r(matrix(c( 5,
                                5), nrow = 2, ncol = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)


pp = plot_diagram(B5, K5, B_subgr_labels = c("Non-switcher","Switcher"))
                  
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----007b, fig.show='hold', fig.width=12.5, fig.height=10.5, warning=FALSE, out.width='100%'----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- 1               # There are no subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(18, nrow = 1)),
  lambdaMat2    = m2r(matrix(11, nrow = 1)),
  lambdaProgMat = m2r(matrix( 9, nrow = 1)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- c(1/3, 2/3) #non-switcher, switcher reponse
K5  <- pop_pchaz(
  T = times, 
  lambdaMat1    = m2r(matrix(c(11, 
                               11), nrow = 2, ncol = 1)),
  lambdaMat2    = m2r(matrix(c( 9, 
                               14), nrow = 2, ncol = 1)),
  lambdaProgMat = m2r(matrix(c( 5,
                                5), nrow = 2, ncol = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

pp = plot_diagram(B5, K5, B_subgr_labels = c("Non-switcher","Switcher"))
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----008,  fig.show='hold', fig.width=12.5, fig.height=10.5, warning=FALSE, out.width='100%'----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- 1               # There are no subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(18,nrow = 1)),
  lambdaMat2    = m2r(matrix(11,nrow = 1)),
  lambdaProgMat = m2r(matrix(9, nrow = 1)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- c(2/3, 1/3) #non-switcher, switcher reponse
K5  <- pop_pchaz(
  T = times, 
  lambdaMat1    = m2r(matrix(c(11, 
                               11), nrow = 2, ncol = 1)),
  lambdaMat2    = m2r(matrix(c(9, 
                               11), nrow = 2, ncol = 1)),
  lambdaProgMat = m2r(matrix(c(5,
                               5),  nrow = 2, ncol = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

pp = plot_diagram(B5, K5, B_subgr_labels = c("Non-switcher","Switcher"))
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----008b,  fig.show='hold', fig.width=12.5, fig.height=10.5, warning=FALSE, out.width='100%'----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- 1               # There are no subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(18, nrow = 1)),
  lambdaMat2    = m2r(matrix(11, nrow = 1)),
  lambdaProgMat = m2r(matrix( 9, nrow = 1)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- c(2/3, 1/3) #non-switcher, switcher reponse
K5  <- pop_pchaz(
  T = times, 
  lambdaMat1    = m2r(matrix(c(11, 
                               11), nrow = 2, ncol = 1)),
  lambdaMat2    = m2r(matrix(c( 9, 
                               14), nrow = 2, ncol = 1)),
  lambdaProgMat = m2r(matrix(c( 5,
                                5), nrow = 2, ncol = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

pp = plot_diagram(B5, K5, B_subgr_labels = c("Non-switcher","Switcher"))
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----009,  fig.show='hold', fig.width=12.5, fig.height=13, warning=FALSE, out.width='100%'----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- c(.5, .5)
B5 <- pop_pchaz(
  T = times,
    lambdaMat1    = m2r(matrix(c( 2,
                                 18), nrow = 2)),
    lambdaMat2    = m2r(matrix(c( 2,
                                 18), nrow = 2)),
    lambdaProgMat = m2r(matrix(c(19, 
                                 19), nrow = 2)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- c(1) 
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(11), nrow = 1, ncol = 1)),
  lambdaMat2    = m2r(matrix(c( 9), nrow = 1, ncol = 1)),
  lambdaProgMat = m2r(matrix(c( 5), nrow = 1, ncol = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

pp = plot_diagram(B5, K5,
                  A_subgr_labels = c("S1", "S2"))
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----010, fig.show='hold', fig.width=12.5, fig.height=13.5, warning=FALSE, out.width='100%'----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- c(.2, .8)
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(30,
                               18), nrow = 2)),
  lambdaMat2    = m2r(matrix(c(30,
                               11), nrow = 2)),
  lambdaProgMat = m2r(matrix(c(15, 
                                9), nrow = 2)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- c(2/3, 1/3*t_resp[1], 1/3*t_resp[2])
K5  <- pop_pchaz(
  T = times, 
  lambdaMat1    = m2r(matrix(c(11), nrow = 3, ncol = 1)),   
  lambdaMat2    = m2r(matrix(c( 9, 
                               25,
                               14), nrow = 3)), 
  lambdaProgMat = m2r(matrix(c( 5), nrow = 3,ncol = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

pp = plot_diagram(B5, K5,
                  A_subgr_labels = c("S1", "S2"),
                  B_subgr_labels = c("Non-switcher","Switcher-S1","Switcher-S2"))
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----010b, fig.show='hold', fig.width=12.5, fig.height=13.5, warning=FALSE, out.width='100%'----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- c(.5, .5)
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(30,
                               18), nrow = 2)),
  lambdaMat2    = m2r(matrix(c(30,
                               11), nrow = 2)),
  lambdaProgMat = m2r(matrix(c(15, 
                                9), nrow = 2)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- c(2/3, 1/3*t_resp[1], 1/3*t_resp[2])
K5  <- pop_pchaz(
  T = times, 
  lambdaMat1    = m2r(matrix(c(11), nrow = 3, ncol = 1)),   
  lambdaMat2    = m2r(matrix(c( 9, 
                               25,
                               14), nrow = 3)), 
  lambdaProgMat = m2r(matrix(c( 5), nrow = 3,ncol = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

pp = plot_diagram(B5, K5,
                  A_subgr_labels = c("S1", "S2"),
                  B_subgr_labels = c("Non-switcher","Switcher-S1","Switcher-S2"))
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

## ----010c, fig.show='hold', fig.width=12.5, fig.height=13.5, warning=FALSE, out.width='100%'----
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- c(.5, .5)
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(30,
                               18), nrow = 2)),
  lambdaMat2    = m2r(matrix(c(20,
                               11), nrow = 2)),
  lambdaProgMat = m2r(matrix(c(15, 
                                9), nrow = 2)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- c(1/3, 2/3*t_resp[1], 2/3*t_resp[2])
K5  <- pop_pchaz(
  T = times, 
  lambdaMat1    = m2r(matrix(c(11), nrow = 3, ncol = 1)),   
  lambdaMat2    = m2r(matrix(c( 9, 
                               25,
                               14), nrow = 3)), 
  lambdaProgMat = m2r(matrix(c( 5), nrow = 3,ncol = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)


pp = plot_diagram(B5, K5,
                  A_subgr_labels = c("S1", "S2"),
                  B_subgr_labels = c("Non-switcher","Switcher-S1","Switcher-S2"))
pp

## ----fig.show='hold', fig.height=3.5, fig.width=10, warning=FALSE, out.width='100%'----
plot_shhr(K5, B5)

##########################
# Simulate tab UI

tabPanel(
  title = "Power",
  id    = "powerTab",
  value = "powerTab",
  name  = "powerTab",
  class = "fade in",
  icon  = icon("line-chart"),
  h1("Calculate power", style = "color:#f8c954"),
  #shiny::wellPanel(
  #  downloadButton('p_downloadCode', "Download R code", class = "start-button"),
  #  h4("Download a reproducible R script for the code to generate the results of this tab")),
  wellPanel(
    fluidRow(
      column(6, sliderInput("p_R", "Number of simulations", min = 10, max = 1e4, value = 10, step = 50))),
    fluidRow(
      column(6, sliderInput("p_r0", "Allocation ratio", min = 0, max = 1, value = 0.5))
      ,column(6, numericInput("p_eventEnde", "Maximal number of events", min = 10, max = 1e6, value = 450))
     ), fluidRow(
      column(4, selectInput("p_lambdaRekr_unit", "Recruitment rate unit",
                             choices = c("day", "month", "year"), selected = "year"))
      ,column(4, numericInput("p_lambdaRekr", "Recruitment rate (Poisson assumption)", min = 0, max = 1e6, value = 300))
     ), fluidRow(
      column(4, selectInput("p_lambdaZens_unit", "Censoring rate unit", 
                              choices = c("day", "month", "year"), selected = "year"))
      ,column(4, numericInput("p_lambdaZens", "Censoring rate (Exponential assumption)", min = 0, max = 1e6, value = 0.013))
     ), fluidRow(
      column(4, selectInput("p_maxRekrKalenderZeit_unit", "Maximal duration of recruitment unit", 
                              choices = c("day", "month", "year"), selected = "year"))
      ,column(4, numericInput("p_maxRekrKalenderZeit", "Maximal duration of recruitment", min = 1, max = 1e6, value = 3))
     ), fluidRow(
      column(4, selectInput("p_maxKalender_unit", "Maximal study duration unit", 
                              choices = c("day", "month", "year"), selected = "year"))
      ,column(4, numericInput("p_maxKalender", "Maximal study duration", min = 1, max = 1e6, value = 4))
     ), fluidRow(
      column(6, numericInput("p_alpha",
                              "Significance level for hypothesis test of equal hazard functions in both groups", 
                              min = 0, max = 1, value = 0.05))
      ,column(6, numericInput("p_seed",
                              "Seed for simulations", 
                              min = 0, max = 1e5, value = floor(runif(1, min = 0, max = 1e5))))
    )
    )
  ,wellPanel(
    actionButton("p_calculate", "Calculate power", icon = icon("calculator")),
    conditionalPanel(
      condition = "input.p_calculate > 0",
      fluidRow(column(width = 12,
                      withSpinner(tableOutput("p_results")), 
                                  align="center")),
      fluidRow(column(width = 6,
                      h3("Create report"),
                      radioButtons('format', 'Document format', c('PDF', 'HTML', 'Word'),
                                   inline = TRUE),
                      downloadButton('downloadReport'))))
  )
)

# This helper function calculates the hazard rate per day of an exponential 
# distribution from the median given in months.
fu <- function(x){
  12 / 365.25 * log(2) / x
}
#A function to show survival, hazard and hazard ratio of two groups as function of time
panplot <- function(K,
                    B,
                    main = "",
                    savepdf = NA,
                    savepng = NA) {
  if (!is.na(savepdf))
    pdf(paste(savepdf, ".pdf", sep = ""),
        width = 10,
        height = 4)
  if (!is.na(savepng))
    jpeg(
      paste(savepng, ".png", sep = ""),
      width = 25,
      height = 8,
      unit = "cm",
      res = 300
    )
  CX <- 2
  CX2 <- 1
  dd <- 4
  par(
    lwd = dd,
    font = 2,
    font.axis = 2,
    font.lab = 2,
    cex = CX,
    cex.lab = CX,
    cex.axis = CX,
    cex.main = CX * 1.5
  )
  par(
    mfrow = c(1, 3),
    oma = c(0, 0, 2, 0),
    mar = c(5, 5, 3, 2)
  )
  plot(K,
       ylim = c(0, 1),
       ylab = "Survival",
       xlab = "Days")
  plot(B, add = TRUE, lty = 2, col = 3)
  
  ymax = max(B$haz, K$haz, na.rm = TRUE)
  cat(ymax)
  plot(K,
       "haz",
       ylim = c(0, ymax + 0.001),
       ylab = "Hazard",
       xlab = "Days")
  plot(B,
       "haz",
       add = TRUE,
       lty = 2,
       col = 3)
  
  plot(
    K$t,
    B$haz / K$haz,
    ylim = c(0, max(B$haz / K$haz, na.rm = TRUE) + 1),
    ylab = "Hazard ratio",
    xlab = "Days",
    type = "l"
  )
  mtext(
    outer = TRUE,
    side = 3,
    text = main,
    line = -2
  )#,cex=CX2,adj=0)
  if (!is.na(savepng))
    dev.off()
  if (!is.na(savepdf))
    dev.off()
}




#A function to show survival, hazard and hazard ratio of two groups as function of time
intplot <- function(dat, 
                    K,
                    B,
                    main = "",
                    savepdf = NA,
                    savepng = NA) {
  if (!is.na(savepdf))
    pdf(paste(savepdf, ".pdf", sep = ""),
        width = 10,
        height = 4)
  if (!is.na(savepng))
    jpeg(
      paste(savepng, ".png", sep = ""),
      width = 25,
      height = 8,
      unit = "cm",
      res = 300
    )
  CX <- 1
  CX2 <- 1
  dd <- 4
  par(
    lwd = dd,
    font = 2,
    font.axis = 2,
    font.lab = 2,
    cex = CX,
    cex.lab = CX,
    cex.axis = CX,
    cex.main = CX * 1.5
  )
  par(
    # mfrow = c(1, 3),
    oma = c(0, 0, 2, 0),
    mar = c(5, 5, 3, 2)
  )
  km.by.trt <- survival::survfit(survival::Surv(y, event) ~ group, data = dat)
  plot(km.by.trt, col = c(1,3),
       lwd = dd,
       ylab = "Survival",
       xlab = "Days")
  plot(K, add = TRUE, 
       lwd = dd/2,
       ylim = c(0, 1),
       ylab = "Survival",
       xlab = "Days")
  plot(B, 
       lwd = dd/2,
       add = TRUE, lty = 2, col = 3)
  
  mtext(
    outer = TRUE,
    side = 3,
    text = main,
    line = -2
  )#,cex=CX2,adj=0)
  if (!is.na(savepng))
    dev.off()
  if (!is.na(savepdf))
    dev.off()
}





#' Transform median time into rate
#' 
#' This helper function calculates the hazard rate per day of an exponential 
#' distribution from the median given in months.
#' 
#' @param x The median time in months to be transformed into rate
#' 
#' @export
m2r <- function(x){
  12 / 365.25 * log(2) / x
}

#' Draw a state space figure
#' 
#' A figure that shows the states and the possible transitions between them.
#' 
#' @param A An object of class \code{mixpch}, resembling the survival function in treatment group 0
#' @param B An object of class \code{mixpch}, resembling the survival function in treatment group 1
#' @param A_subgr_labels A character vector with the same length as A$p. It indicates names for the subgroups in A
#' @param B_subgr_labels A character vector with the same length as B$p. It indicates names for the subgroups in B
#' @param which Which treatment arm should be shown? One of "Both", "Experimental", "Control".
#' @param treatment_labels A character vector of length 2 indicating the treatment labels.
#' @param colors Either a vector of length two with colors for A and B, of "default".
#' @param show.rate A logical indicating whether the rate should be shown in the diagram
#' 
#' @export
plot_diagram = function(A,
                        B,
                        A_subgr_labels,
                        B_subgr_labels,
                        which = c("Both", "Experimental", "Control"),
                        treatment_labels = c("Experimental", "Control"),
                        colors = "default", show.rate = TRUE){
  which = match.arg(which)
  
  times = A$Tint
  t_pr = A$p
  t_label = A_subgr_labels
  t_lambdaMat1 = A$lambdaMat1
  t_lambdaMat2 = A$lambdaMat2
  t_lambdaProgMat = A$lambdaProgMat
  c_pr = B$p
  c_label = B_subgr_labels
  c_lambdaMat1 = B$lambdaMat1
  c_lambdaMat2 = B$lambdaMat2
  c_lambdaProgMat = B$lambdaProgMat
  
  if (length(times) > 3) stop("Only two intervals are supported")
  
  t_group = rep(t_label, each = 3)
  c_group = rep(c_label, each = 3)
  t_group_id = rep(1:length(t_pr), each = 3)
  c_group_id = rep(1:length(c_pr), each = 3)
  
  dat = data.frame(treatment = c(rep("Experimental", 3*length(t_pr)),
                                 rep("Control", 3*length(c_pr))),
                   treatment_labels = c(rep(treatment_labels[1], 3*length(t_pr)),
                                        rep(treatment_labels[2], 3*length(c_pr))),
                   group = c(t_group, c_group),
                   group_id = c(t_group_id, c_group_id))
  dat$x = c(-1, 0, 1) * 4
  dat$y = c( 1, 0, 1) * 2
  dat$x_begin = c(-1,        0 + 1/4,  1 - 1/4) * 4
  dat$y_begin = c( 1 - .5/2, 0,        1) * 2
  dat$x_end   = c( 0 -  1/4, 1      , -1 + 1/4) * 4
  dat$y_end   = c( 0,        1 -.5/2,  1) * 2
  dat$ends = c("last","last","first")
  dat$label = c("Alive\nNo progression","Progression","Death")
  dat$lambdas = c("lambda[P](t)",
                  "lambda[PD](t)",
                  "lambda[D](t)")
  dat$nudge_x = c(-.4,  0, 0)
  dat$nudge_y = c(-.4,-.4,.4)
  
  t_lambdas = data.frame()
  for (i in 1:length(t_pr)){
    t_lambdas = rbind(t_lambdas,
                      t_lambdaMat1[i,],
                      t_lambdaMat2[i,],
                      t_lambdaProgMat[i,])
  }
  c_lambdas = data.frame()
  for (i in 1:length(c_pr)){
    c_lambdas = rbind(c_lambdas,
                      c_lambdaMat1[i,],
                      c_lambdaMat2[i,],
                      c_lambdaProgMat[i,])
  }
  colnames(t_lambdas) = colnames(c_lambdas) = paste0("T", 1:(length(times)-1))
  dat = cbind(dat, rbind(t_lambdas, c_lambdas))
  if(show.rate){
    if (length(times) == 2){
      dat$lambda_label = paste0(dat$lambdas, ": ", round(dat$T1, 4))
    } else if (length(times) == 3){
      dat$lambdast1 = c(sprintf("lambda[P]^paste('[%s-%s]')*(t)",  0, times[2]),
                        sprintf("lambda[PD]^paste('[%s-%s]')*(t)", 0, times[2]),
                        sprintf("lambda[D]^paste('[%s-%s]')*(t)",  0, times[2]))
      dat$lambdast2 = c(sprintf("lambda[P]^paste('[%s-%s]')*(t)",  times[2], times[3]),
                        sprintf("lambda[PD]^paste('[%s-%s]')*(t)", times[2], times[3]),
                        sprintf("lambda[D]^paste('[%s-%s]')*(t)",  times[2], times[3]))
      dat$lambda_label = paste0("atop(",dat$lambdast1, ": ", round(dat$T1, 4), ",",
                                dat$lambdast2, ": ", round(dat$T2, 4), ")")
    } 
  } else{
    dat$lambda_label = dat$lambdas
  }
  
  if (which == "Experimental"){
    dat %>% 
      filter(treatment == "Experimental") -> dat
  } else if (which == "Control"){
    dat %>% 
      filter(treatment == "Control") -> dat
  }
  
  if(colors == "default"){
    col_trt = "pink" 
    col_ctrl = "lightblue" 
  }
  ggplot(dat) + 
    facet_grid(group ~ treatment_labels) + 
    geom_rect(aes(xmin = x - 1, 
                  xmax = x + 1,
                  ymin = y - 0.5, 
                  ymax = y + 0.5,
                  fill = treatment))  +
    geom_text(aes(x = x, 
                  y = y, label = label))  +
    geom_text(aes(x = (x_begin + x_end)/2 + nudge_x, 
                  y = (y_begin + y_end)/2 + nudge_y,
                  label = lambda_label), 
              parse = TRUE)  +
    geom_segment(aes(x    = x_begin, 
                     xend = x_end ,
                     y    = y_begin , 
                     yend = y_end),
                 arrow = arrow(type = "closed", 
                               length = unit(10, "points"),
                               ends = dat$ends)) +
    coord_equal(ylim = c(-0.5, 3.5), xlim = c(-5, 5)) + 
    theme_void() + 
    theme(legend.position = "none", 
          text = element_text(size = 20)) +
    scale_fill_manual(values = c("Experimental" = col_trt, 
                                 "Control" = col_ctrl)) -> p1
  
  p1
}


#' Plot of survival, hazard and hazard ratio of two groups as a function of time
#' 
#' A convenience function that uses the generic plot function in the nph package to
#' plot the three functions in a layout of 3 columns and 1 row.
#' 
#' @param A An object of class \code{mixpch}, resembling the survival function in treatment group 0
#' @param B An object of class \code{mixpch}, resembling the survival function in treatment group 1
#' @param main An overall title for the plot
#' @param xmax A maximum value for the x-axis. The plots is drawn using xlim = c(0, xmax)
#' 
#' @export
plot_shhr <- function(A, B, main = "", xmax = NULL) {
  if (is.null(xmax)) xmax = length(A$t)
  CX <- 2
  CX2 <- 1
  dd <- 4
  # Set graphical parameters. Save previous setup in old_par
  old_par = par(
    lwd = dd,
    font = 2,
    font.axis = 2,
    font.lab = 2,
    cex = CX,
    cex.lab = CX,
    cex.axis = CX,
    cex.main = CX * 1.5,
    mfrow = c(1, 3),
    oma = c(0, 0, 2, 0),
    mar = c(5, 5, 3, 2)
  )
  
  # Plot the survival functions
  plot(A,
       ylim = c(0, 1),
       xlim = c(0,xmax),
       ylab = "Survival",
       xlab = "Days")
  plot(B, add = TRUE, lty = 2, col = 3)
  
  # Plot the hazard functions
  ymax = max(B$haz, A$haz, na.rm = TRUE)
  plot(A,
       "haz",
       xlim = c(0,xmax),
       ylim = c(0, ymax + 0.001),
       ylab = "Hazard",
       xlab = "Days")
  plot(B,
       "haz",
       add = TRUE,
       lty = 2,
       col = 3)
  
  # Plot the hazard ratio
  plot(
    A$t,
    B$haz / A$haz,
    xlim = c(0,xmax),
    ylim = c(0, max(B$haz / A$haz, na.rm = TRUE) + 1),
    ylab = "Hazard ratio",
    xlab = "Days",
    type = "l"
  )
  
  # Add title
  mtext(
    outer = TRUE,
    side = 3,
    text = main, cex = 2,
    line = -2
  )
  par(old_par)
}

# Home tab UI ##################################################################

tabPanel(
  title = "Home",
  id    = "homeTab",
  value = "homeTab",
  name  = "homeTab",
  class = "home",
  icon  = icon("home"),
  div("nph: Tools for planning and analysing surivival studies under non-proportional hazards", 
      class = "cover-heading"),
  div("In randomized controlled
  studies, survival functions are typically compared using the log-rank test, which
  is most powerful under the assumption of proportional hazards. However, when the
  proportionality assumption is violated, hypothesis tests and visual inspection of estimated
  survival functions may be misleading or hard to interpret.",
      class = "cover-text"),
  div(class="start-button-div",
      actionButton("toSimulate", "START", 
                   class="start-button")),
  div(
  # div("A collaboration by:"),
  div(#img(src="merck.png", align = "left", width="30%"),
      a(href="https://cemsiis.meduniwien.ac.at", target="_blank",
        # img(src="muw.png",   align = "left", width="30%")),
        img(src="cemsiis_en.png",   align = "left", width="60%")),
      class = "cover-footing-img"),
      class = "cover-footing"),
  div(class = "cover-footing-end")
)

# Dataset tab UI ---------------------------------------------------------------

tabPanel(
  title = "Dataset",
  id    = "datasetTab",
  value = "datasetTab",
  name  = "datasetTab",
  class = "fade in",
  icon  = icon("file-excel-o"),

  h1("Analyze dataset using the tail test"),
  wellPanel(
    h2("1. Upload your data"),
    fluidRow(column(5,fileInput("outputfile",label="Select data file in csv")),
    # h5("Preview first 6 rows of the dataset"),
            fluidRow(column(6,tableOutput("head_table")))),
    fluidRow(column(12, "Note: The data should have an observation per row and two columns, one indicating the group (test or reference) and the other one indicating the response value\n")),
    fluidRow(column(3, downloadButton('downloadData', 'Download an example dataset')))),
  wellPanel(
    h2("2. Choose the corresponding variables"),
    fluidRow(column(12,
                           uiOutput("choose_Vars"),
                           uiOutput("choose_groups"))
    )),
  wellPanel(
    h2("3. Choose test options"),
    fluidRow(column(3, numericInput("q", label = "q", value = .1, min = 0, max = .5, step = 0.05)),
                    column(3, numericInput("alpha", label = "significance level (alpha)", value = .05, min = 0, max = 1, step = 0.005)),
                    column(3, numericInput("c", label = "equivalence margin (c)", value = NULL))),
    fluidRow(column(9, "Note: Leave blank the value of c to use Tsong's null hypothesis (please see the paper for details).")),
    fluidRow(column(12, actionButton("submit", "Calculate!")))
    ),
  wellPanel(
    conditionalPanel(
      condition = "input.submit > 0",
      h2("Results", style = "text-align:center;"),
      fluidRow(column(12,verbatimTextOutput("test_result"))),
      fluidRow(column(12,plotOutput("test_plot")))
    )
  )
)

## Quick overview

The package includes functions to model survival distributions in terms of piecewise constant hazards and to simulate data from the specified distributions. Also, a function to calculate weighted log-rank tests for the comparison of two hazard functions is included. Finally, a function to calculate a test using the maximum of a set of test statistics from weighted log-rank tests is provided. This test utilizes the asymptotic multivariate normal joint distribition of the separate test statistics. The correlation is estimated from the data.

Select the **<i class="fa fa-file-excel-o"></i> Dataset** tab to load your own data and perform an analysis.

Select the **<i class="fa fa-table"></i> Simulate** tab to create a dataset.

Select the **<i class="fa fa-line-chart"></i> Examples** tab to check some examples.

## More information

This tool is part of an R package. For more information, please contact <a target="_blank" style="font-weight: bold;color:#f8c954;" href="mailto:robin.ristl@meduniwien.ac.at;nicolas.ballarini@meduniwien.ac.at">contact Robin Ristl and Nicolas Ballarini</a>.

## Contact

For any comments or questions, please <a target="_blank" style="font-weight: bold;color:#f8c954;" href="mailto:robin.ristl@meduniwien.ac.at;nicolas.ballarini@meduniwien.ac.at">contact Robin Ristl and Nicolas Ballarini</a>.

---
title: "nph: report for conditional power - R code"
output: html_document
params:
  input: NA
  cp_pow: NA
  intplot: NA
  dat: NA
  K5: NA
  B5: NA
---

```{r eval=FALSE, include=FALSE}
knitr::opts_chunk(echo = TRUE, eval = FALSE)
```

```{r}
# Output results --------------------------------------------------------
intplot = params$intplot
dat = params$dat
K5  = params$K5
B5  = params$B5
cp_pow = params$cp_pow
```

```{r}
head(dat)
intplot(dat, K5, B5, main = "Interim data")

# Output results --------------------------------------------------------
p_results <- 
  data.frame(
    Test  = c("Maximum Logrank", rep("Weighted Logrank", 4)),
    rho   = c("0011", 0, 0, 1, 1),
    gamma = c("0101", 0, 1, 0, 1),
    power = cp_pow) 

p_results
```

```{r}
# Output results --------------------------------------------------------
knitr::kable(p_results)
```

---
title: "Introduction to nph and Usage Instructions"
author: "Robin Ristl, Nicolas Ballarini"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to nph and Usage Instructions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Overview 

The nph package includes functions to model survival distributions in terms of piecewise constant hazards and 
to simulate data from the specified distributions.

# Installation {-}

The package is available from CRAN and can be installed directly from R.

```{r eval=FALSE, fig.show='hold'}
install.packages("nph")

# For dev version
# install.packages("devtools")
devtools::install_github("repo/nph")
```

# Getting started

Basically, there are three mechanisms for non-proportionality available in this package:

  - Disease progression
  - Different effect by time intervals
  - Subgroups

These scenarios are illustrated in the following figures. Note that the hazard ratio is not constant across time. 

```{r echo=FALSE, fig.height=3.5, fig.show='hold', fig.width=10, warning=FALSE, out.width='100%'}
library(nph)
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- 1              # There are no subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(18, nrow = 1)),
  lambdaMat2    = m2r(matrix(11, nrow = 1)),
  lambdaProgMat = m2r(matrix( 9, nrow = 1)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1              # There are no subgroups
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(11, nrow = 1)),
  lambdaMat2    = m2r(matrix( 9, nrow = 1)),
  lambdaProgMat = m2r(matrix( 5, nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)
plot_shhr(A = K5, B = B5, main = "Different disease progression by treatment")
```

```{r echo=FALSE, fig.height=3.5, fig.show='hold', fig.width=10, warning=FALSE, out.width='100%'}
times <- c(0, 100, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- 1               # There are no subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(11, 18), nrow = 1)),
  lambdaMat2    = m2r(matrix(c( 9, 11), nrow = 1)),
  lambdaProgMat = m2r(matrix(c( 5,  9), nrow = 1)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1 
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(11, 11), nrow = 1)),
  lambdaMat2    = m2r(matrix(c( 9,  9), nrow = 1)),
  lambdaProgMat = m2r(matrix(c( 5,  5), nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

plot_shhr(K5, B5, main = "Different effect by time intervals")
```

```{r echo=FALSE, fig.height=3.5, fig.show='hold', fig.width=10, warning=FALSE, out.width='100%'}
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <-  c(.2, .8)
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(30,
                               18),nrow = 2)),
  lambdaMat2    = m2r(matrix(c(20,
                               11),nrow = 2)),
  lambdaProgMat = m2r(matrix(c(15, 
                               9), nrow = 2)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1 
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(11,nrow = 1)),
  lambdaMat2    = m2r(matrix(9, nrow = 1)),
  lambdaProgMat = m2r(matrix(5, nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)

plot_shhr(K5, B5, main = "Presence of subgroups with differential treatment effect")
```


# Basics

The functions of the package can be grouped according to their functionality.

Functions for modelling/setting the underlying survival model:
  
  - `hazVFfun`
  - `subpop_pchaz`
  - `pop_pchaz`
  
Functions for generating simulated dataset given for a specified survival model:

  - `sample_fun`
  - `sample_conditional_fun`
  
Functions for performing statistical tests:

  - `logrank.test`
  - `logrank.maxtest`

Plotting functions:

  - `plot.mixpch`
  - `plot_diagram`
  - `plot_shhr`


The basic underlying model for the survival mechanism assumes that each patient can be in one of three states: Alive with no progression of disease, Alive with progression of disease, and Dead.

```{r echo=FALSE, fig.height=4, fig.show='hold', fig.width=7, message=FALSE, warning=FALSE, out.width='75%', paged.print=TRUE}
times <- c(0, 5 * 365)   # Time interval boundaries, in days

# Treatment group
t_resp <- 1              # There are no subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(18, nrow = 1)),
  lambdaMat2    = m2r(matrix(11, nrow = 1)),
  lambdaProgMat = m2r(matrix( 9, nrow = 1)),
  p = t_resp,
  timezero = FALSE, discrete_approximation = TRUE
)

# Control group
c_resp <- 1              # There are no subgroups
K5  <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(11, nrow = 1)),
  lambdaMat2    = m2r(matrix( 9, nrow = 1)),
  lambdaProgMat = m2r(matrix( 5, nrow = 1)),
  p = c_resp,
  timezero = TRUE, discrete_approximation = TRUE
)
plot_diagram(K5, B5, which = "Control")
```

## Creating the population model with pop_pchaz

The first step is to create the population model with the `pop_pchaz` function.
As the previous figure shows, there are three hazard rates that need to be defined: the hazard of disease progression $\lambda_P(t)$, the hazard of death given no progression $\lambda_P(t)$, and the hazard of death given progression $\lambda_P(t)$. The arguments `lambdaProgMat`, `lambdaMat1`, and `lambdaMat2` in the `pop_pchaz` function correspond to the three hazard rates, respectively. 

The hazard rates are assumed piecewise constant functions across $k$ time intervals $[t_{j-1}, t_j)$, $j=1, \ldots,k$ with $0=t_0<t_1<\ldots<t_k=\infty$. Therefore, the `pop_pchaz` function has also an argument `T` that is a vector to specify $t_0,t_1,\ldots,t_k$. When `T` is of length 2 (and therefore only one time interval) `lambdaProgMat`, `lambdaMat1`, and `lambdaMat2` are scalars. If `T` is of length > 2, then the `lambdaProgMat`, `lambdaMat1`, and `lambdaMat2` are matrices where the number of columns is equal to the number of time intervals 
\[\begin{bmatrix}\lambda^{[t_0-t_1)} & \lambda^{[t_1-t_2)} & \ldots &\lambda^{[t_{k-1}-t_k)}\end{bmatrix}.\]

For example, if the patients are followed for two years but the hazards change after the first year, then `T` should be specified as `c(0, 365, 2*365)`. If we assume a hazard rate for death of 0.02 and 0.04 for the first and second year respectively, the we should specify `lambdaMat1 = matrix(c(0.02, 0.04), ncol = 2)`. 

Finally, is is also possible to specify different hazard rates for subgroups. The `pop_pchaz` has the argument `p` which is intended to specify the subgroup prevalences. Given $m$ subgroups with relative sizes $p_1, p_2, \ldots, p_m$, then the `p` argument should be specified as `c(p_1, p_2, ..., p_m)`. The `lambdaProgMat`, `lambdaMat1`, and `lambdaMat2` then should have the number of row equal to the number of defined subgroups:

\[
\begin{bmatrix}
\lambda_1 \\ 
\lambda_2 \\ 
\ldots \\
\lambda_m
\end{bmatrix}.
\]

For example, if patients can be divided into two subgroup with prevalences 0.2 and 0.8 with hazard rates a hazard rate for death of 0.02 and 0.03 thoughout a one year interval, then we define `T = c(0, 365)`, `p = c(0.2, 0.8)` and `lambdaMat1 = matrix(c(0.02, 0.03), nrow = 2)`. 

Naturally, it is possible to combine multiple time intervals and subgroups, then the hazard matrices have the form:
<!-- \[ -->
<!-- \begin{bmatrix} -->
<!-- \lambda_1^{[t_0-t_1)} & \lambda_1^{[t_1-t_2)} & \ldots &\lambda_1^{[t_{k-1}-t_k)} \\ -->
<!-- \lambda_2^{[t_0-t_1)} & \lambda_2^{[t_1-t_2)} & \ldots &\lambda_2^{[t_{k-1}-t_k)} \\ -->
<!-- \ldots \\ -->
<!-- \lambda_m^{[t_0-t_1)} & \lambda_m^{[t_1-t_2)} & \ldots &\lambda_m^{[t_{k-1}-t_k)} \\ -->
<!-- \end{bmatrix}. -->
<!-- \] -->

```{r echo=FALSE, fig.align='center', fig.show='hold', fig.width=12.5, warning=FALSE, out.width='50%'}
knitr::include_graphics("lambdamat.png")
```

Below, we consider an example where there two subgroups and two time intervals. In practice, this situation correspond to the case where there is a delayed effect of the drug. Note that for specifying the hazard matrices, we use the median time to death/progression and use the function `m2r` (also provided in the package) to obtain the hazard rates.

```{r, echo=TRUE, fig.show='hold', fig.width=12.5, warning=FALSE, out.width='100%'}
times <- c(0, 100, 5 * 365)   # Time interval boundaries, in days
t_resp <- c(0.2, 0.8) #Proportion of subgroups
B5 <- pop_pchaz(
  T = times,
  lambdaMat1    = m2r(matrix(c(11, 30,
                               11, 18), byrow = TRUE, nrow = 2)),
  lambdaMat2    = m2r(matrix(c( 9, 20,
                                9, 11), byrow = TRUE, nrow = 2)),
  lambdaProgMat = m2r(matrix(c( 5, 15,
                                5,  9), byrow = TRUE, nrow = 2)),
  p = t_resp, discrete_approximation = TRUE
)
```

The results object is of class `mixpch`, which has a dedicated plotting function to visualize the survival and hazard functions. 

```{r, echo=TRUE, fig.show='hold', fig.width=6, warning=FALSE, out.width='33%'}
plot(B5, main = "Survival function")
plot(B5, fun = "haz", main = "Hazard function")
plot(B5, fun = "cumhaz", main = "Cumulative Hazard function")
```

\newpage

## Creating a simulated dataset with sample_fun

The sample_fun function is designed to generate a simulated dataset that would be obtained from a parallel group randomised clinical trial. 

The first step is to create two objects with the (theoretical) survival functions for the treatment and control groups using `pop_pchaz`:
```{r}
times <- c(0, 100, 5 * 365)   # Time interval boundaries, in days
# Treatment group
B5 <- pop_pchaz(T = times,
                   lambdaMat1    = m2r(matrix(c(11, 30,
                                                11, 18), byrow = TRUE, nrow = 2)),
                   lambdaMat2    = m2r(matrix(c( 9, 20,
                                                 9, 11), byrow = TRUE, nrow = 2)),
                   lambdaProgMat = m2r(matrix(c( 5, 15,
                                                 5,  9), byrow = TRUE, nrow = 2)),
                   p = c(0.2, 0.8),#Proportion of subgroups
                discrete_approximation = TRUE 
)
# Control group
K5  <- pop_pchaz(T = times,
                    lambdaMat1    = m2r(matrix(c(11, 11), nrow = 1)),
                    lambdaMat2    = m2r(matrix(c( 9,  9), nrow = 1)),
                    lambdaProgMat = m2r(matrix(c( 5,  5), nrow = 1)),
                    p = 1, discrete_approximation = TRUE
)
```

Then, using the resulting objects, we use them to generate a dataset with the `sample_fun` function: 
```{r}
# Study set up and Simulation of a data set until interim analysis at 150 events
set.seed(15657)
dat <- sample_fun(K5, B5,
                  r0 = 0.5,                     # Allocation ratio
                  eventEnd = 450,               # maximal number of events
                  lambdaRecr = 300 / 365,       # recruitment rate per day (Poisson assumption)
                  lambdaCens = 0.013 / 365,     # censoring rate per day  (Exponential assumption)
                  maxRecrCalendarTime = 3 * 365,# Maximal duration of recruitment
                  maxCalendar = 4 * 365.25)     # Maximal study duration
head(dat)
tail(dat)
```



\newpage

## The weighted log-rank test and the max-LRtest

The weighted log-rank test is implemented using the function `logrank.test`, which uses the statistic:

\[
z = \sum_{t\in\mathcal{D}} w(t)(d_{t, ctr} - e_{t,ctr}) / \sqrt{\sum_{t\in\mathcal{D}} w(t)^2 var(d_{t, ctr})}.
\]

where $w(t)$ are the Fleming-Harrington $\rho-\gamma$ family weights, such that $w(t)=\widehat{S}(t)^{\rho}(1-\widehat{S}(t))^{\gamma}$.
Under the the least favorable configuration in $H_0$, the test statistic is asymptotically
standard normally distributed and large values of $z$ are in favor of the alternative. 

For example, the following code performs the weighted log-rank test using the simulated dataset and $\rho = 1$ and $\gamma = 0$.

```{r}
logrank.test(time  = dat$y,
             event = dat$event,
             group = dat$group,
             # alternative = "greater",
             rho   = 1,
             gamma = 0) 
# survival::survdiff(formula = survival::Surv(time  = dat$y, event = dat$event) ~ dat$group)
```

For a set of $k$ different weight functions $w_1(t), \ldots, w_k(t)$, the maximum log-rank test statistic is $z_{max} = \max_{i=1,\ldots,k}z_i$. Under the least favorable
configuration in $H_0$, approximately $(Z_1, \ldots, Z_k) \sim N_k(0, \Sigma)$.
The $p$-value of the maximum test, $P_{H_0}(Z_{max} > z_{max})$, is calculated based on this multivariate normal approximation via numeric integration.

The following code performs the maximum log-rank test using four combinations of $\rho$ and $\gamma$ for the weights.

```{r}
lrmt = logrank.maxtest(
      time  = dat$y,
      event = dat$event,
      group = dat$group,
      rho   = c(0, 0, 1, 1),
      gamma = c(0, 1, 0, 1)
)
lrmt
```

The individual tests can also be accesed using the `testListe` elements in the resulting object.

```{r}
lrmt$logrank.test[[1]]
lrmt$logrank.test[[2]]
lrmt$logrank.test[[3]]
lrmt$logrank.test[[4]]
```
